#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
# NCPL.pm
# ---------------------------------------------------------------------------------
#
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

package NCPL;

use warnings;
use vars  qw($VERSION @ISA @EXPORT @EXPORT_OK);
use POSIX qw(floor);

use constant TRUE      => 1;
use constant FALSE     => 0;
use constant UNDEFINED => undef;

require Exporter;

# ---------------------------------------------------------------------------------

our @ISA = qw(Exporter);
our @EXPORT = qw($APPLICATION_NAME $APPLICATION_SHORTNAME $APPLICATION_VERSION $APPLICATION_AUTHOR_NAME $APPLICATION_AUTHOR_EMAIL $PATH_PERL $PATH_DIFF $PATH_BACKUP $PATH_EXPORT $PATH_IMPORT $PATH_INSTALLED $NAGIOS_VERSION $NAGIOS_PATH_INITSCRIPT $NAGIOS_PATH_DAEMON $NAGIOS_PATH_CONFIGDIR $NAGIOS_PATH_NAMEDPIPE $NAGIOS_PATH_PIDFILE $NAGIOS_USER $NAGIOS_GROUP init_module);
our @EXPORT_OK = qw(html_start html_pagetitle html_menu html_description html_underconstruction html_dbcombobox html_listbox html_enabledbox html_textbox html_textarea html_passwordbox html_stop html_debug html_w3c html_urlencode html_textencode html_emptyline html_goBackTo
										$HTML_COMBOBOX_MAXSIZE $HTML_LISTBOX_MAXSIZE
										$DB_SERVER $DB_PORT $DB_CATALOG $DB_USERNAME $DB_PASSWORD doQueryDBI ConfirmDelete ListAllRecords $TABLE_COMMANDS $TABLE_CONTACTS $TABLE_CONTACTGROUPS $TABLE_HOSTS $TABLE_HOSTGROUPS $TABLE_HOSTGROUPESCALATIONS $TABLE_HOSTDEPENDENCIES $TABLE_HOSTESCALATIONS $TABLE_HOSTEXTINFO $TABLE_MISCCOMMANDS $TABLE_SERVICES $TABLE_SERVICEGROUPS $TABLE_SERVICEDEPENDENCIES $TABLE_SERVICEESCALATIONS $TABLE_SERVICEEXTINFO $TABLE_TIMEPERIODS $TABLE_CGI $TABLE_MAIN $TABLE_RESOURCES $TABLE_HTPASSWD $TABLE_USERS $TABLE_USERLEVELS $TABLE_SERVERS $TABLE_CATEGORIES $TABLE_FILETYPES $TABLE_FILENAMES $TABLE_ENVIRONMENTS
										$NAGIOSBYDB_DBSERVER $NAGIOSBYDB_PORT $NAGIOSBYDB_CATALOG $NAGIOSBYDB_USERNAME $NAGIOSBYDB_PASSWORD $NAGIOSBYDB_CATEGORY $NAGIOSBYDB_SERVERNAME
										$MYSQLDUMP_PATH $MYSQLDUMP_DUMPFILE Export_Database
										$SQL_CHECKCOMMANDS $SQL_CONTACTS $SQL_CONTACTGROUPS $SQL_HOSTS $SQL_HOSTGROUPS $SQL_MISCCOMMANDS $SQL_SERVICEGROUPS $SQL_SERVICEGROUPMEMBERS $SQL_TEMPLATES_HOSTS $SQL_TEMPLATES_HOSTEXTINFO $SQL_TEMPLATES_SERVICES $SQL_SERVERS $SQL_TIMEPERIODS $SQL_USERLEVELS $SQL_FILETYPES $SQL_FILENAMES $SQL_ENVIRONMENTS
										$SMTP_SERVER $SMTP_PORT $SMTP_HEADER $SMTP_MIME $SMTP_RETRIES $SMTP_DELAY $SMTP_FROM $SMTP_TO $SMTP_CC $SMTP_BCC $SMTP_SUBJECT $SMTP_ENABLED sendEmail
										DisplayExportForm initExport %FILENAMES
										DisplayImportForm initImport
										ArrayToCommaSeparatedList CommaSeparatedListToArray DelimitedListToHash DelimitedListToUrl execCMD getFilenameByFiletype getFiletypeByAlias getActiveItems getInactiveItems
										UserAccessControl DisplayLogonForm);
										
our %EXPORT_TAGS = (HTML 		   => [qw(html_start html_pagetitle html_menu html_description html_underconstruction html_dbcombobox html_listbox html_enabledbox html_textbox html_textarea html_passwordbox html_stop html_debug html_w3c $HTML_COMBOBOX_MAXSIZE $HTML_LISTBOX_MAXSIZE html_urlencode html_textencode html_emptyline html_goBackTo)],
										DB			   => [qw($DB_SERVER $DB_PORT $DB_CATALOG $DB_USERNAME $DB_PASSWORD doQueryDBI ConfirmDelete ListAllRecords $TABLE_COMMANDS $TABLE_CONTACTS $TABLE_CONTACTGROUPS $TABLE_HOSTS $TABLE_HOSTGROUPS $TABLE_HOSTGROUPESCALATIONS $TABLE_HOSTDEPENDENCIES $TABLE_HOSTESCALATIONS $TABLE_HOSTEXTINFO $TABLE_MISCCOMMANDS $TABLE_SERVICES $TABLE_SERVICEGROUPS $TABLE_SERVICEDEPENDENCIES $TABLE_SERVICEESCALATIONS $TABLE_SERVICEEXTINFO $TABLE_TIMEPERIODS $TABLE_CGI $TABLE_MAIN $TABLE_RESOURCES $TABLE_HTPASSWD $TABLE_USERS $TABLE_USERLEVELS $TABLE_SERVERS $TABLE_CATEGORIES $TABLE_FILETYPES $TABLE_FILENAMES $TABLE_ENVIRONMENTS)],
										EXPORT		 => [qw(DisplayExportForm initExport %FILENAMES Export_Database)],
										IMPORT		 => [qw(DisplayImportForm initImport)],
										MYSQLDUMP	 => [qw(Export_Database)],
										FUNCTIONS	 => [qw(ArrayToCommaSeparatedList CommaSeparatedListToArray DelimitedListToHash DelimitedListToUrl execCMD getFilenameByFiletype getFiletypeByAlias getActiveItems getInactiveItems)],
										SECURITY   => [qw(DisplayLogonForm UserAccessControl)],
										SMTP       => [qw($SMTP_SERVER $SMTP_PORT $SMTP_HEADER $SMTP_MIME $SMTP_RETRIES $SMTP_DELAY $SMTP_FROM $SMTP_TO $SMTP_CC $SMTP_BCC $SMTP_SUBJECT $SMTP_ENABLED sendEmail)],
										SQL        => [qw($SQL_CHECKCOMMANDS $SQL_CONTACTS $SQL_CONTACTGROUPS $SQL_HOSTS $SQL_HOSTGROUPS $SQL_MISCCOMMANDS $SQL_SERVICEGROUPS $SQL_SERVICEGROUPMEMBERS $SQL_TEMPLATES_HOSTS $SQL_TEMPLATES_HOSTEXTINFO $SQL_TEMPLATES_SERVICES $SQL_SERVERS $SQL_TIMEPERIODS $SQL_USERLEVELS $SQL_FILETYPES $SQL_FILENAMES $SQL_ENVIRONMENTS)],
                    NAGIOSBYDB => [qw($NAGIOSBYDB_DBSERVER $NAGIOSBYDB_PORT $NAGIOSBYDB_CATALOG $NAGIOSBYDB_USERNAME $NAGIOSBYDB_PASSWORD $NAGIOSBYDB_CATEGORY $NAGIOSBYDB_SERVERNAME)]);

# ---------------------------------------------------------------------------------

our $VERSION = 1.000.043;

# ---------------------------------------------------------------------------------

# [APPLICATION]
our $APPLICATION_NAME           = "Nagios Control Panel (NCPL)";
our $APPLICATION_SHORTNAME      = "NCPL";
our $APPLICATION_VERSION        = "1.000.043";
our $APPLICATION_AUTHOR_NAME    = "Yves Van den Hove";
our $APPLICATION_AUTHOR_EMAIL   = "yvdhove\@users.sourceforge.net";

# ---------------------------------------------------------------------------------

# [SYSTEM]
our $PATH_PERL                  = "/usr/bin/perl";
our $PATH_DIFF									= "/usr/local/bin/diff";

# [NAVIGATION]
our $NAVIGATION_RECORDSONPAGE   = 25;
our $NAVIGATION_MAXPAGES        = 50;

# [DBCONFIG]
our $DB_SERVER                  = "localhost";
our $DB_PORT                    = "3306";
our $DB_CATALOG                 = "ncpl";
our $DB_USERNAME                = "ncpl";
our $DB_PASSWORD                = "<put your pwd here>";

# [NAGIOSBYDB]
our $NAGIOSBYDB_DBSERVER        = "localhost";
our $NAGIOSBYDB_PORT            = "3306";
our $NAGIOSBYDB_CATALOG         = "nagios";
our $NAGIOSBYDB_USERNAME        = "nagios";
our $NAGIOSBYDB_PASSWORD        = "<put your pwd here>";
our $NAGIOSBYDB_SERVERNAME      = "nagios";
our $NAGIOSBYDB_CATEGORY        = "NAGIOSBYDB";

# [TABLES]
our $TABLE_COMMANDS             = "nagios_commands";
our $TABLE_CONTACTS             = "nagios_contacts";
our $TABLE_CONTACTGROUPS        = "nagios_contactgroups";
our $TABLE_HOSTS                = "nagios_hosts";
our $TABLE_HOSTDEPENDENCIES     = "nagios_hostdependencies";
our $TABLE_HOSTESCALATIONS      = "nagios_hostescalations";
our $TABLE_HOSTEXTINFO          = "nagios_hostextinfo";
our $TABLE_HOSTGROUPS           = "nagios_hostgroups";
our $TABLE_HOSTGROUPESCALATIONS = "nagios_hostgroupescalations";
our $TABLE_MISCCOMMANDS         = "nagios_misccommands";
our $TABLE_SERVICES             = "nagios_services";
our $TABLE_SERVICEGROUPS        = "nagios_servicegroups";
our $TABLE_SERVICEDEPENDENCIES  = "nagios_servicedependencies";
our $TABLE_SERVICEESCALATIONS   = "nagios_serviceescalations";
our $TABLE_SERVICEEXTINFO       = "nagios_serviceextinfo";
our $TABLE_TIMEPERIODS          = "nagios_timeperiods";
our $TABLE_CGI                  = "nagios_cgi";
our $TABLE_MAIN                 = "nagios_main";
our $TABLE_RESOURCES            = "nagios_resources";
our $TABLE_HTPASSWD             = "nagios_htpasswd";
our $TABLE_USERS                = "ncpl_users";
our $TABLE_USERLEVELS           = "ncpl_userlevels";
our $TABLE_SERVERS              = "ncpl_servers";
our $TABLE_CATEGORIES						= "ncpl_categories";
our $TABLE_FILETYPES						= "ncpl_filetypes";
our $TABLE_FILENAMES						= "ncpl_filenames";
our $TABLE_ENVIRONMENTS					= "ncpl_environments";

# [FILENAMES]
our %FILENAMES = (1  => 'checkcommands.cfg', 
                  2  => 'contacts.cfg', 
                  3  => 'contactgroups.cfg', 
                  6  => 'hosts.cfg', 
                  7  => 'hostdependencies.cfg', 
                  8  => 'hostgroupescalations.cfg', 
                  9  => 'hostextinfo.cfg', 
                  10 => 'hostgroups.cfg', 
                  11 => 'services.cfg', 
                  12 => 'servicedependencies.cfg', 
                  13 => 'serviceescalations.cfg', 
                  14 => 'serviceextinfo.cfg', 
                  15 => 'servicegroups.cfg', 
                  16 => 'timeperiods.cfg', 
                  17 => 'misccommands.cfg',
                  18 => 'resource.cfg');

# [HTML]
our $HTML_COMBOBOX_MAXSIZE      = 1;
our $HTML_LISTBOX_MAXSIZE       = 25;

# [DEFAULT DIRECTORIES]
our $PATH_BACKUP                = "/var/www/ncpl/tmp/backup";
our $PATH_EXPORT                = "/var/www/ncpl/tmp/export";
our $PATH_IMPORT                = "/var/www/ncpl/tmp/import";
our $PATH_INSTALLED             = "/var/www/ncpl/tmp/installed";

# [NAGIOS]
our $NAGIOS_VERSION			        = "1.3";
our $NAGIOS_PATH_INITSCRIPT     = "/etc/init.d/nagios";
our $NAGIOS_PATH_DAEMON         = "/usr/sbin/nagios";
our $NAGIOS_PATH_CONFIGDIR      = "/etc/nagios";
our $NAGIOS_PATH_NAMEDPIPE      = "/var/run/nagios/nagios.cmd";
our $NAGIOS_PATH_PIDFILE        = "/var/run/nagios/nagios.pid";
our $NAGIOS_USER						    = "nagios";
our $NAGIOS_GROUP               = "nagios";

# [MYSQLDUMP]
our $MYSQLDUMP_PROG 	          = "/usr/bin/mysqldump";
our $MYSQLDUMP_DUMPFILE         = "$PATH_BACKUP/$DB_CATALOG";

# [SESSIONS]
our $SESSIONS_EXPIRE  	        = "+15m";

# [SMTP]
our $SMTP_SERVER                = "smtp.localdomain";
our $SMTP_PORT                  = 25;
our $SMTP_HEADER                = "$APPLICATION_SHORTNAME v$APPLICATION_VERSION";
our $SMTP_MIME                  = 1;
our $SMTP_RETRIES               = 3;
our $SMTP_DELAY                 = 5;
our $SMTP_FROM                  = "root\@localdomain";
our $SMTP_TO                    = "yvdhove\@users.sourceforge.net";
our $SMTP_CC                    = "";
our $SMTP_BCC                   = "";
our $SMTP_SUBJECT               = "$APPLICATION_SHORTNAME v$APPLICATION_VERSION - Nagios Config Reloaded";

# A user defined config file overrides the default parameters!
if (-s 'NCPL.conf') { require 'NCPL.conf'; } 

# ---------------------------------------------------------------------------------

require "NCPL_DATABASE.pm";
require "NCPL_EXPORT.pm";
require "NCPL_HTML.pm";
require "NCPL_IMPORT.pm";
require "NCPL_SMTP.pm";
require "NCPL_SESSIONS.pm";
require "NCPL_SQL.pm";

# ---------------------------------------------------------------------------------

sub ArrayToCommaSeparatedList {
	my ($array) = @_;
	my $CommaSeparatedList = "";
	
	if($array) {
		for(my $i=0;$i<@$array;$i++) {
			$CommaSeparatedList .= $array->[$i] . ",";	
		}
	}	
	chop $CommaSeparatedList;
	return $CommaSeparatedList;
}

# ---------------------------------------------------------------------------------

sub CommaSeparatedListToArray {
	
  my ($CommaSeparatedList, $aantal) = @_;
	my @items  = split (/,/, $CommaSeparatedList);
  my $number = 0; my $value = '';	my @array;	
	
	foreach my $item (@items) {
	  $value .= "$item,";
	  $number++;
	
	  if ($number == $aantal) {
	    chop $value;
	    push( @array, "$value");
	    $value = ''; $number = 0;
	  }
	}
	
	return @array;
	
}

# ---------------------------------------------------------------------------------

sub DelimitedListToHash {
	
	my ($list, $debug) = @_;
	
	use Tie::IxHash;
	tie(%myHASH, Tie::IxHash);
	%myHASH = map { my ($key, $value) = split (/=/) } split (/\|/, $list);
	
	print "<p>Delimited List: " . $list . "</p>\n" if ($debug);
	print "<p>Size of myHASH: " . keys( %$myHASH ) . "</p>\n"  if ($debug);
	
	return \%myHASH;
	
}

# ---------------------------------------------------------------------------------

sub DelimitedListToUrl {
	my($DelimitedList, $debug) = @_;
	my $tHASH = DelimitedListToHash($DelimitedList,$debug);	
	my $tURL = "";
	foreach my $k(sort keys %$tHASH) { 
		$tURL .= $k . "=" . $tHASH->{$k} . "&amp;";
	}	
	$tURL =~ s/&amp;$//;
	return html_urlencode($tURL);
}

# ---------------------------------------------------------------------------------

sub execCMD {
	my ($cmd, $debug) = @_;
	open(CMD, "$cmd 2>&1 |") or die "Can't run '$cmd'\n$!\n";
	print "<p>$cmd</p>\n" if($debug);
	my $output = '';
	while(<CMD>){
		$output .= $_;
	}
	return $output;
}

# ---------------------------------------------------------------------------------

sub getFilenameByFiletype {
	
	my ($filetype, $dbh, $debug) = @_;
	
	my $filename = '';
	
	my $sql = "select filetype_filename from $TABLE_FILENAMES where filetype_id=\"$filetype\" and enabled=1";
	
	print "<p>getFilenameByFiletype: $sql</p>" if ($debug);
	
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref()) { 
		$filename = $ref->{"filename"} if (defined $ref->{"filename"}); 
	} 
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
	return $filename;
	
}

# ---------------------------------------------------------------------------------

sub getFiletypeByAlias {
	
	my ($filetype_alias, $dbh, $debug) = @_;
	
	my $filetype_id = '';
	
	my $sql = "select filetype_id from $TABLE_FILETYPES where filetype_alias=\"$filetype_alias\" and enabled=1";
	
	print "<p>getFiletypeByAlias: $sql</p>" if ($debug);
	
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref()) { 
		$filetype_id = $ref->{"filetype_id"} if (defined $ref->{"filetype_id"}); 
	} 
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
	return $filetype_id;
	
}

# ---------------------------------------------------------------------------------

sub init_module {
	
	my ($cgi, $sortkeys, $ukeys_current, $filetype) = @_;
	
	my $action  	  = (defined $cgi->param('action'))  	  ?  $cgi->param('action')     : '';
	my $submit      = (defined $cgi->param('submitForm')) ?  $cgi->param('submitForm') : 0;
	my $confirmed   = (defined $cgi->param('confirmed'))  ?  $cgi->param('confirmed')  : 0;
	my $sortfield   = (defined $cgi->param('sortfield'))  ?  $cgi->param('sortfield')  : $sortkeys;
	my $sortorder   = (defined $cgi->param('sortorder'))  ?  $cgi->param('sortorder')  : 'asc';
	my ($filterkey) = (defined $cgi->param('sortfield'))  ?  split(/,/, $sortfield)    : split(/,/, $sortkeys);
	my $filter     	= (defined $cgi->param('filter'))  		?  $cgi->param('filter')     : '';
	my $page			  = (defined $cgi->param('page')) 		  ?  $cgi->param('page') 		   : 1;
	my $debug       = (defined $cgi->param('debug'))  		?  $cgi->param('debug')  		 : 0;

	return ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug);
	
}

# ---------------------------------------------------------------------------------

sub getActiveItems {
	
	my ($inputstr, $aantal, $items_inactive) = @_;
	
	# Initialisation of variables
	my @items_all = CommaSeparatedListToArray($inputstr, $aantal);
		
	# Generating outputstr
	my $outputstr = ""; for my $t (@items_all) { $outputstr .= $t . ',' if(! exists $items_inactive->{$t}); } chop($outputstr);

	return $outputstr;
	
}

# ---------------------------------------------------------------------------------

sub getInactiveItems {
	
	my($dbh, $sql_exclude) = @_;
	
	my %items_inactive = ();
	
	# Putting the inactive items in the hash
	my ($sth,$rows) = doQueryDBI($dbh, $sql_exclude, '', '', $debug);
	my $names = $sth->{'NAME'};
	my $numFields = $sth->{'NUM_OF_FIELDS'};
	while (my $ref = $sth->fetchrow_hashref()) { 
		my $key = ""; for (my $i = 0;  $i < $numFields;  $i++) { $key .= $ref->{$$names[$i]} . ','; } chop($key);
		$items_inactive{$key} = 0;
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql_exclude, $dbh->errstr, $debug);
	
	return %items_inactive;
	
}

# ---------------------------------------------------------------------------------

1;