# ---------------------------------------------------------------------------------
# NCPL_DATABASE.pm
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

sub doQueryDBI 
{
	my ($dbh, $sql, $msgSucces, $msgFail, $debug) = @_;
	
	my ($sth, $rows);
	
	$sth = $dbh->prepare($sql) or doErrorTrapDBI('$dbh->prepare()', $sql, $dbh->errstr, $debug);
	$sth->execute() or doErrorTrapDBI('sth->execute()', $sql, $dbh->errstr, $debug);
	$rows = $sth->rows();
	if ( (defined $msgSucces && $msgSucces ne '') or (defined $msgFail && $msgFail ne '') ) {
		print "<p>" . ( (! $dbh->errstr) ? $msgSucces : $msgFail ) . "</p>\n";
	}
	print "<p>Query: " . $sql . "</p>\n" if $debug;
	print "<p>Rows returned by query: " . $rows . "</p>\n" if $debug;
	return ($sth,$rows);
}

# ---------------------------------------------------------------------------------

sub doErrorTrapDBI {
	my ($cmd, $sql, $errstr, $debug) = @_;
	
	print "<table>\n";
	print "<tr><th colspan=\"2\">--- BEGIN doErrorTrapDBI ---</th></tr>\n";
	print "<tr><td width=\"120\">DBI CMD:</td><td>$cmd</td></tr>\n" if(defined $cmd && $cmd ne ''); 
	print "<tr><td width=\"120\">DBI QUERY:</td><td>$sql</td></tr>\n" if(defined $sql && $sql ne '');
	print "<tr><td width=\"120\">DBI ERROR:</td><td>$errstr</td></tr>\n" if(defined $errstr && $errstr ne '');
	print "<tr><th colspan=\"2\">--- END doErrorTrapDBI ---</th></tr>\n";
	print "</table>\n";
	print "<br>\n";
	
	#print "DBI CMD: $cmd<br>\n" if(defined $cmd && $cmd ne '');
	#print "DBI QUERY: $sql<br>\n" if(defined $sql && $sql ne '');
	#print "DBI ERROR: $errstr<br>\n" if(defined $errstr && $errstr ne '');

	return 0;
}

# ---------------------------------------------------------------------------------

sub ListAllRecords {

my ($prgscriptname, $dbh, $sql, $filter, $ukeys, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug) = @_;

#
# Generating filter clause
#
if($filter ne '') {
	$sql .= ($sql =~ /where/i) ? " and " : " where ";
	$sql .= " `$filterkey` like '$filter%'";
}

#
# Generating order by clause
#
$sql .= " order by `" . join('`,`', split(/,/, $sortfield)) . "` $sortorder";
my @keys = CommaSeparatedListToArray($sortkeys,1);
foreach $k(@keys) 
{ 
	$sql .= ",$k asc" if ( ! $sortfield =~ /$k/i );
}

#
# Connecting to the database
#
my $offset = ($page * $NAVIGATION_RECORDSONPAGE) - $NAVIGATION_RECORDSONPAGE;
my $sql_limit = $sql . " LIMIT $offset,$NAVIGATION_RECORDSONPAGE"; 
my ($sth1, $rows1) = doQueryDBI($dbh, $sql_limit, '', '', $debug);

#	
# Printing table headers
#
print <<END
<br>
<table>
  <tr>
END
;
my $names = $sth1->{'NAME'};
my $numFields = $sth1->{'NUM_OF_FIELDS'};
for (my $i = 0;  $i < $numFields;  $i++) {
  if ($sortfield =~ /$$names[$i]/i) {
  	print "    <th><a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$$names[$i]&amp;sortorder=asc\"><img border=\"0\" alt=\"sort ascending\" title=\"sort ascending\" src=\"" . ( $sortorder =~ /asc/i ? "../images/icon_up_red.gif" : "../images/icon_up.gif") . "\" /></a>&nbsp;<font color=\"#FF0000\">" . $$names[$i] . "</font>&nbsp;<a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$$names[$i]&amp;sortorder=desc\"><img border=\"0\" alt=\"sort descending\" title=\"sort descending\" src=\"" . ( $sortorder =~ /desc/i ? "../images/icon_down_red.gif" : "../images/icon_down.gif") . "\" /></a></th>\n";
  }
  else {
   	print "    <th><a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$$names[$i]&amp;sortorder=asc\"><img border=\"0\" alt=\"sort ascending\" title=\"sort ascending\" src=\"../images/icon_up.gif\" /></a>&nbsp;" . $$names[$i] . "&nbsp;<a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$$names[$i]&amp;sortorder=desc\"><img border=\"0\" alt=\"sort descending\" title=\"sort descending\" src=\"../images/icon_down.gif\" /></a></th>\n";
  }
}
print <<END
    <th>actions</th>
  </tr>
END
;

#
# Printing records
#
if($rows1 > 0)
{
	my $pkeys = DelimitedListToHash($ukeys,$debug);
	while (my $ref = $sth1->fetchrow_arrayref) {
		print "  <tr>\n";
		my $keystring = "";
		for (my $i = 0;  $i < $numFields;  $i++) {	
			if(defined $$ref[$i] && $$ref[$i] ne '') { print "    <td class=\"recordset\">" . html_textencode($$ref[$i],$debug) . "</td>\n"; }
			else { print "    <td class=\"recordset\">&nbsp;</td>\n"; }
			#if (exists $pkeys->{$$names[$i]}) { $keystring .= $$names[$i] . "=" . $$ref[$i] . "&amp;"; }
			if (exists $pkeys->{$$names[$i]}) { $keystring .= $$names[$i] . "=" . html_urlencode($$ref[$i], $debug) . "&amp;"; }
		}
		$keystring =~ s/&amp;$//;
		#print "    <td class=\"recordset\"><a href=\"$prgscriptname?action=detail&amp;" . html_urlencode($keystring, $debug) . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_detail.gif\" border=\"0\" alt=\"Detail\" title=\"Detail\" /></a> <a href=\"$prgscriptname?action=edit&amp;" . html_urlencode($keystring, $debug) . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_edit.gif\" border=\"0\" alt=\"Edit\" title=\"Edit\" /></a> <a href=\"$prgscriptname?action=duplicate&amp;" . html_urlencode($keystring, $debug) . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_duplicate.gif\" border=\"0\" alt=\"Duplicate\" title=\"Duplicate\" /></a> <a href=\"$prgscriptname?action=delete&amp;" . html_urlencode($keystring, $debug) . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_delete.gif\" border=\"0\" alt=\"Delete\" title=\"Delete\" /></a></td>\n";
		print "    <td class=\"recordset\"><a href=\"$prgscriptname?action=detail&amp;" . $keystring . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_detail.gif\" border=\"0\" alt=\"Detail\" title=\"Detail\" /></a> <a href=\"$prgscriptname?action=edit&amp;" . $keystring . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_edit.gif\" border=\"0\" alt=\"Edit\" title=\"Edit\" /></a> <a href=\"$prgscriptname?action=duplicate&amp;" . $keystring . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_duplicate.gif\" border=\"0\" alt=\"Duplicate\" title=\"Duplicate\" /></a> <a href=\"$prgscriptname?action=delete&amp;" . $keystring . "&amp;debug=$debug&amp;CGISESSID=$CGISESSID\"><img src=\"../images/icon_delete.gif\" border=\"0\" alt=\"Delete\" title=\"Delete\" /></a></td>\n";
		print "  </tr>\n";
	}
}
else 
{
	print "  <tr>\n";
	print "    <td class=\"recordset\" colspan=\"" . ($numFields + 1) . "\">There are currently no records in the database!</td>\n";
	print "  </tr>\n";
}
my $rc1 = $sth1->finish() or doErrorTrapDBI('$sth1->finish()', $sql_limit, $dbh->errstr, $debug);

#
# Printing navigation bar
#
my ($sth2, $rows2)  = doQueryDBI($dbh, $sql, '', '', $debug);
my $numberOfRecords = $rows2;
my $numberOfPages   = floor($numberOfRecords / $NAVIGATION_RECORDSONPAGE) + (($numberOfRecords % $NAVIGATION_RECORDSONPAGE) == 0 ? 0 : 1);
my $nextPage  = $page + 1;
my $prevPage  = $page - 1;
my $iconFirst = $page > 1 ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=1&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><img src=\"../images/icon_first.gif\" alt=\"First page\" title=\"First page\" border=\"0\" /></a>" : "";
my $iconPrev  = $page > 1 ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$prevPage&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><img src=\"../images/icon_prev.gif\" alt=\"Previous page\" title=\"Previous page\" border=\"0\" /></a>" : "";
my $iconNext  = $page < $numberOfPages ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$nextPage&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><img src=\"../images/icon_next.gif\" alt=\"Next page\" title=\"Next page\" border=\"0\" /></a>" : "";
my $iconLast  = $page < $numberOfPages ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$numberOfPages&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><img src=\"../images/icon_last.gif\" alt=\"Last page\" title=\"Last page\" border=\"0\" /></a>" : "";
my $colspan   = ($numFields + 1);

if($numberOfPages <= $NAVIGATION_MAXPAGES) 
{
	for(my $i=1;$i<=$numberOfPages;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
}
else 
{
	if(($page == 15 + 1) || ($page == $numberOfPages - 15)) 
	{
		# navigation bar with 2 parts
		for(my $i=1;$i<=15+1;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
		$navPages .= "&nbsp;...&nbsp;";
		for(my $i=(($numberOfPages - 15 - 1) + 1);$i<=$numberOfPages;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
	}
	elsif(($page > 15) && ($page <= $numberOfPages - 15)) 
	{
		# navigation bar with 3 parts
		for(my $i=1;$i<=10;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
		$navPages .= "&nbsp;...&nbsp;";
		for(my $i=($page - 5);$i<=($page + 5);$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
		$navPages .= "&nbsp;...&nbsp;";
		for(my $i=(($numberOfPages - 10) + 1);$i<=$numberOfPages;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
	}
	else 
	{
		# navigation bar with 2 parts
		for(my $i=1;$i<=15+1;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
		$navPages .= "&nbsp;...&nbsp;";
		for(my $i=(($numberOfPages - 15 - 1) + 1);$i<=$numberOfPages;$i++) { $navPages .= $i == $page ? "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\"><font color=\"#FF0000\"><b>$i</b></font></a>" : "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;page=$i&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$i</a>"; $navPages .= "&nbsp;" if $i < $numberOfPages; }
	}
}

my $rc2 = $sth2->finish() or doErrorTrapDBI('$sth2->finish()', $sql, $dbh->errstr, $debug);

#
# Printing filter bar
#
@letters = ('A'...'Z');
foreach $l (@letters) {
	$filterBar .= "<a href=\"$prgscriptname?action=listall&amp;filter=$l&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$l</a>";
	$filterBar .= "&nbsp;" if ($l ne 'Z');
}
$filterBar .= "&nbsp;/&nbsp;<a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">ALL</a>";

$filterBar .= "<br>";

@numbers = (0...9);
foreach $n (@numbers) {
	$filterBar .= "<a href=\"$prgscriptname?action=listall&amp;filter=$n&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">$n</a>";
	$filterBar .= "&nbsp;" if ($n != 9);
}
$filterBar .= "&nbsp;/&nbsp;<a href=\"$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">ALL</a>";

#
# Screen Output
#
if($numberOfPages > 1) 
{
	# Navigationbar
	print "  <tr>\n";
	print "    <td colspan=\"$colspan\">\n";
	print "      <table width=\"100%\">\n";
	print "        <tr>\n";
	print "          <td align=\"center\" class=\"navigation\" width=\"155\">$page/$numberOfPages</td>\n";
	print "          <td align=\"center\" class=\"navigation\" colspan=\"" . $numFields . "\">$iconFirst&nbsp;$iconPrev&nbsp;&nbsp;$navPages&nbsp;&nbsp;$iconNext&nbsp;$iconLast</td>\n";
	print "        </tr>\n";
	print "      </table>\n";
	print "    </td>\n";
	print "  </tr>\n";
}

# Filterbar
print "  <tr>\n";
print "    <td colspan=\"$colspan\">\n";
print "      <table width=\"100%\">\n";
print "        <tr>\n";
(defined $filter and $filter ne '') ? print "          <td align=\"center\" class=\"navigation\" width=\"155\"><font color=\"#FF0000\">Filter<br>($filterkey)</font></td>\n" : print "          <td align=\"center\" class=\"navigation\" width=\"155\">Filter<br>($filterkey)</td>\n";
print "          <td align=\"center\" class=\"navigation\" colspan=\"" . $numFields . "\">$filterBar</td>\n";
print "        </tr>\n";
print "      </table>\n";
print "    </td>\n";
print "  </tr>\n";
	
print <<END
</table>
<br>
END
;

}

# ---------------------------------------------------------------------------------

sub ConfirmDelete {
	
my ($prgscriptname, $ukeys_current, $CGISESSID, $debug) = @_;

my $ukeystring = DelimitedListToUrl($ukeys_current,$debug);

print <<END
<br>
<p><b>Are you sure you want me to delete this record?</b></p>
<p><b><a href="$prgscriptname?action=delete&amp;$ukeystring&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;confirmed=1">YES</a>&nbsp;&nbsp;&nbsp;&nbsp;/&nbsp;&nbsp;&nbsp;&nbsp;<a href="$prgscriptname?action=listall&amp;debug=$debug&amp;CGISESSID=$CGISESSID">NO</a></b></p>
<br>
END
;

}

# ---------------------------------------------------------------------------------

1;