# ---------------------------------------------------------------------------------
# NCPL_HTML.pm
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

sub html_start {

$| = 1;

my ($title, $js, $css, $body, $debug) = @_;

print "Content-type: text/html\n\n";

print <<END
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<link rel="icon" href="../images/favicon.ico" type="image/ico">
<link rel="SHORTCUT ICON" href="../images/favicon.ico">	
<meta HTTP-EQUIV="Content-Type" content="text/html; charset=iso-8859-1">
END
;

html_nocache($debug);
html_noindex($debug);
html_stylesheets($css, $debug);
html_javascripts($js, $debug);

print <<END
<title>$title</title>
</head>
END
;

if(defined $body) { print "<body $body>\n"; } else { print "<body>\n"; }

;

}

# ---------------------------------------------------------------------------------

sub html_stop {

my ($html_generated, $html_banners, $alignment, $debug) = @_;

html_generated($alignment, $debug) if (defined $html_generated && $html_generated);

html_banners($alignment, $debug) if (defined $html_banners && $html_banners);

print <<END
</body>
</html>
END
;

$| = 0;

}

# ---------------------------------------------------------------------------------

sub html_nocache {

my $debug = shift;

print <<END
<META HTTP-EQUIV="Expires" CONTENT="Sat, 01 Jan 2005 00:00:01 GMT">
<META HTTP-EQUIV="Pragma" CONTENT="no-cache">
<META HTTP-EQUIV="Cache-Control" CONTENT="no-store, no-cache, must-revalidate">
<META HTTP-EQUIV="Cache-Control" CONTENT="post-check=0, pre-check=0, false">
END
;

}

# ---------------------------------------------------------------------------------

sub html_noindex {

my $debug = shift;

print <<END
<META HTTP-EQUIV="ROBOTS" CONTENT="NOINDEX">
END
;

}

# ---------------------------------------------------------------------------------

sub html_generated {

my ($alignment, $debug) = @_;
my $localtime = localtime();

print <<END
<p class="generated" align="$alignment">Page generated on: $localtime</p>
END
;

}

# ---------------------------------------------------------------------------------

sub html_javascripts {
	my ($js, $debug) = @_;
	if($js) {
		for(my $i=0;$i<@$js;$i++) {
			print "<script type=\"text/javascript\" language=\"JavaScript1.2\" src=\"" . $js->[$i] . "\"></script>\n";
		}
	}
}

# ---------------------------------------------------------------------------------

sub html_stylesheets {
	my ($css, $debug) = @_;
	if($css) {
		for(my $i=0;$i<@$css;$i++) {
			print "<link rel=\"stylesheet\" type=\"text/css\" href=\"" . $css->[$i] . "\">\n";
		}
	}	
}

# ---------------------------------------------------------------------------------

sub html_pagetitle {
	
my ($pagetitle, $CGISESSID, $debug) = @_;

print <<END
<table border="0" width="100%">
  <tr>
    <td align="left"><a href="index.pl?CGISESSID=$CGISESSID&amp;debug=$debug"><img src="../images/icon_toggle.gif" alt="Toggle" title="Toggle" border="0" /></a></td>
    <td align="left"  width="90%"><p class="pagetitle">$pagetitle</p></td>
    <td align="right" width="100%">Log Off <a href="index.pl?action=logoff&amp;submitForm=1&amp;CGISESSID=$CGISESSID&amp;debug=$debug"><img src="../images/icon_logoff.jpg" alt="Log Off" title="Log Off" border="0" /></a></td>
  </tr>
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub html_description {

my ($description, $debug) = @_;

$description = html_textencode($description,$debug);

if ($description) {
print <<END
<br>
<hr>
<table>
  <tr>
	  <td><img src="../images/icon_question.gif" alt="Info" title="Info" /></td>
	  <td>$description</td>
  </tr>
</table>
<hr>
<br>
END
;
}

}

# ---------------------------------------------------------------------------------

sub html_banners {

my ($alignment, $debug) = @_;

print <<END
<p align="$alignment">
	<a href="http://validator.w3.org/check?uri=referer" target="new"><img border="0" src="../images/valid-html401.png" title="Valid HTML 4.01!" alt="Valid HTML 4.01!"></a>
  <a href="https://sourceforge.net/donate/index.php?group_id=132836" target="new"><img border="0" src="../images/project-support.jpg" title="Support this project!" alt="Support this project!"></a>
</p>
END
;

}


# ---------------------------------------------------------------------------------

sub html_menu {
	
my ($scriptname, $titles, $actions, $CGISESSID, $debug) = @_;	

if (@$titles != @$actions) {
	print "A problem occured in function html_menu!\n";
}	
else {
	print "<table>\n";
	print "<tr>\n";
	for(my $i=0;$i<@$titles;$i++) {
		print "<td>[<a href=\"$scriptname?action=" . $actions->[$i] . "&amp;debug=" . $debug . "&amp;CGISESSID=" . $CGISESSID . "\">" . $titles->[$i] . "</a>]</td>\n";
	}
	print "</tr>\n";
	print "</table>\n";
}

}

# ---------------------------------------------------------------------------------

sub html_underconstruction {

my $debug = shift;

print <<END
<p><b><font size=\"3\">This option is currently under construction ...</font></b></p>
END
;
	
}

# ---------------------------------------------------------------------------------

sub html_dbcombobox {

  my ($name, $label, $style, $selected, $dbh, $sql, $tag, $text, $template, $key, $action, $debug) = @_;
	
	my ($sth, $rows) = doQueryDBI($dbh, $sql, '', '', $debug);

	print "<tr>\n";
	print "	<td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	print " <td>\n";
	if($sth and $rows) 
	{ 
		if( ($action  =~ "edit") and $key ) 
		{
			if(! $selected eq '') 
			{
				while (my $ref = $sth->fetchrow_arrayref()) { 	print html_textencode($$ref[1],$debug) . "<br>" if((defined $selected) && ($selected eq $$ref[0]));	}
			}
			else 
			{
				print "-<br>";
			}		
		}
		elsif( ($action =~ "add")  or ($action  =~ "edit") or ($action =~ "doImport") or ($action =~ "doExport") or ($action =~ "duplicate") or ($action =~ "initInstall") )
		{
			print "      <select size=\"$HTML_COMBOBOX_MAXSIZE\" name=\"$name\" $tag>\n";
			print "        <option value=\"\">-Please select-</option>\n";
			while (my $ref = $sth->fetchrow_arrayref())
			{
				if((defined $selected) && ($selected eq $$ref[0])) 
				{
					print "        <option value=\"" . html_textencode($$ref[0],$debug) . "\" selected>" . html_textencode($$ref[1],$debug) . "</option>\n";
				}
				else 
				{
					print "        <option value=\"" . html_textencode($$ref[0],$debug) . "\">" . html_textencode($$ref[1],$debug) . "</option>\n";
				}
			}
			print "      </select>\n";
		}
		else
		{
			if(! $selected eq '') 
			{
				while (my $ref = $sth->fetchrow_arrayref()) { 	print html_textencode($$ref[1],$debug) . "<br>" if((defined $selected) && ($selected eq $$ref[0]));	}
			}
			else 
			{
				print "-<br>";
			}
		}
	}
	else 
	{
		print "No records available in the database!\n";
	}
	print "  </td>\n";
	print "  <td>&nbsp;$text</td>\n";
	print "</tr>\n";
	if(defined $template and $template ne '') { 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: " . html_textencode($template,$debug) . "</td></tr>\n";
	}
	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
}

# ---------------------------------------------------------------------------------

sub html_listbox {

  my ($name, $label, $style, $selected, $dbh, $sql, $tag, $text, $template, $key, $action, $debug) = @_;
	
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	
	print "<tr>\n";
	print "	<td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	print " <td>\n";
	if($sth and $rows)
	{
		my $size = $rows > $HTML_LISTBOX_MAXSIZE ? $HTML_LISTBOX_MAXSIZE : $rows;
		if(($action =~ "add") or ($action =~ "edit") or ($action =~ "duplicate")) 
		{
			print "  <select size=\"$size\" name=\"$name\" multiple $tag>\n";
			while (my $ref = $sth->fetchrow_arrayref()) 
			{
				my $found = 0;
				if(defined $selected) { for(my $i=0;$i<@$selected;$i++) { if($selected->[$i] eq $$ref[0]) { $found=1; next; } } }	
				if($found) { print "   <option value=\"" . html_textencode($$ref[0],$debug) . "\" selected>" . html_textencode($$ref[1],$debug) . "</option>\n"; } 
				else { print "   <option value=\"" . html_textencode($$ref[0],$debug) . "\">" . html_textencode($$ref[1],$debug) . "</option>\n"; }
			} 
			print "  </select>\n";
		}
		else 
		{
			my $aantal = 0;
			while (my $ref = $sth->fetchrow_arrayref()) 
			{
				my $found = 0;
				if(defined $selected) { for(my $i=0;$i<@$selected;$i++) { if($selected->[$i] eq $$ref[0]) { $found=1; $aantal++; next; } } }	
				if($found) { print html_textencode($$ref[1],$debug) . "<br>\n"; } 
			}
			if ($aantal == 0) { print "-\n"; }
		}
	}
	else 
	{
		print "No records available in the database!\n";
	}
	print "  </td>\n";
	print "  <td>&nbsp;$text</td>\n";
	print "</tr>\n";
	if(defined $template and $template ne '') 
	{ 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: " . html_textencode($template,$debug) . "</td></tr>\n";
	}
	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
}

# ---------------------------------------------------------------------------------

sub html_enabledbox {

	my ($name, $label, $style, $selected, $tag, $text, $template, $key, $action, $debug) = @_;
	
	print "<tr>\n";
	print "	<td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	print "	<td>\n";
	if(($action =~ "add") or ($action =~ "edit") or ($action =~ "duplicate")) 
	{
		print "		<select size=\"1\" name=\"$name\" $tag>\n";
		print "			<option value=\"0\"" . ($selected == 0 ? 'selected' : '') . ">No</option>\n";
		print "			<option value=\"1\"" . ($selected == 1 ? 'selected' : '') . ">Yes</option>\n";
		print "		</select>\n";
	} 
	else 
	{
		if($selected == 0)   { print "No\n";  }
		elsif($selected == 1){ print "Yes\n"; }
		else                 { print "-\n";   }
	}
	print "</td>\n";
	print "<td>&nbsp;$text</td>\n";
	print "</tr>\n";
	if(defined $template and $template ne '') { 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: " . ($template == 0 ? 'No' : 'Yes' ) . "</td></tr>\n";
	}
}

# ---------------------------------------------------------------------------------

sub html_textbox {

  my ($name, $label, $style, $size, $value, $tag, $text, $template, $key, $action, $debug) = @_;
	
	print "  <tr>\n";
	print "    <td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	if( ($action  =~ "edit") and $key ) 
	{
		print "    <td>" . (defined $value and $value ne '' ? html_textencode($value,$debug) : "-") . "</td>\n";
	}
	elsif( ($action =~ "add") or ($action  =~ "edit") or ($action =~ "doImport") or ($action =~ "doExport") or ($action =~ "duplicate") or ($action =~ "initInstall") ) 
	{
		print "    <td><input type=\"text\" name=\"$name\" size=\"$size\" value=\"" . html_textencode($value,$debug) . "\" $tag></td>\n";
	}
	else 
	{
		print "    <td>" . (defined $value and $value ne '' ? html_textencode($value,$debug) : "-") . "</td>\n";
	}
	print "    <td>&nbsp;$text</td>\n";
	print "  </tr>\n";
	if(defined $template and $template ne '') { 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: " . html_textencode($template,$debug) . "</td></tr>\n";
	}
}

# ---------------------------------------------------------------------------------

sub html_passwordbox {

	my ($name, $label, $style, $size, $value, $tag, $text, $template, $key, $action, $debug) = @_;
	
	print "  <tr>\n";
	print "    <td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	if( ($action  =~ "edit") and $key ) 
	{
		print "    <td>" . (defined $value and $value ne '' ? "**********" : "**********") . "</td>\n";
	}
	elsif( ($action =~ "add") or ($action  =~ "edit") or ($action =~ "duplicate") ) 
	{
		print "    <td><input type=\"password\" name=\"$name\" size=\"$size\" value=\"$value\" $tag></td>\n";
	}
	else 
	{
		print "    <td>" . (defined $value and $value ne '' ? "**********" : "**********") . "</td>\n";
	}
	print "    <td>&nbsp;$text</td>\n";
	print "  </tr>\n";
	if(defined $template and $template ne '') { 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: **********</td></tr>\n";
	}
}

# ---------------------------------------------------------------------------------

sub html_textarea {

	my ($name, $label, $style, $rows, $cols, $value, $tag, $text, $template, $key, $action, $debug) = @_;
	
	print "  <tr>\n";
	print "    <td class=\"$style\" valign=\"top\"><b>$label</b></td>\n";
	if( ($action  =~ "edit") and $key ) 
	{
		print "    <td>" . (defined $value and $value ne '' ? html_textencode($value,$debug) : "-") . "</td>\n";
	}
	elsif( ($action =~ "add") or ($action  =~ "edit") or ($action =~ "duplicate") ) 
	{
		print "    <td><textarea rows=\"$rows\" name=\"$name\" cols=\"$cols\" $tag>" . html_textencode($value,$debug) . "</textarea></td>\n";
	}
	else 
	{
		print "    <td>" . (defined $value and $value ne '' ? html_textencode($value,$debug) : "-") . "</td>\n";
	}
	print "    <td>&nbsp;$text</td>\n";
	print "  </tr>\n";
	if(defined $template and $template ne '') { 
		print "  <tr><td>&nbsp;</td><td class=\"inherited\">Inherited value: " . html_textencode($template,$debug) . "</td></tr>\n";
	}
}

# ---------------------------------------------------------------------------------

sub html_emptyline {
	
	my ($colspan, $action, $debug) = @_;
	
	print "  <tr>\n";
	print "    <td colspan=\"$colspan\">&nbsp;</td>\n";
	print "  </tr>\n";
	
}

# ---------------------------------------------------------------------------------

sub html_debug {

my $debug = shift;

if($debug) {	

print <<END
<hr>
<h1>HTTP DEBUG INFORMATION</h1>
<table border="0">
END
;

foreach my $key(sort keys %ENV) {
	print "<tr>\n";
  print "<td width=\"150\" valign=\"top\"><font color=\"#0000FF\">$key</font></td>\n";
  if($key ne "QUERY_STRING") 
  {
  	print "<td valign=\"top\">$ENV{$key}</td>\n";  	
	}
	else  
	{
		print "<td valign=\"top\">\n";
		my %querystring = map {my($k,$v) = split(/=/)} split(/&/, $ENV{"QUERY_STRING"});
		foreach my $key(sort keys %querystring) { print "$key=" . html_urlencode($querystring{$key}) . "<br>" }
		print "</td>\n"; 	
	}
	print "</tr>\n";
}

print <<END
</table>
<hr>
END
;

}

}

# ---------------------------------------------------------------------------------

sub html_urlencode {
	my ($url, $debug) = @_;

  #########################
  # "Reserved characters" #
  #########################
	# $url =~ s/\$/%24/g; # Dollar ("$")
	$url =~ s/\&/%26/g;   # Ampersand ("&")
	$url =~ s/\+/%2B/g;   # Plus ("+")
	# $url =~ s/\,/%2C/g; # Comma (",")
	# $url =~ s/\//%2F/g; # Forward slash/Virgule ("/")
	# $url =~ s/\:/%3A/g; # Colon (":")
	# $url =~ s/\;/%3B/g; # Semi-colon (";")
	# $url =~ s/\=/%3D/g; # Equals ("=")
	# $url =~ s/\?/%3F/g; # Question mark ("?")
	# $url =~ s/\@/%40/g; # 'At' symbol ("@")

  #######################
	# "Unsafe characters" #
	#######################
	$url =~ s/ /%20/g;    # Space
	# $url =~ s/\"/%22/g; # Quotation marks
	# $url =~ s/\</%3C/g; # 'Less Than' symbol ("<")
	# $url =~ s/\>/%3E/g; # 'Greater Than' symbol (">")
	$url =~ s/\#/%23/g;   # 'Pound' character ("#")
	# $url =~ s/\%/%25/g; # Percent character ("%")
	# $url =~ s/\{/%7B/g; # Left Curly Brace ("{")
	# $url =~ s/\}/%7D/g; # Right Curly Brace ("}")
  $url =~ s/\|/%7C/g;   # Vertical Bar/Pipe ("|")
  # $url =~ s/\\/%5C/g; # Backslash ("\")
  # $url =~ s/\^/%5E/g; # Caret ("^")
  # $url =~ s/\~/%7E/g; # Tilde ("~")
  # $url =~ s/\[/%5B/g; # Left Square Bracket ("[")
  # $url =~ s/\]/%5D/g; # Right Square Bracket ("]")
  # $url =~ s/\`/%60/g; # Grave Accent ("`") 
	
	return $url;
}

# ---------------------------------------------------------------------------------

sub html_textencode {
	
	my ($text, $debug) = @_;
	
	use HTML::Entities;
	
	if(defined $text && $text ne '') { 
		return encode_entities($text); 
	}
	else {
		return '';
	}
}

# ---------------------------------------------------------------------------------

sub html_goBackTo {
	
	my($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug) = @_;

	print "<a href=\"$prgscriptname?action=listall&amp;filter=$filter&amp;page=$page&amp;debug=$debug&amp;CGISESSID=$CGISESSID&amp;sortfield=$sortfield&amp;sortorder=$sortorder\">Go back</a><br>\n";
	
}

# ---------------------------------------------------------------------------------

1;