#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :SQL :FUNCTIONS);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname        = "NCPL :: Category Management";
my $prgtext        = "Category Management";
my $prgscriptname  = $ENV{"SCRIPT_NAME"};
my $version 	     = "1.000.000";
my $description    = "The Category Management module provides the System Administrator with the ability to control the available categories.";

# ---------------------------------------------------------------------------------

my @html_js         = ('../js/logon.js' , '../js/md5.js');
my @html_css        = ('../stylesheets/ncpl.css');
my @menu_titles     = ('Add new category'  , 'Search category' , 'List all categories');
my @menu_actions    = ('add'               , 'search'          , 'listall');

# ---------------------------------------------------------------------------------

sub AddCategory();
sub DeleteCategory();
sub DetailCategory();
sub DuplicateCategory();
sub EditCategory();
sub SearchCategory();

# ---------------------------------------------------------------------------------

my $cgi               = new CGI;

my $category_name     = (defined $cgi->param('category_name')) 	   ?  $cgi->param('category_name') 		 : '';
my $category_filetype = (defined $cgi->param('category_filetype')) ?  $cgi->param('category_filetype') : '';
my $category_alias    = (defined $cgi->param('category_alias')) 	 ?  $cgi->param('category_alias') 	 : '';
my $enabled           = (defined $cgi->param('enabled'))           ?  $cgi->param('enabled')           : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "category_filetype,category_name", "category_filetype=$category_filetype|category_name=$category_name", undef);

# ---------------------------------------------------------------------------------

my $sql_list_all      = "select $TABLE_FILETYPES.filetype_alias as category_filetype, $TABLE_CATEGORIES.category_name, $TABLE_CATEGORIES.enabled from $TABLE_CATEGORIES, $TABLE_FILETYPES where $TABLE_CATEGORIES.category_filetype=$TABLE_FILETYPES.filetype_id"; 

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);

	if($action    eq 'listall')   { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')       { AddCategory();       }
	elsif($action eq 'search')    { SearchCategory();    }
	elsif($action eq 'edit')      { EditCategory();      }
	elsif($action eq 'duplicate') { DuplicateCategory(); }
	elsif($action eq 'delete')    { DeleteCategory();    }
	elsif($action eq 'detail')    { DetailCategory();    }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddCategory() {

if ($submit) {
	my $sql = "insert into $TABLE_CATEGORIES values(\"$category_filetype\", \"$category_name\", \"$category_alias\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Category added to the database!', 'Category not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="categoriesForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchCategory() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteCategory() {

if($confirmed) 
{
	$category_filetype = getFiletypeByAlias($category_filetype,$dbh,$debug);
	my $sql = "delete from $TABLE_CATEGORIES where category_filetype=\"$category_filetype\" and category_name=\"$category_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Category deleted!', 'Category not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailCategory() {

$category_filetype = getFiletypeByAlias($category_filetype,$dbh,$debug);
my $sql = "select * from $TABLE_CATEGORIES where category_filetype=\"$category_filetype\" and category_name=\"$category_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$category_alias = $ref->{"category_alias"};
	$enabled        = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditCategory() {

if ($submit) 
{
	my $sql = "update $TABLE_CATEGORIES set category_filetype=\"$category_filetype\", category_name=\"$category_name\", category_alias=\"$category_alias\", enabled=$enabled where category_filetype=\"$category_filetype\" and category_name=\"$category_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Category updated!', 'Category not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	$category_filetype = getFiletypeByAlias($category_filetype,$dbh,$debug);
	my $sql = "select * from $TABLE_CATEGORIES where category_filetype=\"$category_filetype\" and category_name=\"$category_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$category_alias = $ref->{"category_alias"};
		$enabled        = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="categoriesForm">
<input type="hidden" name="action"            size="50" value="$action">
<input type="hidden" name="category_filetype" size="50" value="$category_filetype">
<input type="hidden" name="category_name"     size="50" value="$category_name">
<input type="hidden" name="debug"             size="50" value="$debug">
<input type="hidden" name="CGISESSID"         size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DuplicateCategory() {

if ($submit) 
{
  my $sql = "insert into $TABLE_CATEGORIES values(\"$category_filetype\", \"$category_name\", \"$category_alias\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Category duplicated!', 'Category not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	$category_filetype = getFiletypeByAlias($category_filetype,$dbh,$debug);
	my $sql = "select * from $TABLE_CATEGORIES where category_filetype=\"$category_filetype\" and category_name=\"$category_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$category_alias = $ref->{"category_alias"};
		$enabled        = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="categoriesForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_dbcombobox("category_filetype", "category_filetype:", "required", $category_filetype, $dbh, $SQL_FILETYPES, '', '', '', 1, $action, $debug);
	html_textbox("category_name", "category_name:", "required", 50, $category_name, '', '', '', 1, $action, $debug);	
	html_textbox("category_alias", "category_alias:", "general", 50, $category_alias, '', '', '', 0, $action, $debug);	
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------