#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Commands";
my $prgtext       = "Commands";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.021";
my $description   = "A command definition is just that. It defines a command. Commands that can be defined include service checks, service notifications, service event handlers, host checks, host notifications, and host event handlers. Command definitions can contain macros, but you must make sure that you include only those macros that are \"valid\" for the circumstances when the command will be used.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/commands.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new command'  , 'Search command' , 'List all commands' , 'Import old config' , 'Export new config');
my @menu_actions  = ('add'              , 'search'         , 'listall'           , 'doImportCommands'  , 'doExportCommands' );

# ---------------------------------------------------------------------------------

sub AddCommand();
sub DeleteCommand();
sub DetailCommand();
sub DuplicateCommand();
sub EditCommand();
sub SearchCommand();

# ---------------------------------------------------------------------------------

my $cgi             = new CGI;

my $server_name     = (defined $cgi->param('server_name')) 	   ?  $cgi->param('server_name')      : '';
my $category        = (defined $cgi->param('category')) 	     ?  $cgi->param('category')         : '';
my $command_name    = (defined $cgi->param('command_name')) 	 ?  $cgi->param('command_name')     : '';
my $line            = (defined $cgi->param('line')) 	         ?  $cgi->param('line') 	          : '';
my $desc            = (defined $cgi->param('desc')) 	         ?  $cgi->param('desc') 	          : '';
my $enabled         = (defined $cgi->param('enabled'))         ?  $cgi->param('enabled')          : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "command_name", "command_name=$command_name", 1);

# ---------------------------------------------------------------------------------

my $sql_list_all    = "select server_name, category, command_name, enabled from $TABLE_COMMANDS";
my $sql_categories  = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	if($action    eq 'listall')          { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')              { AddCommand();          }
	elsif($action eq 'search')           { SearchCommand();       }
	elsif($action eq 'doImportCommands') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportCommands') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')             { EditCommand();         }
	elsif($action eq 'duplicate')        { DuplicateCommand();    }
	elsif($action eq 'delete')           { DeleteCommand();       }
	elsif($action eq 'detail')           { DetailCommand();       }                           
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddCommand() {

if ($submit) {
	my $sql = "insert into $TABLE_COMMANDS values(\"$server_name\"" . "," . "\"$category\"" . "," . "\"$command_name\"" . "," . $dbh->quote($line) . "," . $dbh->quote($desc) . "," . $enabled . ")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Command added to the database!', 'Commands not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" name="commandsForm" action="$prgscriptname" onsubmit="return validateCommandsForm();">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub SearchCommand() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub EditCommand() {
	
if ($submit) 
{
	my $sql = "update $TABLE_COMMANDS set `server_name`=\"$server_name\", `category`=\"$category\", `line`=" . $dbh->quote($line) . ",`desc`=" . $dbh->quote($desc) . ", enabled=$enabled where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Command updated!', 'Command not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_COMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{ 
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" name="commandsForm" action="$prgscriptname" onsubmit="return validateCommandsForm();">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="command_name" size="50" value="$command_name">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DeleteCommand() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_COMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Command deleted!', 'Command not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailCommand() {

	my $sql = "select * from $TABLE_COMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub DuplicateCommand() {
	
if ($submit) 
{
	my $sql = "insert into $TABLE_COMMANDS values(\"$server_name\"" . "," . "\"$category\"" . "," . "\"$command_name\"" . "," . $dbh->quote($line) . "," . $dbh->quote($desc) . "," . $enabled . ")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Command duplicated!', 'Command not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_COMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{ 
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" name="commandsForm" action="$prgscriptname" onsubmit="return validateCommandsForm();">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
  html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
  html_emptyline(2, $action, $debug);
  html_textbox("command_name", "command_name:", "required", 50, $command_name, '', '', '', 1, $action, $debug);	
  html_textarea("desc", "desc:", "general", 2, 80, $desc, '', '', '', 0, $action, $debug);	
  html_textarea("line", "line:", "required", 5, 80, $line, '', '', '', 0, $action, $debug);	
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------