#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :FUNCTIONS :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Contacts";
my $prgtext       = "Contacts";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.022";
my $description   = "A contact definition is used to identify someone who should be contacted in the event of a problem on your network.";

# ---------------------------------------------------------------------------------

my @html_js         = ('../js/logon.js' , '../js/export.js' , '../js/md5.js');
my @html_css        = ('../stylesheets/ncpl.css');
my @menu_titles     = ('Add new contact'  , 'Search contact' , 'List all contacts' , 'Import old config' , 'Export new config');
my @menu_actions    = ('add'              , 'search'         , 'listall'           , 'doImportContacts'  , 'doExportContacts' );

# ---------------------------------------------------------------------------------

sub AddContact();
sub DeleteContact();
sub DetailContact();
sub DuplicateContact();
sub EditContact();
sub SearchContact();

# ---------------------------------------------------------------------------------

my $cgi       									  = new CGI;

my $server_name                   = (defined $cgi->param('server_name')) 	                  ?  $cgi->param('server_name')                   : '';
my $category                      = (defined $cgi->param('category')) 	                    ?  $cgi->param('category')                      : '';
my $contact_name     						  = (defined $cgi->param('contact_name')) 		 						  ?  $cgi->param('contact_name') 		 							: '';
my $alias    										  = (defined $cgi->param('alias')) 										      ?  $cgi->param('alias') 		 										: '';
my @contactgroups								  = (defined $cgi->param('contactgroups')) 									?  $cgi->param('contactgroups') 		 						: '';
my $contactgroups                 = ArrayToCommaSeparatedList(\@contactgroups);	
my $email    										  = (defined $cgi->param('email')) 												  ?  $cgi->param('email') 												: '';
my $pager    										  = (defined $cgi->param('pager')) 												  ?  $cgi->param('pager') 												: '';
my $addresses											= (defined $cgi->param('addresses'))											?  $cgi->param('addresses')											: '';
my $host_notification_period      = (defined $cgi->param('host_notification_period')) 	 		?  $cgi->param('host_notification_period') 	    : '';
my $service_notification_period   = (defined $cgi->param('service_notification_period')) 	  ?  $cgi->param('service_notification_period') 	: '';
my $host_notification_options     = (defined $cgi->param('host_notification_options')) 	    ?  $cgi->param('host_notification_options') 	  : '';
my $service_notification_options  = (defined $cgi->param('service_notification_options')) 	?  $cgi->param('service_notification_options') 	: '';
my @host_notification_commands    = (defined $cgi->param('host_notification_commands')) 	  ?  $cgi->param('host_notification_commands') 	  : '';
my @service_notification_commands = (defined $cgi->param('service_notification_commands')) 	?  $cgi->param('service_notification_commands') : '';
my $host_notification_commands    = ArrayToCommaSeparatedList(\@host_notification_commands);	
my $service_notification_commands = ArrayToCommaSeparatedList(\@service_notification_commands);
my $enabled    									  = (defined $cgi->param('enabled')) 	 									    ?  $cgi->param('enabled')   										: 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "contact_name", "contact_name=$contact_name", 2);

# ---------------------------------------------------------------------------------

my $sql_list_all    = "select server_name, category, contact_name, alias, enabled from $TABLE_CONTACTS";
my $sql_categories  = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')          { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')              { AddContact();             }
	elsif($action eq 'search')           { SearchContact();          }
	elsif($action eq 'doImportContacts') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportContacts') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')             { EditContact();            }
	elsif($action eq 'duplicate')        { DuplicateContact();       }	
	elsif($action eq 'delete')           { DeleteContact();          }
	elsif($action eq 'detail')           { DetailContact();          }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddContact() {

if ($submit) {
	my $sql = "insert into $TABLE_CONTACTS values(\"$server_name\", \"$category\", \"$contact_name\",\"$alias\",\"$contactgroups\",\"$host_notification_period\",\"$service_notification_period\",\"$host_notification_options\",\"$service_notification_options\",\"$host_notification_commands\",\"$service_notification_commands\",\"$email\",\"$pager\",\"$addresses\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Contact added!', 'Contact not added to the database!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="contactsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchContact() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteContact() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_CONTACTS where contact_name=\"$contact_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Contact deleted!', 'Contact not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailContact() {

my $sql = "select * from $TABLE_CONTACTS where contact_name=\"$contact_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$server_name                   = $ref->{"server_name"};
	$category                      = $ref->{"category"};
  $alias                         = $ref->{"alias"};
  $contactgroups                 = $ref->{"contactgroups"};
  @contactgroups                 = CommaSeparatedListToArray($contactgroups, 1);
  $host_notification_period      = $ref->{"host_notification_period"};
  $service_notification_period   = $ref->{"service_notification_period"};
  $host_notification_options     = $ref->{"host_notification_options"};
  $service_notification_options  = $ref->{"service_notification_options"};
  $host_notification_commands    = $ref->{"host_notification_commands"};
  @host_notification_commands    = CommaSeparatedListToArray($host_notification_commands, 1);
  $service_notification_commands = $ref->{"service_notification_commands"};
  @service_notification_commands = CommaSeparatedListToArray($service_notification_commands, 1);  
  $email                         = $ref->{"email"}; 
  $pager                         = $ref->{"pager"};
	$addresses							 			 = $ref->{"addresses"};
	$enabled											 = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditContact() {

if ($submit) 
{
	my $sql = "update $TABLE_CONTACTS set server_name=\"$server_name\", category=\"$category\", alias=\"$alias\",contactgroups=\"$contactgroups\",host_notification_period=\"$host_notification_period\",service_notification_period=\"$service_notification_period\",host_notification_options=\"$host_notification_options\",service_notification_options=\"$service_notification_options\",host_notification_commands=\"$host_notification_commands\",service_notification_commands=\"$service_notification_commands\",email=\"$email\",pager=\"$pager\",addresses=\"$addresses\",enabled=$enabled where contact_name=\"$contact_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Contact updated!', 'Contact not updated!', $debug);	
  my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
  html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_CONTACTS where contact_name=\"$contact_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                   = $ref->{"server_name"};
		$category                      = $ref->{"category"};
	  $alias                         = $ref->{"alias"};
	  $contactgroups                 = $ref->{"contactgroups"};
		@contactgroups                 = CommaSeparatedListToArray($contactgroups, 1);
	  $host_notification_period      = $ref->{"host_notification_period"};
	  $service_notification_period   = $ref->{"service_notification_period"};
	  $host_notification_options     = $ref->{"host_notification_options"};
	  $service_notification_options  = $ref->{"service_notification_options"};
	  $host_notification_commands    = $ref->{"host_notification_commands"};
	  @host_notification_commands    = CommaSeparatedListToArray($host_notification_commands, 1);
	  $service_notification_commands = $ref->{"service_notification_commands"};
	  @service_notification_commands = CommaSeparatedListToArray($service_notification_commands, 1);  
	  $email                         = $ref->{"email"}; 
	  $pager                         = $ref->{"pager"};
		$addresses							       = $ref->{"addresses"};
		$enabled											 = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="contactsForm">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="contact_name" size="50" value="$contact_name">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;	
}

}

# ---------------------------------------------------------------------------------

sub DuplicateContact() {

if ($submit) 
{
	my $sql = "insert into $TABLE_CONTACTS values(\"$server_name\", \"$category\", \"$contact_name\",\"$alias\",\"$contactgroups\",\"$host_notification_period\",\"$service_notification_period\",\"$host_notification_options\",\"$service_notification_options\",\"$host_notification_commands\",\"$service_notification_commands\",\"$email\",\"$pager\",\"$addresses\",\"$enabled\")";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Contact duplicated!', 'Contact not duplicated!', $debug);	
  my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
  html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_CONTACTS where contact_name=\"$contact_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                   = $ref->{"server_name"};
		$category                      = $ref->{"category"};
	  $alias                         = $ref->{"alias"};
	  $contactgroups                 = $ref->{"contactgroups"};
		@contactgroups                 = CommaSeparatedListToArray($contactgroups, 1);
	  $host_notification_period      = $ref->{"host_notification_period"};
	  $service_notification_period   = $ref->{"service_notification_period"};
	  $host_notification_options     = $ref->{"host_notification_options"};
	  $service_notification_options  = $ref->{"service_notification_options"};
	  $host_notification_commands    = $ref->{"host_notification_commands"};
	  @host_notification_commands    = CommaSeparatedListToArray($host_notification_commands, 1);
	  $service_notification_commands = $ref->{"service_notification_commands"};
	  @service_notification_commands = CommaSeparatedListToArray($service_notification_commands, 1);  
	  $email                         = $ref->{"email"}; 
	  $pager                         = $ref->{"pager"};
		$addresses							       = $ref->{"addresses"};
		$enabled											 = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="contactsForm">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;	
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

  html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
	html_textbox("contact_name", "contact_name:", "required", 50, $contact_name, '', '', '', 1, $action, $debug);	
	html_textbox("alias", "alias:", "required", 50, $alias, '', '', '', 0, $action, $debug);	
	html_listbox("contactgroups", "contactgroups<br>(Nagios 2.x required):", "general", \@contactgroups, $dbh, $SQL_CONTACTGROUPS, '', '', '', 0, $action, $debug);
	html_dbcombobox("host_notification_period", "host_notification_period:", "required", $host_notification_period, $dbh, $SQL_TIMEPERIODS, '', '', '', 0, $action, $debug);
	html_dbcombobox("service_notification_period", "service_notification_period:", "required", $service_notification_period, $dbh, $SQL_TIMEPERIODS, '', '', '', 0, $action, $debug);
	html_textbox("host_notification_options", "host_notification_options:", "required", 50, $host_notification_options, '', 'format: [d,u,r,f,n]', '', 0, $action, $debug);	
	html_textbox("service_notification_options", "service_notification_options:", "required", 50, $service_notification_options, '', 'format: [w,u,c,r,f,n]', '', 0, $action, $debug);	
	html_listbox("host_notification_commands", "host_notification_commands:", "general", \@host_notification_commands, $dbh, $SQL_MISCCOMMANDS, '', '', '', 0, $action, $debug);
	html_listbox("service_notification_commands", "service_notification_commands:", "general", \@service_notification_commands, $dbh, $SQL_MISCCOMMANDS, '', '', '', 0, $action, $debug);
	html_textbox("email", "email:", "general", 50, $email, '', '', '', 0, $action, $debug);	
	html_textbox("pager", "pager:", "general", 50, $pager, '', '', '', 0, $action, $debug);	
	html_textarea("addresses", "addresses<br>(Nagios 2.x required):", "general", "6", "38", $addresses, '', '', '', 0, $action, $debug);
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);

}

# ---------------------------------------------------------------------------------