#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Environment Management";
my $prgtext       = "Environment Management";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.000";
my $description   = "The Environment Management module provides the System Administrator with the ability to control the available environments.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new environment'  , 'Search environment' , 'List all environments');
my @menu_actions  = ('add'                  , 'search'             , 'listall');

# ---------------------------------------------------------------------------------

sub AddEnvironment();
sub DeleteEnvironment();
sub DetailEnvironment();
sub DuplicateEnvironment();
sub EditEnvironment();
sub SearchEnvironment();

# ---------------------------------------------------------------------------------

my $cgi               = new CGI;

my $environment_name  = (defined $cgi->param('environment_name'))  ?  $cgi->param('environment_name')  : '';
my $environment_alias = (defined $cgi->param('environment_alias')) ?  $cgi->param('environment_alias') : '';
my $enabled           = (defined $cgi->param('enabled'))           ?  $cgi->param('enabled')           : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "environment_name", "environment_name=$environment_name", undef);

# ---------------------------------------------------------------------------------

my $sql_list_all      = "select environment_name, environment_alias, enabled from $TABLE_ENVIRONMENTS";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')   { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')       { AddEnvironment();       }
	elsif($action eq 'search')    { SearchEnvironment();    }
	elsif($action eq 'edit')      { EditEnvironment();      }
	elsif($action eq 'duplicate') { DuplicateEnvironment(); }
	elsif($action eq 'delete')    { DeleteEnvironment();    }
	elsif($action eq 'detail')    { DetailEnvironment();    }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddEnvironment() {

if ($submit) {
	my $sql = "insert into $TABLE_ENVIRONMENTS values(\"$environment_name\", \"$environment_alias\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Environment added to the database!', 'Environment not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="environmentsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchEnvironment() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteEnvironment() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_ENVIRONMENTS where environment_name=\"$environment_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Environment deleted!', 'Environment not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailEnvironment() {

my $sql = "select * from $TABLE_ENVIRONMENTS where environment_name=\"$environment_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$environment_alias = $ref->{"environment_alias"};
	$enabled           = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditEnvironment() {

if ($submit) 
{
	my $sql = "update $TABLE_ENVIRONMENTS set environment_name=\"$environment_name\", environment_alias=\"$environment_alias\", enabled=$enabled where environment_name=\"$environment_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Environment updated!', 'Environment not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_ENVIRONMENTS where environment_name=\"$environment_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$environment_alias = $ref->{"environment_alias"};
		$enabled           = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="environmentsForm">
<input type="hidden" name="action"           size="50" value="$action">
<input type="hidden" name="environment_name" size="50" value="$environment_name">
<input type="hidden" name="debug"            size="50" value="$debug">
<input type="hidden" name="CGISESSID"        size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DuplicateEnvironment() {

if ($submit) 
{
	my $sql = "insert into $TABLE_ENVIRONMENTS values(\"$environment_name\", \"$environment_alias\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Environment duplicated!', 'Environment not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_ENVIRONMENTS where environment_name=\"$environment_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$environment_alias = $ref->{"environment_alias"};
		$enabled           = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="environmentsForm">
<input type="hidden" name="action"      size="50" value="$action">
<input type="hidden" name="debug"       size="50" value="$debug">
<input type="hidden" name="CGISESSID"   size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_textbox("environment_name", "environment_name:", "required", 50, $environment_name, '', '', '', 1, $action, $debug);	
	html_textbox("environment_alias", "environment_alias:", "required", 50, $environment_alias, '', '', '', 0, $action, $debug);	
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------