#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :FUNCTIONS :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Hostextinfo";
my $prgtext       = "Hostextinfo";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.002";
my $description   = "Extended host information entries are basically used to make the output from the status, statusmap, statuswrl, and extinfo CGIs look pretty. They have no effect on monitoring and are completely optional.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js', '../js/hostextinfo.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new hostextinfo'  , 'Search hostextinfo', 'List all hostextinfo', 'Add new template' ,  'Search template' ,  'List all templates' , 'Import old config'    , 'Export new config');
my @menu_actions  = ('add_hostextinfo'      , 'search'            , 'list_all'            , 'add_template'     ,  'search'          ,  'list_templates'     , 'doImportHostextinfo'  , 'doExportHostextinfo');

# ---------------------------------------------------------------------------------

sub AddHostextinfo();
sub DeleteHostextinfo();
sub DetailHostextinfo();
sub DuplicateHostextinfo();
sub EditHostextinfo();
sub SearchHostextinfo();

# ---------------------------------------------------------------------------------

my $cgi       			  = new CGI;

my $server_name       = (defined $cgi->param('server_name')) 		 	 ? $cgi->param('server_name') 		 		: '';
my $category          = (defined $cgi->param('category')) 		 	   ? $cgi->param('category') 		 	     	: '';
my $host_name     		= (defined $cgi->param('host_name')) 		 		 ? $cgi->param('host_name') 		 			: '';
my $use_template      = (defined $cgi->param('use_template'))      ? $cgi->param('use_template')        : '';
my $notes             = (defined $cgi->param('notes'))             ? $cgi->param('notes')               : '';
my $notes_url         = (defined $cgi->param('notes_url'))         ? $cgi->param('notes_url')           : '';
my $action_url        = (defined $cgi->param('action_url'))        ? $cgi->param('action_url')          : '';
my $icon_image        = (defined $cgi->param('icon_image'))        ? $cgi->param('icon_image')          : '';
my $icon_image_alt    = (defined $cgi->param('icon_image_alt'))    ? $cgi->param('icon_image_alt')      : '';
my $vrml_image        = (defined $cgi->param('vrml_image'))        ? $cgi->param('vrml_image')          : '';
my $statusmap_image   = (defined $cgi->param('statusmap_image'))   ? $cgi->param('statusmap_image')     : '';
my $coords_2d         = (defined $cgi->param('coords_2d'))         ? $cgi->param('coords_2d')           : '';
my $coords_3d         = (defined $cgi->param('coords_3d'))         ? $cgi->param('coords_3d')           : '';
my $register          = (defined $cgi->param('register')) 	       ? $cgi->param('register') 	          : 0;
my $enabled    				= (defined $cgi->param('enabled')) 	 				 ? $cgi->param('enabled')   					: 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "host_name", "host_name=$host_name", 9);

# ---------------------------------------------------------------------------------

my $template_notes           = '';
my $template_notes_url       = '';
my $template_action_url      = '';
my $template_icon_image      = '';
my $template_icon_image_alt  = '';
my $template_vrml_image      = '';
my $template_statusmap_image = '';
my $template_coords_2d       = '';
my $template_coords_3d       = '';

# ---------------------------------------------------------------------------------

my $sql_list_all       = "select server_name, category, host_name, `use`, register, enabled from $TABLE_HOSTEXTINFO where register=1";
my $sql_list_templates = "select server_name, category, host_name, register, enabled from $TABLE_HOSTEXTINFO where register=0";
my $sql_categories     = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'list_all')            { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'list_templates')      { ListAllRecords($prgscriptname, $dbh, $sql_list_templates, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add_hostextinfo')     { AddHostextinfo();             }
	elsif($action eq 'add_template')        { AddHostextinfo();             }
	elsif($action eq 'search')              { SearchHostextinfo();          }
	elsif($action eq 'doImportHostextinfo') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportHostextinfo') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')                { EditHostextinfo();            }
	elsif($action eq 'duplicate')           { DuplicateHostextinfo();       }	
	elsif($action eq 'delete')              { DeleteHostextinfo();          }
	elsif($action eq 'detail')              { DetailHostextinfo();          }
	else                                    { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }

	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddHostextinfo() {

if ($submit) {
	my $sql = "insert into $TABLE_HOSTEXTINFO values(\"$server_name\", \"$category\", \"$host_name\", \"$use_template\", \"$notes\", \"$notes_url\", \"$action_url\", \"$icon_image\", \"$icon_image_alt\", \"$vrml_image\", \"$statusmap_image\", \"$coords_2d\", \"$coords_3d\", \"$register\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Hostextinfo/Template added!', 'Hostextinfo/Template not added to the database!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
	
if($action =~ "template") { $register = 0; } else { $register = 1; }

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostextinfoForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="register"  size="50" value="$register">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchHostextinfo() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteHostextinfo() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_HOSTEXTINFO where host_name=\"$host_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Hostextinfo/Template deleted!', 'Hostextinfo/Template not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailHostextinfo() {

my $sql = "select * from $TABLE_HOSTEXTINFO where host_name=\"$host_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$server_name     = $ref->{"server_name"};
	$category        = $ref->{"category"};
	$use_template    = $ref->{"use"};
	$notes           = $ref->{"notes"};
	$notes_url       = $ref->{"notes_url"};
	$action_url      = $ref->{"action_url"};
	$icon_image      = $ref->{"icon_image"};
	$icon_image_alt  = $ref->{"icon_image_alt"};
	$vrml_image      = $ref->{"vrml_image"};
	$statusmap_image = $ref->{"statusmap_image"};
	$coords_2d       = $ref->{"2d_coords"};
	$coords_3d       = $ref->{"3d_coords"};
	$register        = $ref->{"register"};
	$enabled         = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditHostextinfo() {

if ($submit) 
{
	my $sql = "update $TABLE_HOSTEXTINFO set server_name=\"$server_name\", category=\"$category\", `use`=\"$use_template\", notes=\"$notes\", notes_url=\"$notes_url\", action_url=\"$action_url\", icon_image=\"$icon_image\", icon_image_alt=\"$icon_image_alt\", vrml_image=\"$vrml_image\", statusmap_image=\"$statusmap_image\", `2d_coords`=\"$coords_2d\", `3d_coords`=\"$coords_3d\", register=\"$register\", enabled=\"$enabled\" where host_name=\"$host_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Hostextinfo/Template updated!', 'Hostextinfo/Template not updated!', $debug);	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_HOSTEXTINFO where host_name=\"$host_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name     = $ref->{"server_name"};
		$category        = $ref->{"category"};
		$use_template    = $ref->{"use"} if($use_template eq '');
		$notes           = $ref->{"notes"};
		$notes_url       = $ref->{"notes_url"};
		$action_url      = $ref->{"action_url"};
		$icon_image      = $ref->{"icon_image"};
		$icon_image_alt  = $ref->{"icon_image_alt"};
		$vrml_image      = $ref->{"vrml_image"};
		$statusmap_image = $ref->{"statusmap_image"};
		$coords_2d       = $ref->{"2d_coords"};
		$coords_3d       = $ref->{"3d_coords"};
		$register        = $ref->{"register"};
		$enabled         = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostextinfoForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="host_name" size="50" value="$host_name">
<input type="hidden" name="register"  size="50" value="$register">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DuplicateHostextinfo() {

if ($submit) 
{
	my $sql = "insert into $TABLE_HOSTEXTINFO values(\"$server_name\", \"$category\", \"$host_name\", \"$use_template\", \"$notes\", \"$notes_url\", \"$action_url\", \"$icon_image\", \"$icon_image_alt\", \"$vrml_image\", \"$statusmap_image\", \"$coords_2d\", \"$coords_3d\", \"$register\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Hostextinfo/Template duplicated!', 'Hostextinfo/Template not duplicated!', $debug);	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_HOSTEXTINFO where host_name=\"$host_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name     = $ref->{"server_name"};
		$category        = $ref->{"category"};
		$use_template    = $ref->{"use"} if($use_template eq '');
		$notes           = $ref->{"notes"};
		$notes_url       = $ref->{"notes_url"};
		$action_url      = $ref->{"action_url"};
		$icon_image      = $ref->{"icon_image"};
		$icon_image_alt  = $ref->{"icon_image_alt"};
		$vrml_image      = $ref->{"vrml_image"};
		$statusmap_image = $ref->{"statusmap_image"};
		$coords_2d       = $ref->{"2d_coords"};
		$coords_3d       = $ref->{"3d_coords"};
		$register        = $ref->{"register"};
		$enabled         = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostextinfoForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {
	
	if($register) { GetTemplate(); }
	html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
	if($register) { html_dbcombobox("host_name", "host_name:", "required", $host_name, $dbh, $SQL_HOSTS, '', '', '', 1, $action, $debug); }
	else          { html_textbox("host_name", "name:", "required", 50, $host_name, '', '', '', 1, $action, $debug); } 	
	html_dbcombobox("use_template", "use_template:", "general", $use_template, $dbh, $SQL_TEMPLATES_HOSTEXTINFO, 'onChange="javascript:submitMyForm();"', '', '', 0, $action, $debug);
	html_textbox("notes", "notes:", "general", 50, $notes, '', '', $template_notes, 0, $action, $debug);
	html_textbox("notes_url", "notes_url:", "general", 50, $notes_url, '', '', $template_notes_url, 0, $action, $debug);
	html_textbox("action_url", "action_url:", "general", 50, $action_url, '', '', $template_action_url, 0, $action, $debug);
	html_textbox("icon_image", "icon_image:", "general", 50, $icon_image, '', '', $template_icon_image, 0, $action, $debug);
	html_textbox("icon_image_alt", "icon_image_alt:", "general", 50, $icon_image_alt, '', '', $template_icon_image_alt, 0, $action, $debug);
	html_textbox("vrml_image", "vrml_image:", "general", 50, $vrml_image, '', '', $template_vrml_image, 0, $action, $debug);
  html_textbox("statusmap_image", "statusmap_image:", "general", 50, $statusmap_image, '', '', $template_statusmap_image, 0, $action, $debug);
	html_textbox("coords_2d", "2d_coords:", "general", 50, $coords_2d, '', 'format: x_coord,y_coord', $template_coords_2d, 0, $action, $debug);
	html_textbox("coords_3d", "3d_coords:", "general", 50, $coords_3d, '', 'format: x_coord,y_coord,z_coord', $template_coords_3d, 0, $action, $debug);
	if ($action eq "duplicate") { html_enabledbox("register", "register:", "required", $register, '', '', '', 0, $action, $debug); }
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);		

}

# ---------------------------------------------------------------------------------	

sub GetTemplate() {
	my $sql = "select * from $TABLE_HOSTEXTINFO where enabled=1 and register=0 and host_name=\"$use_template\" order by host_name";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);		
	while (my $ref = $sth->fetchrow_hashref())
	{
		$template_notes           = $ref->{"notes"};
		$template_notes_url       = $ref->{"notes_url"};
		$template_action_url      = $ref->{"action_url"};
		$template_icon_image      = $ref->{"icon_image"};
		$template_icon_image_alt  = $ref->{"icon_image_alt"};
		$template_vrml_image      = $ref->{"vrml_image"};
		$template_statusmap_image = $ref->{"statusmap_image"};
		$template_coords_2d       = $ref->{"2d_coords"};
		$template_coords_3d       = $ref->{"3d_coords"};
	}		
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
}

# ---------------------------------------------------------------------------------	