#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :FUNCTIONS :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Hosts";
my $prgtext       = "Hosts";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.008";
my $description   = "A host definition is used to define a physical server, workstation, device, etc. that resides on your network.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/hosts.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new host'  , 'Search host', 'List all hosts', 'Add new template' ,  'Search template' ,  'List all templates' , 'Import old config' , 'Export new config');
my @menu_actions  = ('add_host'      , 'search'     , 'list_all'      , 'add_template'     ,  'search'          ,  'list_templates'     , 'doImportHosts'  , 'doExportHosts' );

# ---------------------------------------------------------------------------------

sub AddHost();
sub DeleteHost();
sub DetailHost();
sub DuplicateHost();
sub EditHost();
sub SearchHost();

# ---------------------------------------------------------------------------------

my $cgi       									  = new CGI;

my $server_name                   = (defined $cgi->param('server_name')) 	                  ?  $cgi->param('server_name')                   : '';
my $category                      = (defined $cgi->param('category')) 	                    ?  $cgi->param('category')                      : '';
my @environments                  = (defined $cgi->param('environments')) 	                ?  $cgi->param('environments')                  : '';
my $environments                  = ArrayToCommaSeparatedList(\@environments);	
my $host_name     								= (defined $cgi->param('host_name')) 		 									?  $cgi->param('host_name') 		 								: '';
my $alias    										  = (defined $cgi->param('alias')) 										      ?  $cgi->param('alias') 		 										: '';
my $use_template								  = (defined $cgi->param('use_template'))										?  $cgi->param('use_template')									: '';
my $address											  = (defined $cgi->param('address'))											  ?  $cgi->param('address')										  	: '';
my @parents                       = (defined $cgi->param('parents')) 	                      ?  $cgi->param('parents')                       : '';
my $parents                       = ArrayToCommaSeparatedList(\@parents);	
my @hostgroups                    = (defined $cgi->param('hostgroups')) 	                  ?  $cgi->param('hostgroups')                    : '';
my $hostgroups                    = ArrayToCommaSeparatedList(\@hostgroups);	
my $check_command                 = (defined $cgi->param('check_command')) 	                ?  $cgi->param('check_command') 	              : '';
my $max_check_attempts            = (defined $cgi->param('max_check_attempts')) 	          ?  $cgi->param('max_check_attempts') 	          : '';
my $checks_enabled         				= (defined $cgi->param('checks_enabled')) 	        			?  $cgi->param('checks_enabled') 	      				: '0';
my $check_interval                = (defined $cgi->param('check_interval')) 	              ?  $cgi->param('check_interval') 	              : '';
my $active_checks_enabled         = (defined $cgi->param('active_checks_enabled')) 	        ?  $cgi->param('active_checks_enabled') 	      : '0';
my $passive_checks_enabled        = (defined $cgi->param('passive_checks_enabled')) 	      ?  $cgi->param('passive_checks_enabled') 	      : '0';
my $check_period                  = (defined $cgi->param('check_period')) 	                ?  $cgi->param('check_period') 	                : '';
my $obsess_over_host              = (defined $cgi->param('obsess_over_host')) 	            ?  $cgi->param('obsess_over_host') 	            : '0';
my $check_freshness               = (defined $cgi->param('check_freshness')) 	              ?  $cgi->param('check_freshness') 	            : '0';
my $freshness_threshold           = (defined $cgi->param('freshness_threshold')) 	          ?  $cgi->param('freshness_threshold') 	        : '';
my $event_handler                 = (defined $cgi->param('event_handler')) 	                ?  $cgi->param('event_handler') 	              : '';
my $event_handler_enabled         = (defined $cgi->param('event_handler_enabled')) 	        ?  $cgi->param('event_handler_enabled') 	      : '0';
my $low_flap_threshold            = (defined $cgi->param('low_flap_threshold')) 	          ?  $cgi->param('low_flap_threshold') 	          : '';
my $high_flap_threshold           = (defined $cgi->param('high_flap_threshold')) 	          ?  $cgi->param('high_flap_threshold') 	        : '';
my $flap_detection_enabled        = (defined $cgi->param('flap_detection_enabled')) 	      ?  $cgi->param('flap_detection_enabled') 	      : '0';
my $process_perf_data             = (defined $cgi->param('process_perf_data')) 	            ?  $cgi->param('process_perf_data') 	          : '0';
my $retain_status_information     = (defined $cgi->param('retain_status_information')) 	    ?  $cgi->param('retain_status_information') 	  : '0';
my $retain_nonstatus_information  = (defined $cgi->param('retain_nonstatus_information')) 	?  $cgi->param('retain_nonstatus_information') 	: '0';
my @contact_groups                = (defined $cgi->param('contact_groups')) 	              ?  $cgi->param('contact_groups')                : '';
my $contact_groups                = ArrayToCommaSeparatedList(\@contact_groups);	
my $notification_interval         = (defined $cgi->param('notification_interval')) 	        ?  $cgi->param('notification_interval') 	      : '';
my $notification_period           = (defined $cgi->param('notification_period')) 	          ?  $cgi->param('notification_period') 	        : '';
my $notification_options          = (defined $cgi->param('notification_options')) 	        ?  $cgi->param('notification_options') 	        : '';
my $notifications_enabled         = (defined $cgi->param('notifications_enabled')) 	        ?  $cgi->param('notifications_enabled') 	      : '0';
my $stalking_options              = (defined $cgi->param('stalking_options')) 	            ?  $cgi->param('stalking_options') 	            : '';
my $register              				= (defined $cgi->param('register')) 	           					?  $cgi->param('register') 	            				: 0;
my $enabled    									  = (defined $cgi->param('enabled')) 	 									    ?  $cgi->param('enabled')   										: 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "host_name", "host_name=$host_name", 6);

# ---------------------------------------------------------------------------------

my $template_parents                       = '';
my $template_hostgroups                    = '';
my $template_check_command                 = '';
my $template_max_check_attempts            = '';
my $template_checks_enabled         			 = '';
my $template_check_interval                = '';
my $template_active_checks_enabled         = '';
my $template_passive_checks_enabled        = '';
my $template_check_period                  = '';
my $template_obsess_over_host              = '';
my $template_check_freshness               = '';
my $template_freshness_threshold           = '';
my $template_event_handler                 = '';
my $template_event_handler_enabled         = '';
my $template_low_flap_threshold            = '';
my $template_high_flap_threshold           = '';
my $template_flap_detection_enabled        = '';
my $template_process_perf_data             = '';
my $template_retain_status_information     = '';
my $template_retain_nonstatus_information  = '';
my $template_contact_groups                = '';
my $template_notification_interval         = '';
my $template_notification_period           = '';
my $template_notification_options          = '';
my $template_notifications_enabled         = '';
my $template_stalking_options              = '';

# ---------------------------------------------------------------------------------

my $sql_list_all       = "select server_name, category, host_name, alias, environments, register, enabled from $TABLE_HOSTS where register=1";
my $sql_list_templates = "select server_name, category, host_name, alias, register, enabled from $TABLE_HOSTS where register=0";
my $sql_categories     = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'list_all')       { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'list_templates') { ListAllRecords($prgscriptname, $dbh, $sql_list_templates, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add_host')       { AddHost();       }
	elsif($action eq 'add_template')   { AddHost();       }
	elsif($action eq 'search')         { SearchHost();    }
	elsif($action eq 'doImportHosts')  { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportHosts')  { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')           { EditHost();      }
	elsif($action eq 'duplicate')      { DuplicateHost(); }	
	elsif($action eq 'delete')         { DeleteHost();    }
	elsif($action eq 'detail')         { DetailHost();    }
	else                               { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }

	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddHost() {

if ($submit) {
	my $sql = "insert into $TABLE_HOSTS values(\"$server_name\", \"$category\", \"$environments\", \"$host_name\", \"$alias\", \"$use_template\", \"$address\", \"$parents\", \"$hostgroups\", \"$check_command\", \"$max_check_attempts\", \"$checks_enabled\", \"$check_interval\", \"$active_checks_enabled\", \"$passive_checks_enabled\", \"$check_period\", \"$obsess_over_host\", \"$check_freshness\", \"$freshness_threshold\", \"$event_handler\", \"$event_handler_enabled\", \"$low_flap_threshold\", \"$high_flap_threshold\", \"$flap_detection_enabled\", \"$process_perf_data\", \"$retain_status_information\", \"$retain_nonstatus_information\", \"$contact_groups\", \"$notification_interval\", \"$notification_period\", \"$notification_options\", \"$notifications_enabled\", \"$stalking_options\", \"$register\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Host/Template added!', 'Host/Template not added to the database!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
	
if($action =~ "template") { $register = 0; } else { $register = 1; }

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="register"  size="50" value="$register">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchHost() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteHost() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_HOSTS where host_name=\"$host_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Host/Template deleted!', 'Host/Template not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailHost() {

my $sql = "select * from $TABLE_HOSTS where host_name=\"$host_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$server_name                  = $ref->{"server_name"};
	$category                     = $ref->{"category"};
	$environments                 = $ref->{"environments"};
	@environments                 = CommaSeparatedListToArray($environments, 1);
	$alias                        = $ref->{"alias"};
	$use_template                 = $ref->{"use"};
	$address                      = $ref->{"address"};
	$parents                      = $ref->{"parents"};
	@parents                      = CommaSeparatedListToArray($parents, 1);
	$hostgroups                   = $ref->{"hostgroups"};
	@hostgroups                   = CommaSeparatedListToArray($hostgroups, 1);
	$check_command                = $ref->{"check_command"};
	$max_check_attempts           = $ref->{"max_check_attempts"};
	$checks_enabled               = $ref->{"checks_enabled"};
	$check_interval               = $ref->{"check_interval"};
	$active_checks_enabled        = $ref->{"active_checks_enabled"};
	$passive_checks_enabled       = $ref->{"passive_checks_enabled"};
	$check_period                 = $ref->{"check_period"};
	$obsess_over_host             = $ref->{"obsess_over_host"};
	$check_freshness              = $ref->{"check_freshness"};
	$freshness_threshold          = $ref->{"freshness_threshold"};
	$event_handler                = $ref->{"event_handler"};
	$event_handler_enabled        = $ref->{"event_handler_enabled"};
	$low_flap_threshold           = $ref->{"low_flap_threshold"};
	$high_flap_threshold          = $ref->{"high_flap_threshold"};
	$flap_detection_enabled       = $ref->{"flap_detection_enabled"};
	$process_perf_data            = $ref->{"process_perf_data"};
	$retain_status_information    = $ref->{"retain_status_information"};
	$retain_nonstatus_information = $ref->{"retain_nonstatus_information"};
	$contact_groups               = $ref->{"contact_groups"};
	@contact_groups               = CommaSeparatedListToArray($contact_groups, 1);
	$notification_interval        = $ref->{"notification_interval"};
	$notification_period          = $ref->{"notification_period"};
	$notification_options         = $ref->{"notification_options"};
	$notifications_enabled        = $ref->{"notifications_enabled"};
	$stalking_options             = $ref->{"stalking_options"};
	$register                     = $ref->{"register"};
	$enabled                      = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditHost() {

if ($submit) 
{
	my $sql = "update $TABLE_HOSTS set server_name=\"$server_name\", category=\"$category\", environments=\"$environments\", alias=\"$alias\", `use`=\"$use_template\", address=\"$address\", parents=\"$parents\", hostgroups=\"$hostgroups\", check_command=\"$check_command\", max_check_attempts=\"$max_check_attempts\", checks_enabled=\"$checks_enabled\", check_interval=\"$check_interval\", active_checks_enabled=\"$active_checks_enabled\", passive_checks_enabled=\"$passive_checks_enabled\", check_period=\"$check_period\", obsess_over_host=\"$obsess_over_host\", check_freshness=\"$check_freshness\", freshness_threshold=\"$freshness_threshold\", event_handler=\"$event_handler\", event_handler_enabled=\"$event_handler_enabled\", low_flap_threshold=\"$low_flap_threshold\", high_flap_threshold=\"$high_flap_threshold\", flap_detection_enabled=\"$flap_detection_enabled\", process_perf_data=\"$process_perf_data\", retain_status_information=\"$retain_status_information\", retain_nonstatus_information=\"$retain_nonstatus_information\", contact_groups=\"$contact_groups\", notification_interval=\"$notification_interval\", notification_period=\"$notification_period\", notification_options=\"$notification_options\", notifications_enabled=\"$notifications_enabled\", stalking_options=\"$stalking_options\", register=\"$register\", enabled=\"$enabled\" where host_name=\"$host_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Host/Template updated!', 'Host/Template not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);	
}
else 
{	
	my $sql = "select * from $TABLE_HOSTS where host_name=\"$host_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                  = $ref->{"server_name"};
		$category                     = $ref->{"category"};
		$environments                 = $ref->{"environments"};
		@environments                 = CommaSeparatedListToArray($environments, 1);
		$alias                        = $ref->{"alias"};
		$use_template                 = $ref->{"use"} if($use_template eq '');
		$address                      = $ref->{"address"};
		$parents                      = $ref->{"parents"};
		@parents                      = CommaSeparatedListToArray($parents, 1);
		$hostgroups                   = $ref->{"hostgroups"};
		@hostgroups                   = CommaSeparatedListToArray($hostgroups, 1);
		$check_command                = $ref->{"check_command"};
		$max_check_attempts           = $ref->{"max_check_attempts"};
		$checks_enabled               = $ref->{"checks_enabled"};
		$check_interval               = $ref->{"check_interval"};
		$active_checks_enabled        = $ref->{"active_checks_enabled"};
		$passive_checks_enabled       = $ref->{"passive_checks_enabled"};
		$check_period                 = $ref->{"check_period"};
		$obsess_over_host             = $ref->{"obsess_over_host"};
		$check_freshness              = $ref->{"check_freshness"};
		$freshness_threshold          = $ref->{"freshness_threshold"};
		$event_handler                = $ref->{"event_handler"};
		$event_handler_enabled        = $ref->{"event_handler_enabled"};
		$low_flap_threshold           = $ref->{"low_flap_threshold"};
		$high_flap_threshold          = $ref->{"high_flap_threshold"};
		$flap_detection_enabled       = $ref->{"flap_detection_enabled"};
		$process_perf_data            = $ref->{"process_perf_data"};
		$retain_status_information    = $ref->{"retain_status_information"};
		$retain_nonstatus_information = $ref->{"retain_nonstatus_information"};
		$contact_groups               = $ref->{"contact_groups"};
		@contact_groups               = CommaSeparatedListToArray($contact_groups, 1);
		$notification_interval        = $ref->{"notification_interval"};
		$notification_period          = $ref->{"notification_period"};
		$notification_options         = $ref->{"notification_options"};
		$notifications_enabled        = $ref->{"notifications_enabled"};
		$stalking_options             = $ref->{"stalking_options"};
		$register                     = $ref->{"register"};
		$enabled                      = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="host_name" size="50" value="$host_name">
<input type="hidden" name="register"  size="50" value="$register">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DuplicateHost() {

if ($submit) 
{
	my $sql = "insert into $TABLE_HOSTS values(\"$server_name\", \"$category\", \"$environments\", \"$host_name\",\"$alias\",\"$use_template\",\"$address\",\"$parents\",\"$hostgroups\",\"$check_command\",\"$max_check_attempts\",\"$checks_enabled\",\"$check_interval\",\"$active_checks_enabled\",\"$passive_checks_enabled\",\"$check_period\",\"$obsess_over_host\",\"$check_freshness\",\"$freshness_threshold\",\"$event_handler\",\"$event_handler_enabled\",\"$low_flap_threshold\",\"$high_flap_threshold\",\"$flap_detection_enabled\",\"$process_perf_data\",\"$retain_status_information\",\"$retain_nonstatus_information\",\"$contact_groups\",\"$notification_interval\",\"$notification_period\",\"$notification_options\",\"$notifications_enabled\",\"$stalking_options\",\"$register\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Host/Template duplicated!', 'Host/Template not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);	
}
else 
{	
	my $sql = "select * from $TABLE_HOSTS where host_name=\"$host_name\"";
  my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                  = $ref->{"server_name"};
		$category                     = $ref->{"category"};
		$environments                 = $ref->{"environments"};
		@environments                 = CommaSeparatedListToArray($environments, 1);
		$alias                        = $ref->{"alias"};
		$use_template                 = $ref->{"use"} if($use_template eq '');
		$address                      = $ref->{"address"};
		$parents                      = $ref->{"parents"};
		@parents                      = CommaSeparatedListToArray($parents, 1);
		$hostgroups                   = $ref->{"hostgroups"};
		@hostgroups                   = CommaSeparatedListToArray($hostgroups, 1);
		$check_command                = $ref->{"check_command"};
		$max_check_attempts           = $ref->{"max_check_attempts"};
		$checks_enabled               = $ref->{"checks_enabled"};
		$check_interval               = $ref->{"check_interval"};
		$active_checks_enabled        = $ref->{"active_checks_enabled"};
		$passive_checks_enabled       = $ref->{"passive_checks_enabled"};
		$check_period                 = $ref->{"check_period"};
		$obsess_over_host             = $ref->{"obsess_over_host"};
		$check_freshness              = $ref->{"check_freshness"};
		$freshness_threshold          = $ref->{"freshness_threshold"};
		$event_handler                = $ref->{"event_handler"};
		$event_handler_enabled        = $ref->{"event_handler_enabled"};
		$low_flap_threshold           = $ref->{"low_flap_threshold"};
		$high_flap_threshold          = $ref->{"high_flap_threshold"};
		$flap_detection_enabled       = $ref->{"flap_detection_enabled"};
		$process_perf_data            = $ref->{"process_perf_data"};
		$retain_status_information    = $ref->{"retain_status_information"};
		$retain_nonstatus_information = $ref->{"retain_nonstatus_information"};
		$contact_groups               = $ref->{"contact_groups"};
		@contact_groups               = CommaSeparatedListToArray($contact_groups, 1);
		$notification_interval        = $ref->{"notification_interval"};
		$notification_period          = $ref->{"notification_period"};
		$notification_options         = $ref->{"notification_options"};
		$notifications_enabled        = $ref->{"notifications_enabled"};
		$stalking_options             = $ref->{"stalking_options"};
		$register                     = $ref->{"register"};
		$enabled                      = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="hostsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {
	
	if($register) { GetTemplate(); }
	html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_listbox("environments", "environments:", "general", \@environments, $dbh, $SQL_ENVIRONMENTS, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
	if($register) { html_textbox("host_name", "host_name:", "required", 50, $host_name, '', '', '', 1, $action, $debug); }
	else          { html_textbox("host_name", "name:", "required", 50, $host_name, '', '', '', 1, $action, $debug); } 
  html_textbox("alias", "alias:", "required", 50, $alias, '', '', '', 0, $action, $debug);		
	html_textbox("address", "address:", "required", 50, $address, '', 'format: [host_name/xxx.xxx.xxx.xxx]', '', 0, $action, $debug);		
	if($register) { html_dbcombobox("use_template", "use_template:", "general", $use_template, $dbh, $SQL_TEMPLATES_HOSTS, 'onChange="javascript:submitMyForm();"', '', '', 0, $action, $debug); }
	html_listbox("parents", "parents:", "general", \@parents, $dbh, $SQL_HOSTS, '', '', $template_parents, 0, $action, $debug);
	html_listbox("hostgroups", "hostgroups<br>(Nagios 2.x required):", "general", \@hostgroups, $dbh, $SQL_HOSTGROUPS, '', '', $template_hostgroups, 0, $action, $debug);
	html_dbcombobox("check_command", "check_command:", "general", $check_command, $dbh, $SQL_CHECKCOMMANDS, '', '', $template_check_command, 0, $action, $debug);
	html_textbox("max_check_attempts", "max_check_attempts:", "required", 50, $max_check_attempts, '', 'format: #', $template_max_check_attempts, 0, $action, $debug);		
  html_enabledbox("checks_enabled", "checks_enabled<br>(Nagios 1.x required):", "general", $checks_enabled, '', '', $template_checks_enabled, 0, $action, $debug);
  html_textbox("check_interval", "check_interval<br>(Nagios 2.x required):", "general", 50, $check_interval, '', 'format: #', $template_check_interval, 0, $action, $debug);		
	html_enabledbox("active_checks_enabled", "active_checks_enabled<br>(Nagios 2.x required):", "general", $active_checks_enabled, '', '', $template_active_checks_enabled, 0, $action, $debug);	
	html_enabledbox("passive_checks_enabled", "passive_checks_enabled<br>(Nagios 2.x required):", "general", $passive_checks_enabled, '', '', $template_passive_checks_enabled, 0, $action, $debug);	
	html_dbcombobox("check_period", "check_period<br>(Nagios 2.x required):", "required", $check_period, $dbh, $SQL_TIMEPERIODS, '', '', $template_check_period, 0, $action, $debug);
	html_enabledbox("obsess_over_host", "obsess_over_host<br>(Nagios 2.x required):", "general", $obsess_over_host, '', '', $template_obsess_over_host, 0, $action, $debug);	
	html_enabledbox("check_freshness", "check_freshness<br>(Nagios 2.x required):", "general", $check_freshness, '', '', $template_check_freshness, 0, $action, $debug);	
	html_textbox("freshness_threshold", "freshness_threshold<br>(Nagios 2.x required):", "general", 50, $freshness_threshold, '', 'format: #', $template_freshness_threshold, 0, $action, $debug);		
	html_dbcombobox("event_handler", "event_handler:", "general", $event_handler, $dbh, $SQL_MISCCOMMANDS, '', '', $template_event_handler, 0, $action, $debug);	
	html_enabledbox("event_handler_enabled", "event_handler_enabled:", "general", $event_handler_enabled, '', '', $template_event_handler_enabled, 0, $action, $debug);	
	html_textbox("low_flap_threshold", "low_flap_threshold:", "general", 50, $low_flap_threshold, '', 'format: #', $template_low_flap_threshold, 0, $action, $debug);		
	html_textbox("high_flap_threshold", "high_flap_threshold:", "general", 50, $high_flap_threshold, '', 'format: #', $template_high_flap_threshold, 0, $action, $debug);		
	html_enabledbox("flap_detection_enabled", "flap_detection_enabled:", "general", $flap_detection_enabled, '', '', $template_flap_detection_enabled, 0, $action, $debug);	
	html_enabledbox("process_perf_data", "process_perf_data:", "general", $process_perf_data, '', '', $template_process_perf_data, 0, $action, $debug);	
	html_enabledbox("retain_status_information", "retain_status_information:", "general", $retain_status_information, '', '', $template_retain_status_information, 0, $action, $debug);	
	html_enabledbox("retain_nonstatus_information", "retain_nonstatus_information:", "general", $retain_nonstatus_information, '', '', $template_retain_nonstatus_information, 0, $action, $debug);	
	html_listbox("contact_groups", "contact_groups<br>(Nagios 2.x required):", "required", \@contact_groups, $dbh, $SQL_CONTACTGROUPS, '', '', $template_contact_groups, 0, $action, $debug);
	html_textbox("notification_interval", "notification_interval:", "required", 50, $notification_interval, '', 'format: #', $template_notification_interval, 0, $action, $debug);	
	html_dbcombobox("notification_period", "notification_period:", "required", $notification_period, $dbh, $SQL_TIMEPERIODS, '', '', $template_notification_period, 0, $action, $debug);		
	html_textbox("notification_options", "notification_options:", "required", 50, $notification_options, '', 'format: [d,u,r,f]', $template_notification_options, 0, $action, $debug);		
	html_enabledbox("notifications_enabled", "notifications_enabled:", "general", $notifications_enabled, '', '', $template_notifications_enabled, 0, $action, $debug);		
	html_textbox("stalking_options", "stalking_options:", "required", 50, $stalking_options, '', 'format: [o,d,u]', $template_stalking_options, 0, $action, $debug);		
	if($action eq "duplicate") { html_enabledbox("register", "register:", "required", $register, '', '', '', 0, $action, $debug); }
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);		

}

# ---------------------------------------------------------------------------------	

sub GetTemplate() {
	my $sql = "select * from $TABLE_HOSTS where enabled=1 and register=0 and host_name=\"$use_template\" order by host_name";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);		
	while (my $ref = $sth->fetchrow_hashref())
	{
		$template_parents                       = $ref->{"parents"};
		$template_hostgroups                    = $ref->{"hostgroups"};
		$template_check_command                 = $ref->{"check_command"};
		$template_max_check_attempts            = $ref->{"max_check_attempts"};
		$template_checks_enabled         				= $ref->{"checks_enabled "};
		$template_check_interval                = $ref->{"check_interval"};
		$template_active_checks_enabled         = $ref->{"active_checks_enabled"};
		$template_passive_checks_enabled        = $ref->{"passive_checks_enabled"};
		$template_check_period                  = $ref->{"check_period"};
		$template_obsess_over_host              = $ref->{"obsess_over_host"};
		$template_check_freshness               = $ref->{"check_freshness"};
		$template_freshness_threshold           = $ref->{"freshness_threshold"};
		$template_event_handler                 = $ref->{"event_handler"};
		$template_event_handler_enabled         = $ref->{"event_handler_enabled"};
		$template_low_flap_threshold            = $ref->{"low_flap_threshold"};
		$template_high_flap_threshold           = $ref->{"high_flap_threshold "};
		$template_flap_detection_enabled        = $ref->{"flap_detection_enabled"};
		$template_process_perf_data             = $ref->{"process_perf_data"};
		$template_retain_status_information     = $ref->{"retain_status_information"};
		$template_retain_nonstatus_information  = $ref->{"retain_nonstatus_information"};
		$template_contact_groups                = $ref->{"contact_groups"};
		$template_notification_interval         = $ref->{"notification_interval"};
		$template_notification_period           = $ref->{"notification_period"};
		$template_notification_options          = $ref->{"notification_options"};
		$template_notifications_enabled         = $ref->{"notifications_enabled"};
		$template_stalking_options              = $ref->{"stalking_options"};
	}	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
}

# ---------------------------------------------------------------------------------	

