#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :IMPORT);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Import";
my $prgtext       = "Import Nagios Config";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.006";
my $description   = "This module makes it possible to import existing configuration files.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my $body					= "ONLOAD=\"if (document.images) document.Progress.src='../images/spacer.gif';\"";

# ---------------------------------------------------------------------------------

my $cgi          = new CGI;
my $action       = (defined $cgi->param('action'))      ?  $cgi->param('action')      : 'doImportAll';
my $filename     = (defined $cgi->param('filename'))    ?  $cgi->param('filename')    : '';
my $category     = (defined $cgi->param('category'))    ?  $cgi->param('category')    : '';
my $server_name  = (defined $cgi->param('server_name')) ?  $cgi->param('server_name') : '';
my $submit       = (defined $cgi->param('submitForm'))  ?  $cgi->param('submitForm')  : 0;
my $debug        = (defined $cgi->param('debug'))       ?  $cgi->param('debug')       : 0;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	if($submit) { html_start($prgname, \@html_js, \@html_css, $body, $debug); } else { html_start($prgname, \@html_js, \@html_css, undef, $debug); }
	html_pagetitle($prgtext, $CGISESSID, $debug);
		
	if($submit) {
  	if($action    eq 'doImportAll')                 { initImport($prgscriptname, $filename, 0, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportCommands')            { initImport($prgscriptname, $filename, 1, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportContacts')            { initImport($prgscriptname, $filename, 2, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportContactgroups')       { initImport($prgscriptname, $filename, 3, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportHosts')               { initImport($prgscriptname, $filename, 6, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportHostdependencies')    { initImport($prgscriptname, $filename, 7, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportHostescalations')     { initImport($prgscriptname, $filename, 8, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportHostextinfo')         { initImport($prgscriptname, $filename, 9, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug);  }
	  elsif($action eq 'doImportHostgroups')          { initImport($prgscriptname, $filename, 10, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportServices')            { initImport($prgscriptname, $filename, 11, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportServicedependencies') { initImport($prgscriptname, $filename, 12, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportServiceescalations')  { initImport($prgscriptname, $filename, 13, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportServiceextinfo')      { initImport($prgscriptname, $filename, 14, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportServicegroups')       { initImport($prgscriptname, $filename, 15, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportTimeperiods')         { initImport($prgscriptname, $filename, 16, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportMisccommands')        { initImport($prgscriptname, $filename, 17, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  elsif($action eq 'doImportResources')           { initImport($prgscriptname, $filename, 18, $server_name, $category, $action, $submit, $dbh, $CGISESSID, $debug); }
	  else { DisplayImportForm($prgscriptname, $action, 0, $submit, $dbh, $CGISESSID, $debug); }
	} else {
			DisplayImportForm($prgscriptname, $action, 0, $submit, $dbh, $CGISESSID, $debug); 
  }
  
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------