#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :MYSQLDUMP :SECURITY);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Admin Menu";
my $prgtext       = $APPLICATION_NAME;
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.024";
my $description   = "";
 
# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');

# ---------------------------------------------------------------------------------

sub ShowMenu();

# ---------------------------------------------------------------------------------

my $cgi        = new CGI;
my $action     = (defined $cgi->param('action')) 	   ? $cgi->param('action')     : 'menu';
my $debug      = (defined $cgi->param('debug'))  	   ? $cgi->param('debug')      : 0;
my $submit     = (defined $cgi->param('submitForm')) ? $cgi->param('submitForm') : 0;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);

	if($action eq 'menu')                 { ShowMenu();         }
	elsif($action eq 'doExportDatabase')  { Export_Database();  }
	else { ShowMenu(); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "left", $debug);
}

$dbh->disconnect();
	
exit(0);

# ---------------------------------------------------------------------------------

sub ShowMenu() {

print <<END

<br>
<table border="0" width="100%">
  <tr>
    <td width="50%" valign="top">
			<p class="menutitle">Nagios Config</p>
			<ul>
				<li><a href="commands.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Commands</a></li>
				<li><a href="contacts.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Contacts</a></li>	
				<li><a href="contactgroups.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Contact Groups</a></li>
				<li><a href="hosts.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Hosts</a></li>
				<li>Host Dependencies</li>
				<li>Host Escalations</li>
				<li><a href="hostextinfo.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Host Extended Information</a></li>
				<li><a href="hostgroups.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Host Groups</a></li>
				<li>Host Group Escalations (Nagios 1.x required)</li>
				<li><a href="misccommands.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Misccommands</a></li>	
				<li><a href="services.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Services</a></li>
				<li><a href="servicegroups.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Service Groups</a> (Nagios 2.x required)</li>
				<li>Service Dependencies</li>
				<li>Service Escalations</li>
				<li>Service Extended Information</li>
				<li><a href="timeperiods.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Time Periods</a></li>
			</ul>
			<ul>
				<li>CGI configuration file</li>
				<li>Main configuration file</li>
				<li><a href="resources.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Resources configuration file</a></li>
			</ul>
			
			<p class="menutitle">Nagios Addons</p>
			<ul>
				<li>PerfParse configuration file</li>
			</ul>
			<p class="menutitle">Apache Config</p>
			<ul>
				<li>Authenticated Users</li>
			</ul>
		</td>
    <td width="50%" valign="top">
			<p class="menutitle">Reports</p>
			<ul>
				<li><a href="consistency.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Consistency check</a></li>
				<li><a href="statistics.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Statistics</a></li>
			</ul>

			<p class="menutitle">Tools</p>
			<ul>
				<li><a href="tools.pl?action=nslookupHosts&amp;debug=$debug&amp;CGISESSID=$CGISESSID">NSLOOKUP for hosts</a></li>
				<li><a href="tools.pl?action=syncNagiosByDb&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize NCPL database with nagios-by-db database</a></li>
				<li><a href="tools.pl?action=syncHostextinfoCategoriesWithHostCategories&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize hostextinfo categories with host categories</a></li>
				<li><a href="tools.pl?action=syncServiceCategoriesWithHostCategories&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize service categories with host categories</a></li>
			</ul>
			
			<p class="menutitle">Import/Export/Install/Reload Config</p>
			<ul>
				<li><a href="import.pl?action=doImportAll&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Import all config files</a></li>
				<li><a href="export.pl?action=doExportAll&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Export all config files</a></li>
				<li><a href="install.pl?action=initInstall&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Install new config files</a></li>
				<li>Nagios proces management</li>
			</ul>
						
			<p class="menutitle">NCPL Config</p>
			<ul>
				<li><a href="$prgscriptname?action=doExportDatabase&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Backup NCPL database</a></li>
				<li><a href="categories.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Category management</a></li>
				<li><a href="environments.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Environment management</a></li>
				<li><a href="servers.pl?debug=$debug&amp;CGISESSID=$CGISESSID">Server management</a></li>
				<li><a href="users.pl?debug=$debug&amp;CGISESSID=$CGISESSID">User management</a></li>
			</ul>
		</td>
  </tr>
</table>
END
;

}

# ---------------------------------------------------------------------------------