#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :FUNCTIONS :EXPORT :SMTP :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Install new Nagios Config";
my $prgtext       = "Install new Nagios Config";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.000";
my $description   = "This module makes it possible to install your new nagios config.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/install.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');

# ---------------------------------------------------------------------------------

my $cgi         = new CGI;
my $action      = (defined $cgi->param('action'))      ?  $cgi->param('action')      : 'showMenu';
my $server_name = (defined $cgi->param('server_name')) ?  $cgi->param('server_name') : '';
my $submit      = (defined $cgi->param('submitForm'))  ?  $cgi->param('submitForm')  : 0;
my $debug       = (defined $cgi->param('debug'))       ?  $cgi->param('debug')       : 0;

# ---------------------------------------------------------------------------------

sub installConfig;
sub initInstall;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);

	if($action eq 'initInstall') { initInstall($server_name, $action, $submit,$debug); }
	else { showMenu();  }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();
	
exit(0);

# ---------------------------------------------------------------------------------

sub showMenu {

print <<END
<br>
<ul>
	<li><a href="install.pl?action=installConfig&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Install new config files</a></li>
</ul>
END
;

}

# ---------------------------------------------------------------------------------

sub installConfig { 
	
	my ($exportpath, $installpath, $debug) = @_;
	
	my($HTML, $MAIL, $cmd, $output, $changed);

	$HTML .= "<p class=\"export\">Changes in the new config ...</p>\n";
	$MAIL .= "Changes in the new config:\n";
  $HTML .= "<table border=\"0\">\n";
  for(my $i=1;$i<=18;$i++) { 
  	if($i != 4 && $i != 5) {
  		$HTML .= "<tr><th>$FILENAMES{$i}</th></tr>\n";
  		$MAIL .= "\n$FILENAMES{$i}\n";
  		if( -e "$exportpath/$FILENAMES{$i}" ) 
  		{
		  	$cmd = "$PATH_DIFF -bBaqs -I 'Generated with NCPL v$APPLICATION_VERSION on \\w\\+ \\w\\+[ ]\\+[0-9]\\+ [0-9]\\+:[0-9]\\+:[0-9]\\+ [0-9]\\+' $installpath/$FILENAMES{$i} $exportpath/$FILENAMES{$i}";
  	  	$output = execCMD($cmd, $debug);
		  	$HTML .= "<tr><td><pre>$output</pre></td></tr>\n";
				$HTML .= "<tr><td>&nbsp;</td></tr>\n";
				$MAIL .= "$output\n";
				if($output =~ /differ/) {
					$changed++;
		  		$cmd = "$PATH_DIFF -bBas -I 'Generated with NCPL v$APPLICATION_VERSION on \\w\\+ \\w\\+[ ]\\+[0-9]\\+ [0-9]\\+:[0-9]\\+:[0-9]\\+ [0-9]\\+' $installpath/$FILENAMES{$i} $exportpath/$FILENAMES{$i}";
		  		$output = execCMD($cmd, $debug);
		  		$output =~ s/^</\-/mg; 
		  		$output =~ s/^>/\+/mg; 
		  		$HTML .= "<tr><td><pre>" . html_textencode($output,$debug) . "</pre></td></tr>\n";
		  		$HTML .= "<tr><td>&nbsp;</td></tr>\n";
		  		$MAIL .= "$output\n";
		  	}
		  }
		  else 
		  {
		  	$HTML .= "<tr><td>Please export your new config before running install!</td></tr>\n";
		  	$HTML .= "<tr><td>&nbsp;</td></tr>\n";
		  	$MAIL .= "Please export your new config before running install!\n";
		  }
	  }
  }
	$HTML .= "</table>\n";
	
	$HTML .= "<p class=\"export\">Moving config files to the 'installed' directory ...</p>\n";
	if(! -e $installpath) { 
		print "<p><font color=\"#FFFF00\">WARNING</font> -- Install directory does not exist, directory '$installpath' created!</p>\n";
		mkdir ($installpath); 
	}
	$cmd = "mv $exportpath/*.cfg $installpath";
	$output = execCMD($cmd, $debug);

	$HTML .= "<p class=\"export\">Copying config files to the 'nagios' directory ...</p>\n";
	$HTML .= "<p>Please execute: cp $installpath/*.cfg $NAGIOS_PATH_CONFIGDIR</p>\n";
	$HTML .= "<p>Please execute: chown $NAGIOS_USER:$NAGIOS_GROUP $NAGIOS_PATH_CONFIGDIR/*.cfg</p>\n";
  $HTML .= "<p>Please execute: $NAGIOS_PATH_DAEMON -v $NAGIOS_PATH_CONFIGDIR/nagios.cfg</p>\n";
  $HTML .= "<p>Please execute: $NAGIOS_PATH_INITSCRIPT reload</p>\n";
  
  if($changed) 
  { 
  	print $HTML; 
  	sendEmail($SMTP_FROM, $SMTP_TO, $SMTP_CC, $SMTP_BCC, $SMTP_SUBJECT, $MAIL, 0, $debug) if ($SMTP_ENABLED);
  }
  else 
  {
		print "<p><font color=\"#FFFF00\">WARNING</font> -- The new config is identical with the old config, install cancelled!</p>\n";  
  }
  
}

# ---------------------------------------------------------------------------------

sub initInstall { 
	
	my ($server_name, $action , $submit, $debug) = @_;
	
	my $exportpath  = (defined $server_name && $server_name ne '') ? "$PATH_EXPORT/$server_name"    : "$PATH_EXPORT";
	my $installpath = (defined $server_name && $server_name ne '') ? "$PATH_INSTALLED/$server_name" : "$PATH_INSTALLED";
	
	if($submit) 
	{
		installConfig($exportpath, $installpath, $debug);
	}
	else 
	{
		print "<p>Please enter the required data for the install:</p>\n";
		print "<form method=\"POST\" action=\"$prgscriptname\" name=\"installForm\">\n";
		print "  <input type=\"hidden\" name=\"action\"     size=\"50\" value=\"$action\">\n";
		print "  <input type=\"hidden\" name=\"CGISESSID\"  size=\"50\" value=\"$CGISESSID\">\n";
		print "  <input type=\"hidden\" name=\"debug\"      size=\"50\" value=\"$debug\">\n";
		print "<table>\n";
		html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, 'onChange="javascript:submitInstallForm();"', '', '', 0, $action, $debug);
	  html_textbox("installpath", "directory:", "required", 50, $installpath, '', '', '', 0, $action, $debug);
		print "</table>\n";
		print "<p><input type=\"submit\" name=\"submitForm\" value=\"Install\"> <input type=\"reset\" name=\"reset\" value=\"Reset\"></p>\n";
		print "</form>\n";
	}
	
}
	
# ---------------------------------------------------------------------------------	