#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Misccommands";
my $prgtext       = "Misccommands";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.021";
my $description   = "";

# ---------------------------------------------------------------------------------

my @html_js         = ('../js/logon.js' , '../js/export.js' , '../js/md5.js');
my @html_css        = ('../stylesheets/ncpl.css');
my @menu_titles     = ('Add new misccommand' , 'Search misccommand' , 'List all misccommands' , 'Import old config'    , 'Export new config');
my @menu_actions    = ('add'                 , 'search'             , 'listall'               , 'doImportMisccommands' , 'doExportMisccommands' );

# ---------------------------------------------------------------------------------

sub AddMisccommand();
sub DeleteMisccommand();
sub DetailMisccommand();
sub DuplicateMisccommand();
sub EditMisccommand();
sub SearchMisccommand();

# ---------------------------------------------------------------------------------

my $cgi              = new CGI;

my $server_name      = (defined $cgi->param('server_name')) 	   ?  $cgi->param('server_name')      : '';
my $category         = (defined $cgi->param('category')) 	       ?  $cgi->param('category')         : '';
my $command_name     = (defined $cgi->param('command_name')) 	   ?  $cgi->param('command_name')     : '';
my $line             = (defined $cgi->param('line')) 	           ?  $cgi->param('line') 	          : '';
my $desc             = (defined $cgi->param('desc')) 	           ?  $cgi->param('desc') 	          : '';
my $enabled          = (defined $cgi->param('enabled'))          ?  $cgi->param('enabled')          : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "command_name", "command_name=$command_name", 17);

# ---------------------------------------------------------------------------------

my $sql_list_all    = "select `server_name`, `category`, `command_name`, `desc`, `enabled` from $TABLE_MISCCOMMANDS";
my $sql_categories  = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1) 
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')              { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')                  { AddMisccommand();      }
	elsif($action eq 'search')               { SearchMisccommand();   }
	elsif($action eq 'doImportMisccommands') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportMisccommands') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')                 { EditMisccommand();     }
	elsif($action eq 'duplicate')            { DuplicateMisccommand(); }
	elsif($action eq 'delete')               { DeleteMisccommand();   }
	elsif($action eq 'detail')               { DetailMisccommand();   }
	else                                     { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddMisccommand() {

if ($submit) {
	my $sql = "insert into $TABLE_MISCCOMMANDS values(\"$server_name\", \"$category\", \"$command_name\"" . "," . $dbh->quote($line) . "," . $dbh->quote($desc) . "," . $enabled . ")";
	my ($sth, $rows) = doQueryDBI($dbh, $sql, 'Misccommand added to the database!', 'Misccommand not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}

else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="misccommandsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border=\"0\">
END
;

DisplayForm();

print <<END  
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub SearchMisccommand() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub EditMisccommand() {
	
if ($submit) 
{
	my $sql = "update $TABLE_MISCCOMMANDS set `server_name`=\"$server_name\", `category`=\"$category\", `line`=" . $dbh->quote($line) . ",`desc`=" . $dbh->quote($desc) . ", enabled=$enabled where command_name=\"$command_name\"";
	my ($sth, $rows) = doQueryDBI($dbh, $sql, 'Misccommand updated!', 'Misccommand not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_MISCCOMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
	  $category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="misccommandsForm">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="command_name" size="50" value="$command_name">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DeleteMisccommand() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_MISCCOMMANDS where command_name=\"$command_name\"";
	my ($sth, $rows) = doQueryDBI($dbh, $sql, 'Misccommand deleted!', 'Misccommand not deleted!', $debug); 
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailMisccommand() {

	my $sql = "select * from $TABLE_MISCCOMMANDS where command_name=\"$command_name\"";
	my ($sth, $rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{ 
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub DuplicateMisccommand() {
	
if ($submit) 
{
	my $sql = "insert into $TABLE_MISCCOMMANDS values(\"$server_name\", \"$category\", \"$command_name\"" . "," . $dbh->quote($line) . "," . $dbh->quote($desc) . "," . $enabled . ")";
	my ($sth, $rows) = doQueryDBI($dbh, $sql, 'Misccommand duplicated!', 'Misccommand not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_MISCCOMMANDS where command_name=\"$command_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
	  $category    = $ref->{"category"};
	  $line        = $ref->{"line"};
	  $desc        = $ref->{"desc"};
	  $enabled     = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="misccommandsForm">
<input type="hidden" name="action"       size="50" value="$action">
<input type="hidden" name="debug"        size="50" value="$debug">
<input type="hidden" name="CGISESSID"    size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

  html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
  html_textbox("command_name", "command_name:", "required", 50, $command_name, '', '', '', 1, $action, $debug);	
  html_textarea("desc", "desc:", "general", 2, 80, $desc, '', '', '', 0, $action, $debug);	
  html_textarea("line", "line:", "required", 5, 80, $line, '', '', '', 0, $action, $debug);	
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------