#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :FUNCTIONS);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Nagios";
my $prgtext       = "Nagios";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.000";
my $description   = "This module makes it possible to control your nagios process.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');

# ---------------------------------------------------------------------------------

my $cgi       = new CGI;
my $action    = (defined $cgi->param('action'))     ?  $cgi->param('action')     : 'showMenu';
my $submit    = (defined $cgi->param('submitForm')) ?  $cgi->param('submitForm') : 0;
my $debug     = (defined $cgi->param('debug'))      ?  $cgi->param('debug')      : 0;

# ---------------------------------------------------------------------------------

sub showMenu;
sub startProcess;
sub stopProcess;
sub restartProcess;
sub checkProcess;
sub verifyConfig;
sub reloadConfig;
sub forceReloadConfig;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);

	if($action eq 'startProcess')          { startProcess($debug);       }
	elsif($action eq 'stopProcess')        { stopProcess($debug);        }
	elsif($action eq 'restartProcess')     { restartProcess($debug);     }
	elsif($action eq 'checkProcess')       { checkProcess($debug);       }
	elsif($action eq 'verifyConfig')       { verifyConfig($debug);       }
	elsif($action eq 'reloadConfig')       { reloadConfig($debug);       }
	elsif($action eq 'forceReloadConfig')  { forceReloadConfig($debug);  }
	else { showMenu();  }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();
	
exit(0);

# ---------------------------------------------------------------------------------

sub showMenu {

print <<END
<br>
<ul>
	<li><a href="nagios.pl?action=verifyConfig&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Nagios pre-flight check</a></li>
	<li><a href="nagios.pl?action=reloadConfig&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Reload nagios config</a></li>
	<li><a href="nagios.pl?action=forceReloadConfig&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Force-reload nagios config</a></li>
</ul>
<ul>
	<li><a href="nagios.pl?action=startProcess&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Start nagios proces</a></li>
	<li><a href="nagios.pl?action=stopProcess&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Stop nagios proces</a></li>
	<li><a href="nagios.pl?action=restartProcess&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Restart nagios proces</a></li>
</ul>
<ul>
	<li><a href="nagios.pl?action=checkProcess&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Check nagios proces</a></li>
</ul>
END
;

}

# ---------------------------------------------------------------------------------

sub startProcess { 
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_INITSCRIPT start";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------

sub stopProcess { 
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_INITSCRIPT stop";
	execCMD($cmd, $debug);	
}

# ---------------------------------------------------------------------------------

sub restartProcess {
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_INITSCRIPT restart";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------

sub checkProcess {
	my $debug = shift;
	my $cmd = "ps -f -u nagios | grep \"$NAGIOS_PATH_DAEMON $NAGIOS_PATH_CONFIGDIR/nagios.cfg\"";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------

sub verifyConfig { 
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_DAEMON -v $NAGIOS_PATH_CONFIGDIR/nagios.cfg";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------

sub reloadConfig { 
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_INITSCRIPT reload";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------

sub forceReloadConfig { 
	my $debug = shift;
	my $cmd = "$NAGIOS_PATH_INITSCRIPT force-reload";
	execCMD($cmd, $debug);
}

# ---------------------------------------------------------------------------------