#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :SECURITY);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Resources";
my $prgtext       = "Resources";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.013";
my $description   = "Resource files can be used to store user-defined macros. Resource files can also contain other information (like database connection settings), although this will depend on how you've compiled Nagios. The main point of having resource files is to use them to store sensitive configuration information and not make them available to the CGIs.";

# ---------------------------------------------------------------------------------

my @html_js         = ('../js/logon.js' , '../js/export.js' , '../js/md5.js');
my @html_css        = ('../stylesheets/ncpl.css');
my @menu_titles     = ('Edit resource file' , 'View resource file' , 'Import old config', 'Export new config');
my @menu_actions    = ('edit'               , 'view'               , 'doImportResources', 'doExportResources');

# ---------------------------------------------------------------------------------

sub EditResources();
sub ViewResources();

# ---------------------------------------------------------------------------------

my $cgi           = new CGI;

my $server_name   = (defined $cgi->param('server_name'))   ? $cgi->param('server_name')   : '';
my $category      = (defined $cgi->param('category'))      ? $cgi->param('category')      : '';
my $resource_name = (defined $cgi->param('resource_name')) ? $cgi->param('resource_name') : '';
my $USER1         = (defined $cgi->param('USER1'))         ? $cgi->param('USER1') 	      : '';
my $USER2         = (defined $cgi->param('USER2'))         ? $cgi->param('USER2') 	      : '';
my $USER3         = (defined $cgi->param('USER3'))         ? $cgi->param('USER3') 	      : '';
my $USER4         = (defined $cgi->param('USER4'))         ? $cgi->param('USER4') 	      : '';
my $USER5         = (defined $cgi->param('USER5'))         ? $cgi->param('USER5') 	      : '';
my $USER6         = (defined $cgi->param('USER6'))         ? $cgi->param('USER6') 	      : '';
my $USER7         = (defined $cgi->param('USER7'))         ? $cgi->param('USER7') 	      : '';
my $USER8         = (defined $cgi->param('USER8'))         ? $cgi->param('USER8') 	      : '';
my $USER9         = (defined $cgi->param('USER9'))         ? $cgi->param('USER9') 	      : '';
my $USER10        = (defined $cgi->param('USER10'))        ? $cgi->param('USER10')        : '';
my $USER11        = (defined $cgi->param('USER11'))        ? $cgi->param('USER11')        : '';
my $USER12        = (defined $cgi->param('USER12'))        ? $cgi->param('USER12')        : '';
my $USER13        = (defined $cgi->param('USER13'))        ? $cgi->param('USER13')        : '';
my $USER14        = (defined $cgi->param('USER14'))        ? $cgi->param('USER14')        : '';
my $USER15        = (defined $cgi->param('USER15'))        ? $cgi->param('USER15')        : '';
my $USER16        = (defined $cgi->param('USER16'))        ? $cgi->param('USER16')        : '';
my $USER17        = (defined $cgi->param('USER17'))        ? $cgi->param('USER17')        : '';
my $USER18        = (defined $cgi->param('USER18'))        ? $cgi->param('USER18')        : '';
my $USER19        = (defined $cgi->param('USER19'))        ? $cgi->param('USER19')        : '';
my $USER20        = (defined $cgi->param('USER20'))        ? $cgi->param('USER20')        : '';
my $USER21        = (defined $cgi->param('USER21'))        ? $cgi->param('USER21')        : '';
my $USER22        = (defined $cgi->param('USER22'))        ? $cgi->param('USER22')        : '';
my $USER23        = (defined $cgi->param('USER23'))        ? $cgi->param('USER23')        : '';
my $USER24        = (defined $cgi->param('USER24'))        ? $cgi->param('USER24')        : '';
my $USER25        = (defined $cgi->param('USER25'))        ? $cgi->param('USER25')        : '';
my $USER26        = (defined $cgi->param('USER26'))        ? $cgi->param('USER26')        : '';
my $USER27        = (defined $cgi->param('USER27'))        ? $cgi->param('USER27')        : '';
my $USER28        = (defined $cgi->param('USER28'))        ? $cgi->param('USER28')        : '';
my $USER29        = (defined $cgi->param('USER29'))        ? $cgi->param('USER29')        : '';
my $USER30        = (defined $cgi->param('USER30'))        ? $cgi->param('USER30')        : '';
my $USER31        = (defined $cgi->param('USER31'))        ? $cgi->param('USER31')        : '';
my $USER32        = (defined $cgi->param('USER32'))        ? $cgi->param('USER32')        : '';
my $enabled       = (defined $cgi->param('enabled'))       ? $cgi->param('enabled')       : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "resource_name", "resource_name=$resource_name", 18);

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'view')              { ViewResources(); }
	elsif($action eq 'edit')              { EditResources(); }
	elsif($action eq 'doImportResources') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug);       }
	elsif($action eq 'doExportResources') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	else                                  { ViewResources(); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();	

exit(0);

# ---------------------------------------------------------------------------------

sub ViewResources() {

my $sql = "select * from $TABLE_RESOURCES where resource_name=\"default\" and enabled=1";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
my $ref = $sth->fetchrow_hashref();

print <<END
<br>
<table border="0" width="50%">
END
;

for (my $teller=1; $teller <= 32; $teller++) {
  my $label = "USER$teller";
	print "  <tr><td>\$$label\$=" .$ref->{ $label }. "</td></tr>\n"  if $ref->{ $label };
}

print <<END
</table>
END
;

my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

}

# ---------------------------------------------------------------------------------

sub EditResources() {
	
if ($submit) 
{
	my $sql = "update $TABLE_RESOURCES set USER1=\"$USER1\", USER2=\"$USER2\", USER3=\"$USER3\", USER4=\"$USER4\", USER5=\"$USER5\", USER6=\"$USER6\", USER7=\"$USER7\", USER8=\"$USER8\", USER9=\"$USER9\", USER10=\"$USER10\", USER11=\"$USER11\", USER12=\"$USER12\", USER13=\"$USER13\", USER14=\"$USER14\", USER15=\"$USER15\", USER16=\"$USER16\", USER17=\"$USER17\", USER18=\"$USER18\", USER19=\"$USER19\", USER20=\"$USER20\", USER21=\"$USER21\", USER22=\"$USER22\", USER23=\"$USER23\", USER24=\"$USER24\", USER25=\"$USER25\", USER26=\"$USER26\", USER27=\"$USER27\", USER28=\"$USER28\", USER29=\"$USER29\", USER30=\"$USER30\", USER31=\"$USER31\", USER32=\"$USER32\" WHERE resource_name=\"default\" and enabled=1";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Resources updated!', 'Resources not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_RESOURCES WHERE resource_name=\"default\" and enabled=1";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name   = $ref->{"server_name"};
		$category      = $ref->{"category"};
		$resource_name = $ref->{"resource_name"};
		$USER1         = $ref->{"USER1"} if $ref>{"USER1"};
		$USER2         = $ref->{"USER2"} if $ref>{"USER2"};
		$USER3         = $ref->{"USER3"} if $ref>{"USER3"};
		$USER4         = $ref->{"USER4"} if $ref>{"USER4"};
		$USER5         = $ref->{"USER5"} if $ref>{"USER5"};
		$USER6         = $ref->{"USER6"} if $ref>{"USER6"};
		$USER7         = $ref->{"USER7"} if $ref>{"USER7"};
		$USER8         = $ref->{"USER8"} if $ref>{"USER8"};
		$USER9         = $ref->{"USER9"} if $ref>{"USER9"};
		$USER10        = $ref->{"USER10"} if $ref>{"USER10"};
		$USER11        = $ref->{"USER11"} if $ref>{"USER11"};
		$USER12        = $ref->{"USER12"} if $ref>{"USER12"};
		$USER13        = $ref->{"USER13"} if $ref>{"USER13"};
		$USER14        = $ref->{"USER14"} if $ref>{"USER14"};
		$USER15        = $ref->{"USER15"} if $ref>{"USER15"};
		$USER16        = $ref->{"USER16"} if $ref>{"USER16"};
		$USER17        = $ref->{"USER17"} if $ref>{"USER17"};
		$USER18        = $ref->{"USER18"} if $ref>{"USER18"};
		$USER19        = $ref->{"USER19"} if $ref>{"USER19"};
		$USER20        = $ref->{"USER20"} if $ref>{"USER20"};
		$USER21        = $ref->{"USER21"} if $ref>{"USER21"};
		$USER22        = $ref->{"USER22"} if $ref>{"USER22"};
		$USER23        = $ref->{"USER23"} if $ref>{"USER23"};
		$USER24        = $ref->{"USER24"} if $ref>{"USER24"};
		$USER25        = $ref->{"USER25"} if $ref>{"USER25"};
		$USER26        = $ref->{"USER26"} if $ref>{"USER26"};
		$USER27        = $ref->{"USER27"} if $ref>{"USER27"};
		$USER28        = $ref->{"USER28"} if $ref>{"USER28"};
		$USER29        = $ref->{"USER29"} if $ref>{"USER29"};
		$USER30        = $ref->{"USER30"} if $ref>{"USER30"};
		$USER31        = $ref->{"USER31"} if $ref>{"USER31"};
		$USER32        = $ref->{"USER32"} if $ref>{"USER32"};
		$enabled       = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="resourcesForm">
<input type="hidden" name="action"        size="25" value="$action">
<input type="hidden" name="debug"         size="25" value="$debug">
<input type="hidden" name="CGISESSID"     size="25" value="$CGISESSID">
<input type="hidden" name="server_name"   size="25" value="$server_name">
<input type="hidden" name="category"      size="25" value="$category">
<input type="hidden" name="resource_name" size="25" value="$resource_name">

<table border="0" width="50%">
END
;

print <<END
  <tr>
    <td width="50%"><b>\$USER1\$:</b><br><input type="text" name="USER1" size="35" value="$USER1"></td>
    <td width="50%"><b>\$USER17\$:</b><br><input type="text" name="USER17" size="35" value="$USER17"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER2\$:</b><br><input type="text" name="USER2" size="35" value="$USER2"></td>
    <td width="50%"><b>\$USER18\$:</b><br><input type="text" name="USER18" size="35" value="$USER18"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER3\$:</b><br><input type="text" name="USER3" size="35" value="$USER3"></td>
    <td width="50%"><b>\$USER19\$:</b><br><input type="text" name="USER19" size="35" value="$USER19"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER4\$:</b><br><input type="text" name="USER4" size="35" value="$USER4"></td>
    <td width="50%"><b>\$USER20\$:</b><br><input type="text" name="USER20" size="35" value="$USER20"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER5\$:</b><br><input type="text" name="USER5" size="35" value="$USER5"></td>
    <td width="50%"><b>\$USER21\$:</b><br><input type="text" name="USER21" size="35" value="$USER21"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER6\$:</b><br><input type="text" name="USER6" size="35" value="$USER6"></td>
    <td width="50%"><b>\$USER22\$:</b><br><input type="text" name="USER22" size="35" value="$USER22"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER7\$:</b><br><input type="text" name="USER7" size="35" value="$USER7"></td>
    <td width="50%"><b>\$USER23\$:</b><br><input type="text" name="USER23" size="35" value="$USER23"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER8\$:</b><br><input type="text" name="USER8" size="35" value="$USER8"></td>
    <td width="50%"><b>\$USER24\$:</b><br><input type="text" name="USER24" size="35" value="$USER24"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER9\$:</b><br><input type="text" name="USER9" size="35" value="$USER9"></td>
    <td width="50%"><b>\$USER25\$:</b><br><input type="text" name="USER25" size="35" value="$USER25"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER10\$:</b><br><input type="text" name="USER10" size="35" value="$USER10"></td>
    <td width="50%"><b>\$USER26\$:</b><br><input type="text" name="USER26" size="35" value="$USER26"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER11\$:</b><br><input type="text" name="USER11" size="35" value="$USER11"></td>
    <td width="50%"><b>\$USER27\$:</b><br><input type="text" name="USER27" size="35" value="$USER27"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER12\$:</b><br><input type="text" name="USER12" size="35" value="$USER12"></td>
    <td width="50%"><b>\$USER28\$:</b><br><input type="text" name="USER28" size="35" value="$USER28"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER13\$:</b><br><input type="text" name="USER13" size="35" value="$USER13"></td>
    <td width="50%"><b>\$USER29\$:</b><br><input type="text" name="USER29" size="35" value="$USER29"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER14\$:</b><br><input type="text" name="USER14" size="35" value="$USER14"></td>
    <td width="50%"><b>\$USER30\$:</b><br><input type="text" name="USER30" size="35" value="$USER30"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER15\$:</b><br><input type="text" name="USER15" size="35" value="$USER15"></td>
    <td width="50%"><b>\$USER31\$:</b><br><input type="text" name="USER31" size="35" value="$USER31"></td>
  </tr>
  <tr>
    <td width="50%"><b>\$USER16\$:</b><br><input type="text" name="USER16" size="35" value="$USER16"></td>
    <td width="50%"><b>\$USER32\$:</b><br><input type="text" name="USER32" size="35" value="$USER32"></td>
  </tr>
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------