#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Server Management";
my $prgtext       = "Server Management";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.002";
my $description   = "The Server Management module provides the System Administrator with the ability to control the available nagios servers.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new server'  , 'Search server' , 'List all servers');
my @menu_actions  = ('add'             , 'search'        , 'listall');

# ---------------------------------------------------------------------------------

sub AddServer();
sub DeleteServer();
sub DetailServer();
sub DuplicateServer();
sub EditServer();
sub SearchServer();

# ---------------------------------------------------------------------------------

my $cgi           = new CGI;

my $server_name   = (defined $cgi->param('server_name')) 	?  $cgi->param('server_name') 	: '';
my $server_alias  = (defined $cgi->param('server_alias')) ?  $cgi->param('server_alias') 	: '';
my $server_type   = (defined $cgi->param('server_type')) 	?  $cgi->param('server_type') 	: '';
my $enabled       = (defined $cgi->param('enabled'))      ?  $cgi->param('enabled')       : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "server_name", "server_name=$server_name", undef);

# ---------------------------------------------------------------------------------

my $sql_list_all  = "select server_name, server_alias, server_type, enabled from $TABLE_SERVERS";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')   { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')       { AddServer();       }
	elsif($action eq 'search')    { SearchServer()     }
	elsif($action eq 'edit')      { EditServer();      }
	elsif($action eq 'duplicate') { DuplicateServer(); }
	elsif($action eq 'delete')    { DeleteServer();    }
	elsif($action eq 'detail')    { DetailServer();    }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddServer() {

if ($submit) {
	my $sql = "insert into $TABLE_SERVERS values(\"$server_name\",\"$server_alias\",\"$server_type\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Server added to the database!', 'Server not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="serversForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchServer() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteServer() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_SERVERS where server_name=\"$server_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Server deleted!', 'Server not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailServer() {

my $sql = "select * from $TABLE_SERVERS where server_name=\"$server_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$server_alias = $ref->{"server_alias"};
	$server_type  = $ref->{"server_type"};
	$enabled      = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditServer() {

if ($submit) 
{
	my $sql = "update $TABLE_SERVERS set server_name=\"$server_name\", server_alias=\"$server_alias\", server_type=\"$server_type\", enabled=$enabled where server_name=\"$server_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Server updated!', 'Server not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVERS where server_name=\"$server_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_alias = $ref->{"server_alias"};
		$server_type  = $ref->{"server_type"};
		$enabled      = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="serversForm">
<input type="hidden" name="action"      size="50" value="$action">
<input type="hidden" name="server_name" size="50" value="$server_name">
<input type="hidden" name="debug"       size="50" value="$debug">
<input type="hidden" name="CGISESSID"   size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DuplicateServer() {

if ($submit) 
{
	my $sql = "insert into $TABLE_SERVERS values(\"$server_name\",\"$server_alias\",\"$server_type\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Server duplicated!', 'Server not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVERS where server_name=\"$server_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_alias = $ref->{"server_alias"};
		$server_type  = $ref->{"server_type"};
		$enabled      = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="serversForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_textbox("server_name", "server_name:", "required", 50, $server_name, '', '', '', 1, $action, $debug);	
	html_textbox("server_alias", "server_alias:", "required", 50, $server_alias, '', '', '', 0, $action, $debug);	
	html_textbox("server_type", "server_type:", "required", 50, $server_type, '', 'format: [CENTRAL/DISTRIBUTED]', '', 0, $action, $debug);
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------