#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :FUNCTIONS :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Servicegroups";
my $prgtext       = "Servicegroups";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.008";
my $description   = "A service group definition is used to group one or more services together for display purposes in the CGIs.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new servicegroup'  , 'Search servicegroup' , 'List all servicegroup' , 'Import old config'      , 'Export new config');
my @menu_actions  = ('add'                   , 'search'              , 'listall'               , 'doImportServicegroups'  , 'doExportServicegroups' );

# ---------------------------------------------------------------------------------

sub AddServicegroup();
sub DeleteServicegroup();
sub DetailServicegroup();
sub DuplicateServicegroup();
sub EditServicegroup();
sub SearchServicegroup();

# ---------------------------------------------------------------------------------

my $cgi               = new CGI;

my $server_name       = (defined $cgi->param('server_name'))       ?  $cgi->param('server_name')       : '';
my $category          = (defined $cgi->param('category'))          ?  $cgi->param('category')          : '';
my $servicegroup_name = (defined $cgi->param('servicegroup_name')) ?  $cgi->param('servicegroup_name') : '';
my $alias             = (defined $cgi->param('alias')) 			       ?  $cgi->param('alias') 	           : '';
my @members           = (defined $cgi->param('members')) 	 	       ?  $cgi->param('members')           : '';
my $members           = ArrayToCommaSeparatedList(\@members);	 
my $enabled           = (defined $cgi->param('enabled')) 	 	       ?  $cgi->param('enabled')           : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "servicegroup_name", "servicegroup_name=$servicegroup_name", 15);

# ---------------------------------------------------------------------------------

my $sql_list_all    = "select server_name, category, servicegroup_name, alias, enabled from $TABLE_SERVICEGROUPS";
my $sql_categories  = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')               { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')                   { AddServicegroup();           }
	elsif($action eq 'search')                { SearchServicegroup();        } 
	elsif($action eq 'doImportServicegroups') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportServicegroups') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')                  { EditServicegroup();          }
	elsif($action eq 'duplicate')             { DuplicateServicegroup();     }	
	elsif($action eq 'delete')                { DeleteServicegroup();        }
	elsif($action eq 'detail')                { DetailServicegroup();        }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddServicegroup() {

if ($submit) {
	my $sql = "insert into $TABLE_SERVICEGROUPS values(\"$server_name\", \"$category\", \"$servicegroup_name\", \"$alias\", \"$members\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Servicegroup added!', 'Servicegroup not added to the database!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="servicegroupsForm">
<input type="hidden" name="action"          size="50" value="$action">
<input type="hidden" name="debug"           size="50" value="$debug">
<input type="hidden" name="CGISESSID"       size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchServicegroup() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteServicegroup() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_SERVICEGROUPS where servicegroup_name=\"$servicegroup_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Servicegroup deleted!', 'Servicegroup not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
  ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}	

}

# ---------------------------------------------------------------------------------

sub DetailServicegroup() {

my $sql = "select * from $TABLE_SERVICEGROUPS where servicegroup_name=\"$servicegroup_name\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
  $server_name = $ref->{"server_name"};
  $category    = $ref->{"category"};
  $alias       = $ref->{"alias"};
  $members     = $ref->{"members"};
  @members     = CommaSeparatedListToArray($members, 2);
  $enabled     = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditServicegroup() {

if ($submit) 
{
	my $sql = "update $TABLE_SERVICEGROUPS set server_name=\"$server_name\", category=\"$category\", alias=\"$alias\", members=\"$members\", enabled=$enabled where servicegroup_name=\"$servicegroup_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Servicegroup updated!', 'Servicegroup not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVICEGROUPS where servicegroup_name=\"$servicegroup_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $alias       = $ref->{"alias"};
	  $members     = $ref->{"members"};
	  @members     = CommaSeparatedListToArray($members, 2);
		$enabled	   = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname"   name="servicegroupsForm">
<input type="hidden" name="action"            size="50" value="$action">
<input type="hidden" name="servicegroup_name" size="50" value="$servicegroup_name">
<input type="hidden" name="debug"             size="50" value="$debug">
<input type="hidden" name="CGISESSID"         size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;	
}

}

# ---------------------------------------------------------------------------------

sub DuplicateServicegroup() {

if ($submit) 
{
	my $sql = "insert into $TABLE_SERVICEGROUPS values(\"$server_name\", \"$category\", \"$servicegroup_name\", \"$alias\", \"$members\", \"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Servicegroup duplicated!', 'Servicegroup not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVICEGROUPS where servicegroup_name=\"$servicegroup_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $alias       = $ref->{"alias"};
	  $members     = $ref->{"members"};
	  @members     = CommaSeparatedListToArray($members, 2);
		$enabled	   = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="servicegroupsForm">
<input type="hidden" name="action"          size="50" value="$action">
<input type="hidden" name="debug"           size="50" value="$debug">
<input type="hidden" name="CGISESSID"       size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;	
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {
  
  html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
  html_textbox("servicegroup_name", "servicegroup_name:", "required", 50, $servicegroup_name, '', '', '', 1, $action, $debug);	
  html_textbox("alias", "alias:", "required", 50, $alias, '', '', '', 0, $action, $debug);	
	html_listbox("members", "members:", "required", \@members, $dbh, $SQL_SERVICEGROUPMEMBERS, '', '', '', 0, $action, $debug);
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------