#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :FUNCTIONS :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Services";
my $prgtext       = "Services";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.002";
my $description   = "A service definition is used to identify a \"service\" that runs on a host. The term \"service\" is used very loosely. It can mean an actual service that runs on the host (POP, SMTP, HTTP, etc.) or some other type of metric associated with the host (response to a ping, number of logged in users, free disk space, etc.).";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/services.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new service'  , 'Search service', 'List all services', 'Add new template' ,  'Search template' ,  'List all templates' , 'Import old config' , 'Export new config');
my @menu_actions  = ('add_service'      , 'search'        , 'list_all'         , 'add_template'     ,  'search'          ,  'list_templates'     , 'doImportServices'  , 'doExportServices' );

# ---------------------------------------------------------------------------------

sub AddService();
sub DeleteService();
sub DetailService();
sub DuplicateService();
sub EditService();
sub SearchService();

# ---------------------------------------------------------------------------------

my $cgi       									  = new CGI;

my $server_name                   = (defined $cgi->param('server_name')) 	                     ?  $cgi->param('server_name')                  : '';
my $category                      = (defined $cgi->param('category')) 	                       ?  $cgi->param('category')                     : '';
my $host_name                     = (defined $cgi->param('host_name'))  									     ?  $cgi->param('host_name')    								: '';
my $service_description           = (defined $cgi->param('service_description'))  					   ?  $cgi->param('service_description')    			: '';
my $use_template                  = (defined $cgi->param('use_template'))  									   ?  $cgi->param('use_template')    							: '';
my @servicegroups                 = (defined $cgi->param('servicegroups'))  								   ?  $cgi->param('servicegroups')    						: '';
my $servicegroups                 = ArrayToCommaSeparatedList(\@servicegroups);	
my $is_volatile                   = (defined $cgi->param('is_volatile')) 	 									   ?  $cgi->param('is_volatile')   							  : 0;
my $check_command                 = (defined $cgi->param('check_command'))  							     ?  $cgi->param('check_command')    						: '';
my $check_command_parameters      = (defined $cgi->param('check_command_parameters'))  			   ?  $cgi->param('check_command_parameters') 		: '';
my $max_check_attempts            = (defined $cgi->param('max_check_attempts')) 	 					   ?  $cgi->param('max_check_attempts')   			  : 0;
my $normal_check_interval         = (defined $cgi->param('normal_check_interval')) 	 				   ?  $cgi->param('normal_check_interval')   		  : 0;
my $retry_check_interval          = (defined $cgi->param('retry_check_interval')) 	 				   ?  $cgi->param('retry_check_interval')   			: 0;
my $active_checks_enabled         = (defined $cgi->param('active_checks_enabled')) 	 				   ?  $cgi->param('active_checks_enabled')   			: 0;
my $passive_checks_enabled        = (defined $cgi->param('passive_checks_enabled')) 	 			   ?  $cgi->param('passive_checks_enabled')   		: 0;
my $check_period                  = (defined $cgi->param('check_period'))  									   ?  $cgi->param('check_period')    							: '';
my $parallelize_check             = (defined $cgi->param('parallelize_check')) 	 						   ?  $cgi->param('parallelize_check')   					: 0;
my $obsess_over_service           = (defined $cgi->param('obsess_over_service')) 	 					   ?  $cgi->param('obsess_over_service')   				: 0;
my $check_freshness               = (defined $cgi->param('check_freshness')) 	 							   ?  $cgi->param('check_freshness')   						: 0;
my $freshness_threshold           = (defined $cgi->param('freshness_threshold')) 	 					   ?  $cgi->param('freshness_threshold')   				: 0;
my $event_handler                 = (defined $cgi->param('event_handler'))  								   ?  $cgi->param('event_handler')    						: '';
my $event_handler_enabled         = (defined $cgi->param('event_handler_enabled')) 	 				   ?  $cgi->param('event_handler_enabled')   			: 0;
my $low_flap_threshold            = (defined $cgi->param('low_flap_threshold')) 	 					   ?  $cgi->param('low_flap_threshold')   				: 0;
my $high_flap_threshold           = (defined $cgi->param('high_flap_threshold')) 	 					   ?  $cgi->param('high_flap_threshold')   				: 0;
my $flap_detection_enabled        = (defined $cgi->param('flap_detection_enabled')) 	 		     ?  $cgi->param('flap_detection_enabled')   		: 0;
my $process_perf_data             = (defined $cgi->param('process_perf_data')) 	 						   ?  $cgi->param('process_perf_data')   					: 0;
my $retain_status_information     = (defined $cgi->param('retain_status_information')) 	 		   ?  $cgi->param('retain_status_information')   	: 0;
my $retain_nonstatus_information  = (defined $cgi->param('retain_nonstatus_information')) 	   ?  $cgi->param('retain_nonstatus_information') : 0;
my $notification_interval         = (defined $cgi->param('notification_interval')) 	 				   ?  $cgi->param('notification_interval')   			: 0;
my $notification_period           = (defined $cgi->param('notification_period')) 	 					   ?  $cgi->param('notification_period')   				: 0;
my $notification_options          = (defined $cgi->param('notification_options'))  					   ?  $cgi->param('notification_options')    			: '';
my $notifications_enabled         = (defined $cgi->param('notifications_enabled')) 	 				   ?  $cgi->param('notifications_enabled')   			: 0;
my @contact_groups                = (defined $cgi->param('contact_groups'))  								   ?  $cgi->param('contact_groups')    						: '';
my $contact_groups                = ArrayToCommaSeparatedList(\@contact_groups);	
my $stalking_options              = (defined $cgi->param('stalking_options'))  							   ?  $cgi->param('stalking_options')    					: '';
my $register                      = (defined $cgi->param('register')) 	 										   ?  $cgi->param('register')   									: 0;
my $enabled                       = (defined $cgi->param('enabled')) 	 											   ?  $cgi->param('enabled')   										: 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "host_name,service_description", "host_name=$host_name|service_description=$service_description", 11);

# ---------------------------------------------------------------------------------

my $template_servicegroups                 = '';
my $template_is_volatile                   = '';
my $template_check_command                 = '';
my $template_check_command_parameters      = '';
my $template_max_check_attempts            = '';
my $template_normal_check_interval         = '';
my $template_retry_check_interval          = '';
my $template_active_checks_enabled          = '';
my $template_passive_checks_enabled         = '';
my $template_check_period                  = '';
my $template_parallelize_check             = '';
my $template_obsess_over_service           = '';
my $template_check_freshness               = '';
my $template_freshness_threshold           = '';
my $template_event_handler                 = '';
my $template_event_handler_enabled         = '';
my $template_low_flap_threshold            = '';
my $template_high_flap_threshold           = '';
my $template_flap_detection_enabled        = '';
my $template_process_perf_data             = '';
my $template_retain_status_information     = '';
my $template_retain_nonstatus_information  = '';
my $template_notification_interval         = '';
my $template_notification_period           = '';
my $template_notification_options          = '';
my $template_notifications_enabled         = '';
my $template_contact_groups                = '';
my $template_stalking_options              = '';

# ---------------------------------------------------------------------------------

my $sql_list_all         = "select server_name, category, host_name, service_description, register, enabled from $TABLE_SERVICES WHERE register = 1";
my $sql_list_templates   = "select server_name, category, host_name, service_description, register, enabled from $TABLE_SERVICES WHERE register = 0";
my $sql_categories       = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);

	if($action    eq 'list_all')         { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'list_templates')   { ListAllRecords($prgscriptname, $dbh, $sql_list_templates, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add_service')      { AddService();       }
	elsif($action eq 'add_template')     { AddService();       }
	elsif($action eq 'search')           { SearchService();    }
	elsif($action eq 'doImportServices') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportServices') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')             { EditService();      }
	elsif($action eq 'duplicate')        { DuplicateService(); }									
	elsif($action eq 'delete')           { DeleteService();    }
	elsif($action eq 'detail')           { DetailService();    }
	else                                 { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddService() {

if ($submit) {
	my $sqlSet = 'server_name="' .$server_name. '", category="' .$category. '", host_name="' .$host_name. '", service_description=' .$dbh->quote($service_description). ', `use`="' .$use_template. '", servicegroups="' .$servicegroups. '", is_volatile="' .$is_volatile. '", check_command=' .$dbh->quote($check_command). ', check_command_parameters=' .$dbh->quote($check_command_parameters). ', max_check_attempts="' .$max_check_attempts. '", normal_check_interval="' .$normal_check_interval. '", retry_check_interval="' .$retry_check_interval. '", active_checks_enabled="' .$active_checks_enabled. '", passive_checks_enabled="' .$passive_checks_enabled. '", check_period="' .$check_period. '", parallelize_check="' .$parallelize_check. '", obsess_over_service="' .$obsess_over_service. '", check_freshness="' .$check_freshness. '", freshness_threshold="' .$freshness_threshold. '", event_handler="' .$event_handler. '", event_handler_enabled="' .$event_handler_enabled. '", low_flap_threshold="' .$low_flap_threshold. '", high_flap_threshold="' .$high_flap_threshold. '", flap_detection_enabled="' .$flap_detection_enabled. '", process_perf_data="' .$process_perf_data. '", retain_status_information="' .$retain_status_information. '", retain_nonstatus_information="' .$retain_nonstatus_information. '", notification_interval="' .$notification_interval. '", notification_period="' .$notification_period. '", notification_options="' .$notification_options. '", notifications_enabled="' .$notifications_enabled. '", contact_groups="' .$contact_groups. '", stalking_options="' .$stalking_options. '", register="' .$register. '", enabled="' .$enabled. '"';
	my $sql = "INSERT INTO $TABLE_SERVICES SET $sqlSet";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Service/Template added!', 'Service/Template not added to the database!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {

if($action =~ "template") { $register = 0; } else { $register = 1; }

print <<END
<br>
<form method="POST" action="$prgscriptname" name="servicesForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="register"  size="50" value="$register">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();
  
print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchService() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteService() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_SERVICES where host_name=\"$host_name\" and service_description=\"$service_description\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Service/Template deleted!', 'Service/Template not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailService() {

my $sql = "select * from $TABLE_SERVICES where host_name=\"$host_name\" and service_description=\"$service_description\"";

my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$server_name                   = $ref->{"server_name"};
	$category                      = $ref->{"category"};
	$host_name                     = $ref->{"host_name"};
	$service_description           = $ref->{"service_description"};
	$use_template                  = $ref->{"use"};
	$servicegroups                 = $ref->{"servicegroups"};
	@servicegroups                 = CommaSeparatedListToArray($servicegroups, 1);
	$is_volatile                   = $ref->{"is_volatile"};
	$check_command                 = $ref->{"check_command"}; 
	$check_command_parameters      = $ref->{"check_command_parameters"};
	$max_check_attempts            = $ref->{"max_check_attempts"};
	$normal_check_interval         = $ref->{"normal_check_interval"};
	$retry_check_interval          = $ref->{"retry_check_interval"};
	$active_checks_enabled         = $ref->{"active_checks_enabled"};
	$passive_checks_enabled        = $ref->{"passive_checks_enabled"};
	$check_period                  = $ref->{"check_period"};
	$parallelize_check             = $ref->{"parallelize_check"};
	$obsess_over_service           = $ref->{"obsess_over_service"};
	$check_freshness               = $ref->{"check_freshness"};
	$freshness_threshold           = $ref->{"freshness_threshold"};
	$event_handler                 = $ref->{"event_handler"};
	$event_handler_enabled         = $ref->{"event_handler_enabled"};
	$low_flap_threshold            = $ref->{"low_flap_threshold"};
	$high_flap_threshold           = $ref->{"high_flap_threshold"};
	$flap_detection_enabled        = $ref->{"flap_detection_enabled"};
	$process_perf_data             = $ref->{"process_perf_data"};
	$retain_status_information     = $ref->{"retain_status_information"};
	$retain_nonstatus_information  = $ref->{"retain_nonstatus_information"};
	$notification_interval         = $ref->{"notification_interval"};
	$notification_period           = $ref->{"notification_period"};
	$notification_options          = $ref->{"notification_options"};
	$notifications_enabled         = $ref->{"notifications_enabled"};
	$contact_groups                = $ref->{"contact_groups"};
	@contact_groups                = CommaSeparatedListToArray($contact_groups, 1);
	$stalking_options              = $ref->{"stalking_options"};
	$register                      = $ref->{"register"};
	$enabled                       = $ref->{"enabled"};	
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();
  
print <<END   
</table>
END
;


}

# ---------------------------------------------------------------------------------

sub EditService() {
	
if ($submit) 
{
	my $sqlSet   = 'server_name="' .$server_name. '", category="' .$category. '", host_name="' .$host_name. '", service_description=' .$dbh->quote($service_description). ', `use`="' .$use_template. '", servicegroups="' .$servicegroups. '", is_volatile="' .$is_volatile. '", check_command=' .$dbh->quote($check_command). ', check_command_parameters=' .$dbh->quote($check_command_parameters). ',max_check_attempts="' .$max_check_attempts. '", normal_check_interval="' .$normal_check_interval. '", retry_check_interval="' .$retry_check_interval. '", active_checks_enabled="' .$active_checks_enabled. '", passive_checks_enabled="' .$passive_checks_enabled. '", check_period="' .$check_period. '", parallelize_check="' .$parallelize_check. '", obsess_over_service="' .$obsess_over_service. '", check_freshness="' .$check_freshness. '", freshness_threshold="' .$freshness_threshold. '", event_handler="' .$event_handler. '", event_handler_enabled="' .$event_handler_enabled. '", low_flap_threshold="' .$low_flap_threshold. '", high_flap_threshold="' .$high_flap_threshold. '", flap_detection_enabled="' .$flap_detection_enabled. '", process_perf_data="' .$process_perf_data. '", retain_status_information="' .$retain_status_information. '", retain_nonstatus_information="' .$retain_nonstatus_information. '", notification_interval="' .$notification_interval. '", notification_period="' .$notification_period. '", notification_options="' .$notification_options. '", notifications_enabled="' .$notifications_enabled. '", contact_groups="' .$contact_groups. '", stalking_options="' .$stalking_options. '", register="' .$register. '", enabled="' .$enabled. '"';
	my $sqlWhere = 'host_name = "' .$host_name. '" and service_description = ' .$dbh->quote($service_description);
	my $sql = "UPDATE $TABLE_SERVICES SET $sqlSet WHERE $sqlWhere";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Service/Template updated!', 'Service/Template not updated!', $debug);	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVICES where host_name=\"$host_name\" and service_description=\"$service_description\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                   = $ref->{"server_name"};
		$category                      = $ref->{"category"};
		$use_template                  = $ref->{"use"} if($use_template eq '');
		$servicegroups                 = $ref->{"servicegroups"};
		@servicegroups                 = CommaSeparatedListToArray($servicegroups, 1);
		$is_volatile                   = $ref->{"is_volatile"};
		$check_command                 = $ref->{"check_command"}; 
		$check_command_parameters      = $ref->{"check_command_parameters"};
		$max_check_attempts            = $ref->{"max_check_attempts"};
		$normal_check_interval         = $ref->{"normal_check_interval"};
		$retry_check_interval          = $ref->{"retry_check_interval"};
		$active_checks_enabled         = $ref->{"active_checks_enabled"};
		$passive_checks_enabled        = $ref->{"passive_checks_enabled"};
		$check_period                  = $ref->{"check_period"};
		$parallelize_check             = $ref->{"parallelize_check"};
		$obsess_over_service           = $ref->{"obsess_over_service"};
		$check_freshness               = $ref->{"check_freshness"};
		$freshness_threshold           = $ref->{"freshness_threshold"};
		$event_handler                 = $ref->{"event_handler"};
		$event_handler_enabled         = $ref->{"event_handler_enabled"};
		$low_flap_threshold            = $ref->{"low_flap_threshold"};
		$high_flap_threshold           = $ref->{"high_flap_threshold"};
		$flap_detection_enabled        = $ref->{"flap_detection_enabled"};
		$process_perf_data             = $ref->{"process_perf_data"};
		$retain_status_information     = $ref->{"retain_status_information"};
		$retain_nonstatus_information  = $ref->{"retain_nonstatus_information"};
		$notification_interval         = $ref->{"notification_interval"};
		$notification_period           = $ref->{"notification_period"};
		$notification_options          = $ref->{"notification_options"};
		$notifications_enabled         = $ref->{"notifications_enabled"};
		$contact_groups                = $ref->{"contact_groups"};
		@contact_groups                = CommaSeparatedListToArray($contact_groups, 1);
		$stalking_options              = $ref->{"stalking_options"};
		$register                      = $ref->{"register"};
		$enabled                       = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="servicesForm">
<input type="hidden" name="host_name"           size="50" value="$host_name">
<input type="hidden" name="service_description" size="50" value="$service_description">
<input type="hidden" name="register"            size="50" value="$register">
<input type="hidden" name="action"              size="50" value="$action">
<input type="hidden" name="debug"               size="50" value="$debug">
<input type="hidden" name="CGISESSID"           size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();
  
print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DuplicateService() {
	
if ($submit) 
{
	my $sqlSet = 'server_name="' .$server_name. '", category="' .$category. '", host_name="' .$host_name. '", service_description=' .$dbh->quote($service_description). ', `use`="' .$use_template. '", servicegroups="' .$servicegroups. '", is_volatile="' .$is_volatile. '", check_command=' .$dbh->quote($check_command). ', check_command_parameters=' .$dbh->quote($check_command_parameters). ', max_check_attempts="' .$max_check_attempts. '", normal_check_interval="' .$normal_check_interval. '", retry_check_interval="' .$retry_check_interval. '", active_checks_enabled="' .$active_checks_enabled. '", passive_checks_enabled="' .$passive_checks_enabled. '", check_period="' .$check_period. '", parallelize_check="' .$parallelize_check. '", obsess_over_service="' .$obsess_over_service. '", check_freshness="' .$check_freshness. '", freshness_threshold="' .$freshness_threshold. '", event_handler="' .$event_handler. '", event_handler_enabled="' .$event_handler_enabled. '", low_flap_threshold="' .$low_flap_threshold. '", high_flap_threshold="' .$high_flap_threshold. '", flap_detection_enabled="' .$flap_detection_enabled. '", process_perf_data="' .$process_perf_data. '", retain_status_information="' .$retain_status_information. '", retain_nonstatus_information="' .$retain_nonstatus_information. '", notification_interval="' .$notification_interval. '", notification_period="' .$notification_period. '", notification_options="' .$notification_options. '", notifications_enabled="' .$notifications_enabled. '", contact_groups="' .$contact_groups. '", stalking_options="' .$stalking_options. '", register="' .$register. '", enabled="' .$enabled. '"';
	my $sql = "INSERT INTO $TABLE_SERVICES SET $sqlSet";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Service/Template duplicated!', 'Service/Template not duplicated!', $debug);	
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_SERVICES where host_name=\"$host_name\" and service_description=\"$service_description\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);	
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name                   = $ref->{"server_name"};
		$category                      = $ref->{"category"};
		$use_template                  = $ref->{"use"} if($use_template eq '');
		$servicegroups                 = $ref->{"servicegroups"};
		@servicegroups                 = CommaSeparatedListToArray($servicegroups, 1);
		$is_volatile                   = $ref->{"is_volatile"};
		$check_command                 = $ref->{"check_command"}; 
		$check_command_parameters      = $ref->{"check_command_parameters"};
		$max_check_attempts            = $ref->{"max_check_attempts"};
		$normal_check_interval         = $ref->{"normal_check_interval"};
		$retry_check_interval          = $ref->{"retry_check_interval"};
		$active_checks_enabled         = $ref->{"active_checks_enabled"};
		$passive_checks_enabled        = $ref->{"passive_checks_enabled"};
		$check_period                  = $ref->{"check_period"};
		$parallelize_check             = $ref->{"parallelize_check"};
		$obsess_over_service           = $ref->{"obsess_over_service"};
		$check_freshness               = $ref->{"check_freshness"};
		$freshness_threshold           = $ref->{"freshness_threshold"};
		$event_handler                 = $ref->{"event_handler"};
		$event_handler_enabled         = $ref->{"event_handler_enabled"};
		$low_flap_threshold            = $ref->{"low_flap_threshold"};
		$high_flap_threshold           = $ref->{"high_flap_threshold"};
		$flap_detection_enabled        = $ref->{"flap_detection_enabled"};
		$process_perf_data             = $ref->{"process_perf_data"};
		$retain_status_information     = $ref->{"retain_status_information"};
		$retain_nonstatus_information  = $ref->{"retain_nonstatus_information"};
		$notification_interval         = $ref->{"notification_interval"};
		$notification_period           = $ref->{"notification_period"};
		$notification_options          = $ref->{"notification_options"};
		$notifications_enabled         = $ref->{"notifications_enabled"};
		$contact_groups                = $ref->{"contact_groups"};
		@contact_groups                = CommaSeparatedListToArray($contact_groups, 1);
		$stalking_options              = $ref->{"stalking_options"};
		$register                      = $ref->{"register"};
		$enabled                       = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="servicesForm">
<input type="hidden" name="action"              size="50" value="$action">
<input type="hidden" name="debug"               size="50" value="$debug">
<input type="hidden" name="CGISESSID"           size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();
  
print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
	
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	if($register) { GetTemplate(); }
	html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
	if($register) { html_dbcombobox("host_name", "host_name:", "required", $host_name, $dbh, $SQL_HOSTS, '', '', '', 1, $action, $debug); }
	else          { html_textbox("host_name", "name:", "required", 50, $host_name, '', '', '', 1, $action, $debug); } 
	html_textbox("service_description", "service_description:", "required", 50, $service_description, '', '', '', 1, $action, $debug);
	if($register) { html_dbcombobox("use_template", "use_template:", "general", $use_template, $dbh, $SQL_TEMPLATES_SERVICES, 'onChange="javascript:submitMyForm();"', '', '', 0, $action, $debug); }
	html_listbox("servicegroups", "servicegroups<br>(Nagios 2.x required):", "general", \@servicegroups, $dbh, $SQL_SERVICEGROUPS, '', '', $template_servicegroups, 0, $action, $debug);
	html_enabledbox("is_volatile", "is_volatile:", "general", $is_volatile, '', '', $template_is_volatile, 0, $action, $debug);
	html_dbcombobox("check_command", "check_command:", "required", $check_command, $dbh, $SQL_CHECKCOMMANDS, '', '', $template_check_command, 0, $action, $debug);
	html_textbox("check_command_parameters", "check_command_parameters:", "general", 50, $check_command_parameters, '', 'format: p1!p2!p3!...', $template_check_command_parameters, 0, $action, $debug);
	html_textbox("max_check_attempts", "max_check_attempts:", "required", 50, $max_check_attempts, '', 'format: #', $template_max_check_attempts, 0, $action, $debug);
	html_textbox("normal_check_interval", "normal_check_interval:", "required", 50, $normal_check_interval, '', 'format: #', $template_normal_check_interval, 0, $action, $debug);
	html_textbox("retry_check_interval", "retry_check_interval:", "required", 50, $retry_check_interval, '', 'format: #', $template_retry_check_interval, 0, $action, $debug);
	html_enabledbox("active_checks_enabled", "active_checks_enabled:", "general", $active_checks_enabled, '', '', $template_active_checks_enabled, 0, $action, $debug);
	html_enabledbox("passive_checks_enabled", "passive_checks_enabled:", "general", $passive_checks_enabled, '', '', $template_passive_checks_enabled, 0, $action, $debug);
	html_dbcombobox("check_period", "check_period:", "required", $check_period, $dbh, $SQL_TIMEPERIODS, '', '', $template_check_period, 0, $action, $debug);
	html_enabledbox("parallelize_check", "parallelize_check:", "general", $parallelize_check, '', '', $template_parallelize_check, 0, $action, $debug);
	html_enabledbox("obsess_over_service", "obsess_over_service:", "general", $obsess_over_service, '', '', $template_obsess_over_service, 0, $action, $debug);
	html_enabledbox("check_freshness", "check_freshness:", "general", $check_freshness, '', '', $template_check_freshness, 0, $action, $debug);	
	html_textbox("freshness_threshold", "freshness_threshold:", "general", 50, $freshness_threshold, '', 'format: #', $template_freshness_threshold, 0, $action, $debug);
	html_dbcombobox("event_handler", "event_handler:", "general", $event_handler, $dbh, $SQL_MISCCOMMANDS, '', '', $template_event_handler, 0, $action, $debug);
	html_enabledbox("event_handler_enabled", "event_handler_enabled:", "general", $event_handler_enabled, '', '', $template_event_handler_enabled, 0, $action, $debug);	
	html_textbox("low_flap_threshold", "low_flap_threshold:", "general", 50, $low_flap_threshold, '', 'format: #', $template_low_flap_threshold, 0, $action, $debug);
	html_textbox("high_flap_threshold", "high_flap_threshold:", "general", 50, $high_flap_threshold, '', 'format: #', $template_high_flap_threshold, 0, $action, $debug);
	html_enabledbox("flap_detection_enabled", "flap_detection_enabled:", "general", $flap_detection_enabled, '', '', $template_flap_detection_enabled, 0, $action, $debug);	
	html_enabledbox("process_perf_data", "process_perf_data:", "general", $process_perf_data, '', '', $template_process_perf_data, 0, $action, $debug);		
	html_enabledbox("retain_status_information", "retain_status_information:", "general", $retain_status_information, '', '', $template_retain_status_information, 0, $action, $debug);			
	html_enabledbox("retain_nonstatus_information", "retain_nonstatus_information:", "general", $retain_nonstatus_information, '', '', $template_retain_nonstatus_information, 0, $action, $debug);			
	html_textbox("notification_interval", "notification_interval:", "required", 50, $notification_interval, '', 'format: #', $template_notification_interval, 0, $action, $debug);
	html_dbcombobox("notification_period", "notification_period:", "required", $notification_period, $dbh, $SQL_TIMEPERIODS, '', '', $template_notification_period, 0, $action, $debug);
	html_textbox("notification_options", "notification_options:", "required", 50, $notification_options, '', 'format: [w,u,c,r,f]', $template_notification_options, 0, $action, $debug);
	html_enabledbox("notifications_enabled", "notifications_enabled:", "general", $notifications_enabled, '', '', $template_notifications_enabled, 0, $action, $debug);				
	html_listbox("contact_groups", "contact_groups:", "required", \@contact_groups, $dbh, $SQL_CONTACTGROUPS, '', '', $template_contact_groups, 0, $action, $debug);
	html_textbox("stalking_options", "stalking_options:", "general", 50, $stalking_options, '', 'format: [o,w,u,c]', $template_stalking_options, 0, $action, $debug);
	if($action eq "duplicate") { html_enabledbox("register", "register:", "required", $register, '', '', '', 0, $action, $debug); }
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------

sub GetTemplate() {
	my $sql = "select * from $TABLE_SERVICES where enabled=1 and register=0 and host_name=\"$use_template\" order by host_name";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);		
	while (my $ref = $sth->fetchrow_hashref())
	{
		$template_servicegroups                 = $ref->{"servicegroups"};
		$template_is_volatile                   = $ref->{"is_volatile"};
		$template_check_command                 = $ref->{"check_command"};
		$template_check_command_parameters      = $ref->{"check_command_parameters"};
		$template_max_check_attempts            = $ref->{"max_check_attempts"};
		$template_normal_check_interval         = $ref->{"normal_check_interval"};
		$template_retry_check_interval          = $ref->{"retry_check_interval"};
		$template_active_checks_enabled         = $ref->{"active_checks_enabled"};
		$template_passive_checks_enabled        = $ref->{"passive_checks_enabled"};
		$template_check_period                  = $ref->{"check_period"};
		$template_parallelize_check             = $ref->{"parallelize_check"};
		$template_obsess_over_service           = $ref->{"obsess_over_service"};
		$template_check_freshness               = $ref->{"check_freshness"};
		$template_freshness_threshold           = $ref->{"freshness_threshold"};
		$template_event_handler                 = $ref->{"event_handler"};
		$template_event_handler_enabled         = $ref->{"event_handler_enabled"};
		$template_low_flap_threshold            = $ref->{"low_flap_threshold"};
		$template_high_flap_threshold           = $ref->{"high_flap_threshold"};
		$template_flap_detection_enabled        = $ref->{"flap_detection_enabled"};
		$template_process_perf_data             = $ref->{"process_perf_data"};
		$template_retain_status_information     = $ref->{"retain_status_information"};
		$template_retain_nonstatus_information  = $ref->{"retain_nonstatus_information"};
		$template_notification_interval         = $ref->{"notification_interval"};
		$template_notification_period           = $ref->{"notification_period"};
		$template_notification_options          = $ref->{"notification_options"};
		$template_notifications_enabled         = $ref->{"notifications_enabled"};
		$template_contact_groups                = $ref->{"contact_groups"};
		$template_stalking_options              = $ref->{"stalking_options"};
	}		
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
}

# ---------------------------------------------------------------------------------