#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Statistics";
my $prgtext       = "Statistics";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.016";
my $description   = "This module displays a short overview of the available records in the NCPL database.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');

# ---------------------------------------------------------------------------------

sub Print_Summary_Nagios();
sub Print_Summary_NCPL();
sub getStats;

# ---------------------------------------------------------------------------------

my $cgi       = new CGI;

my $action    = (defined $cgi->param('action'))  ?  $cgi->param('action')  : '';
my $submit    = (defined $cgi->param('submit'))  ?  $cgi->param('submit')  : '';
my $debug     = (defined $cgi->param('debug'))   ?  $cgi->param('debug')   : 0;

# ---------------------------------------------------------------------------------

my $categories_enabled            = 0;
my $categories_disabled           = 0;
my $categories_total              = 0;
my $commands_enabled              = 0;
my $commands_disabled             = 0;
my $commands_total                = 0;
my $contacts_enabled              = 0;
my $contacts_disabled             = 0;
my $contacts_total                = 0;
my $contactgroups_enabled         = 0;
my $contactgroups_disabled        = 0;
my $contactgroups_total           = 0;
my $environments_enabled          = 0;
my $environments_disabled         = 0;
my $environments_total            = 0;
my $filetypes_enabled             = 0;
my $filetypes_disabled            = 0;
my $filetypes_total               = 0;
my $hosts_enabled     		        = 0;
my $hosts_disabled    		        = 0;
my $hosts_total       		        = 0;
my $hostdependencies_enabled      = 0;
my $hostdependencies_disabled     = 0;
my $hostdependencies_total        = 0;
my $hostescalations_enabled       = 0;
my $hostescalations_disabled      = 0;
my $hostescalations_total         = 0;
my $hostextinfo_enabled           = 0;
my $hostextinfo_disabled          = 0;
my $hostextinfo_total             = 0;
my $hostgroups_enabled            = 0;
my $hostgroups_disabled           = 0;
my $hostgroups_total              = 0;
my $hostgroupescalations_enabled  = 0;
my $hostgroupescalations_disabled = 0;
my $hostgroupescalations_total    = 0;
my $misccommands_enabled          = 0;
my $misccommands_disabled         = 0;
my $misccommands_total            = 0;
my $resources_enabled             = 0;
my $resources_disabled            = 0;
my $resources_total               = 0;
my $servers_enabled               = 0;
my $servers_disabled              = 0;
my $servers_total                 = 0;
my $services_enabled              = 0;
my $services_disabled             = 0;
my $services_total                = 0;
my $servicedependencies_enabled   = 0;
my $servicedependencies_disabled  = 0;
my $servicedependencies_total     = 0;
my $serviceescalations_enabled    = 0;
my $serviceescalations_disabled   = 0;
my $serviceescalations_total      = 0;
my $serviceextinfo_enabled        = 0;
my $serviceextinfo_disabled       = 0;
my $serviceextinfo_total          = 0;
my $servicegroups_enabled         = 0;
my $servicegroups_disabled        = 0;
my $servicegroups_total           = 0;
my $timeperiods_enabled           = 0;
my $timeperiods_disabled          = 0;
my $timeperiods_total             = 0;
my $userlevels_enabled						= 0;
my $userlevels_disabled						= 0;
my $userlevels_total							= 0;
my $users_enabled                 = 0;
my $users_disabled                = 0;
my $users_total                   = 0;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	
	($categories_enabled, $categories_disabled, $categories_total)                               = getStats($TABLE_CATEGORIES, $dbh, $debug);
	($commands_enabled, $commands_disabled, $commands_total)                                     = getStats($TABLE_COMMANDS, $dbh, $debug);
	($contacts_enabled, $contacts_disabled, $contacts_total)                                     = getStats($TABLE_CONTACTS, $dbh, $debug);
	($contactgroups_enabled, $contactgroups_disabled, $contactgroups_total)                      = getStats($TABLE_CONTACTGROUPS, $dbh, $debug);
	($environments_enabled, $environments_disabled, $environments_total)                         = getStats($TABLE_ENVIRONMENTS, $dbh, $debug);
	($filetypes_enabled, $filetypes_disabled, $filetypes_total)                                  = getStats($TABLE_FILETYPES, $dbh, $debug);
	($hosts_enabled, $hosts_disabled, $hosts_total)                                              = getStats($TABLE_HOSTS, $dbh, $debug);
	($hostdependencies_enabled, $hostdependencies_disabled, $hostdependencies_total)             = getStats($TABLE_HOSTDEPENDENCIES, $dbh, $debug);
	($hostescalations_enabled, $hostescalations_disabled, $hostescalations_total)                = getStats($TABLE_HOSTESCALATIONS, $dbh, $debug);
	($hostextinfo_enabled, $hostextinfo_disabled, $hostextinfo_total)                            = getStats($TABLE_HOSTEXTINFO, $dbh, $debug);
	($hostgroups_enabled, $hostgroups_disabled, $hostgroups_total)                               = getStats($TABLE_HOSTGROUPS, $dbh, $debug);
	($hostgroupescalations_enabled, $hostgroupescalations_disabled, $hostgroupescalations_total) = getStats($TABLE_HOSTGROUPESCALATIONS, $dbh, $debug);
	($misccommands_enabled, $misccommands_disabled, $misccommands_total)                         = getStats($TABLE_MISCCOMMANDS, $dbh, $debug);
	($resources_enabled, $resources_disabled, $resources_total)                                  = getStats($TABLE_RESOURCES, $dbh, $debug);
	($servers_enabled, $servers_disabled, $servers_total)                                        = getStats($TABLE_SERVERS, $dbh, $debug);
	($services_enabled, $services_disabled, $services_total)                                     = getStats($TABLE_SERVICES, $dbh, $debug);
	($servicedependencies_enabled, $servicedependencies_disabled, $servicedependencies_total)    = getStats($TABLE_SERVICEDEPENDENCIES, $dbh, $debug);
	($serviceescalations_enabled, $serviceescalations_disabled, $serviceescalations_total)       = getStats($TABLE_SERVICEESCALATIONS, $dbh, $debug);
	($serviceextinfo_enabled, $serviceextinfo_disabled, $serviceextinfo_total)                   = getStats($TABLE_SERVICEEXTINFO, $dbh, $debug);
	($servicegroups_enabled, $servicegroups_disabled, $servicegroups_total)                      = getStats($TABLE_SERVICEGROUPS, $dbh, $debug);
	($timeperiods_enabled, $timeperiods_disabled, $timeperiods_total)                            = getStats($TABLE_TIMEPERIODS, $dbh, $debug);
	($userlevels_enabled, $userlevels_disabled, $userlevels_total)                               = getStats($TABLE_USERLEVELS, $dbh, $debug);
	($users_enabled, $users_disabled, $users_total)                                              = getStats($TABLE_USERS, $dbh, $debug);

	Print_Summary_Nagios();
	Print_Summary_NCPL();
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub Print_Summary_Nagios() {

print <<END
<br>
<table width="350">
  <tr>
  	<th colspan="4">Nagios Stats</th>
  </tr>
  <tr>
		<th>Table</th>
		<th>Enabled</th>
		<th>Disabled</th>
		<th>Total</th>
  </tr>
  <tr>
		<td class=\"recordset\">Commands</td>
		<td class=\"recordset\" align="center">$commands_enabled</td>
		<td class=\"recordset\" align="center">$commands_disabled</td>
    <td class=\"recordset\" align="center">$commands_total</td>
  </tr>
  <tr>
		<td class=\"recordset\">Contactgroups</td>
		<td class=\"recordset\" align="center">$contactgroups_enabled</td>
		<td class=\"recordset\" align="center">$contactgroups_disabled</td>
    <td class=\"recordset\" align="center">$contactgroups_total</td>
  </tr>
  <tr>
		<td class=\"recordset\">Contacts</td>
		<td class=\"recordset\" align="center">$contacts_enabled</td>
		<td class=\"recordset\" align="center">$contacts_disabled</td>
    <td class=\"recordset\" align="center">$contacts_total</td>
  </tr>
  <tr>
		<td class=\"recordset\">Hosts</td>
		<td class=\"recordset\" align="center">$hosts_enabled</td>
		<td class=\"recordset\" align="center">$hosts_disabled</td>
    <td class=\"recordset\" align="center">$hosts_total</td>
  </tr> 
  <tr>
		<td class=\"recordset\">Hostdependencies</td>
		<td class=\"recordset\" align="center">$hostdependencies_enabled</td>
		<td class=\"recordset\" align="center">$hostdependencies_disabled</td>
    <td class=\"recordset\" align="center">$hostdependencies_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Hostescalations</td>
		<td class=\"recordset\" align="center">$hostescalations_enabled</td>
		<td class=\"recordset\" align="center">$hostescalations_disabled</td>
    <td class=\"recordset\" align="center">$hostescalations_total</td>
  </tr>   
  <tr>
		<td class=\"recordset\">Hostextinfo</td>
		<td class=\"recordset\" align="center">$hostextinfo_enabled</td>
		<td class=\"recordset\" align="center">$hostextinfo_disabled</td>
    <td class=\"recordset\" align="center">$hostextinfo_total</td>
  </tr>    
  <tr>
		<td class=\"recordset\">Hostgroups</td>
		<td class=\"recordset\" align="center">$hostgroups_enabled</td>
		<td class=\"recordset\" align="center">$hostgroups_disabled</td>
    <td class=\"recordset\" align="center">$hostgroups_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Hostgroupescalations</td>
		<td class=\"recordset\" align="center">$hostgroupescalations_enabled</td>
		<td class=\"recordset\" align="center">$hostgroupescalations_disabled</td>
    <td class=\"recordset\" align="center">$hostgroupescalations_total</td>
  </tr> 
  <tr>
		<td class=\"recordset\">Misccommands</td>
		<td class=\"recordset\" align="center">$misccommands_enabled</td>
		<td class=\"recordset\" align="center">$misccommands_disabled</td>
    <td class=\"recordset\" align="center">$misccommands_total</td>
  </tr> 
  <tr>
		<td class=\"recordset\">Resources</td>
		<td class=\"recordset\" align="center">$resources_enabled</td>
		<td class=\"recordset\" align="center">$resources_disabled</td>
    <td class=\"recordset\" align="center">$resources_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Services</td>
		<td class=\"recordset\" align="center">$services_enabled</td>
		<td class=\"recordset\" align="center">$services_disabled</td>
    <td class=\"recordset\" align="center">$services_total</td>
  </tr>
  <tr>
		<td class=\"recordset\">Servicedependencies</td>
		<td class=\"recordset\" align="center">$servicedependencies_enabled</td>
		<td class=\"recordset\" align="center">$servicedependencies_disabled</td>
    <td class=\"recordset\" align="center">$servicedependencies_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Serviceescalations</td>
		<td class=\"recordset\" align="center">$serviceescalations_enabled</td>
		<td class=\"recordset\" align="center">$serviceescalations_disabled</td>
    <td class=\"recordset\" align="center">$serviceescalations_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Serviceextinfo</td>
		<td class=\"recordset\" align="center">$serviceextinfo_enabled</td>
		<td class=\"recordset\" align="center">$serviceextinfo_disabled</td>
    <td class=\"recordset\" align="center">$serviceextinfo_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Servicegroups</td>
		<td class=\"recordset\" align="center">$servicegroups_enabled</td>
		<td class=\"recordset\" align="center">$servicegroups_disabled</td>
    <td class=\"recordset\" align="center">$servicegroups_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">Time Periods</td>
		<td class=\"recordset\" align="center">$timeperiods_enabled</td>
		<td class=\"recordset\" align="center">$timeperiods_disabled</td>
    <td class=\"recordset\" align="center">$timeperiods_total</td>
  </tr>  
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub Print_Summary_NCPL() {

print <<END
<br>
<table width="350">
  <tr>
  	<th colspan="4">NCPL Stats</th>
  </tr>
  <tr>
		<th>Table</th>
		<th>Enabled</th>
		<th>Disabled</th>
		<th>Total</th>
  </tr>  
  <tr>
		<td class=\"recordset\">NCPL - Categories</td>
		<td class=\"recordset\" align="center">$categories_enabled</td>
		<td class=\"recordset\" align="center">$categories_disabled</td>
    <td class=\"recordset\" align="center">$categories_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">NCPL - Environments</td>
		<td class=\"recordset\" align="center">$environments_enabled</td>
		<td class=\"recordset\" align="center">$environments_disabled</td>
    <td class=\"recordset\" align="center">$environments_total</td>
  </tr>   
  <tr>
		<td class=\"recordset\">NCPL - Filetypes</td>
		<td class=\"recordset\" align="center">$filetypes_enabled</td>
		<td class=\"recordset\" align="center">$filetypes_disabled</td>
    <td class=\"recordset\" align="center">$filetypes_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">NCPL - Servers</td>
		<td class=\"recordset\" align="center">$servers_enabled</td>
		<td class=\"recordset\" align="center">$servers_disabled</td>
    <td class=\"recordset\" align="center">$servers_total</td>
  </tr> 
  <tr>
		<td class=\"recordset\">NCPL - Userlevels</td>
		<td class=\"recordset\" align="center">$userlevels_enabled</td>
		<td class=\"recordset\" align="center">$userlevels_disabled</td>
    <td class=\"recordset\" align="center">$userlevels_total</td>
  </tr>  
  <tr>
		<td class=\"recordset\">NCPL - Users</td>
		<td class=\"recordset\" align="center">$users_enabled</td>
		<td class=\"recordset\" align="center">$users_disabled</td>
    <td class=\"recordset\" align="center">$users_total</td>
  </tr>
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub getStats {
  my ($TABLE, $dbh, $debug) = @_;
  
	my ($sth1, $enabled)  = doQueryDBI($dbh, "select * from $TABLE where enabled=1", '', '', $debug);
	my $rc1 = $sth1->finish() or doErrorTrapDBI('$sth1->finish()', "select * from $TABLE where enabled=1", $dbh->errstr, $debug);
	
	my ($sth2, $disabled) = doQueryDBI($dbh, "select * from $TABLE where enabled=0", '', '', $debug);
	my $rc2 = $sth2->finish() or doErrorTrapDBI('$sth2->finish()', "select * from $TABLE where enabled=0", $dbh->errstr, $debug);
	
	my $total = $enabled + $disabled;
	
	return	($enabled, $disabled, $total);
}

# ---------------------------------------------------------------------------------