#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :IMPORT :EXPORT :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Timeperiods";
my $prgtext       = "Timeperiods";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.022";
my $description   = "A time period is a list of times during various days that are considered to be \"valid\" times for notifications and service checks. It consists one or more time periods for each day of the week that \"rotate\" once the week has come to an end. Exceptions to the normal weekly time range rotations are not suported.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/export.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new timeperiod'  , 'Search timeperiod' , 'List all timeperiods' , 'Import old config'  , 'Export new config');
my @menu_actions  = ('add'                 , 'search'            , 'listall'              , 'doImportTimeperiods', 'doExportTimeperiods' );

# ---------------------------------------------------------------------------------

sub AddTimeperiod();
sub DeleteTimeperiod();
sub DetailTimeperiod();
sub DuplicateTimeperiod();
sub EditTimeperiod();
sub SearchTimeperiod();

# ---------------------------------------------------------------------------------

my $cgi             = new CGI;

my $server_name     = (defined $cgi->param('server_name')) 	   ?  $cgi->param('server_name')     : '';
my $category        = (defined $cgi->param('category')) 	     ?  $cgi->param('category')        : '';
my $timeperiod_name = (defined $cgi->param('timeperiod_name')) ?  $cgi->param('timeperiod_name') : '';
my $alias           = (defined $cgi->param('alias')) 		       ?  $cgi->param('alias') 		       : '';
my $sunday          = (defined $cgi->param('sunday')) 	       ?  $cgi->param('sunday') 	       : '';
my $monday          = (defined $cgi->param('monday')) 	       ?  $cgi->param('monday') 	       : '';
my $tuesday         = (defined $cgi->param('tuesday')) 	       ?  $cgi->param('tuesday') 	       : '';
my $wednesday       = (defined $cgi->param('wednesday'))       ?  $cgi->param('wednesday')       : '';
my $thursday        = (defined $cgi->param('thursday'))        ?  $cgi->param('thursday')        : '';
my $friday          = (defined $cgi->param('friday')) 	       ?  $cgi->param('friday') 	       : '';
my $saturday        = (defined $cgi->param('saturday'))        ?  $cgi->param('saturday')        : '';
my $enabled         = (defined $cgi->param('enabled')) 	       ?  $cgi->param('enabled')	       : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "timeperiod_name", "timeperiod_name=$timeperiod_name", 16);

# ---------------------------------------------------------------------------------

my $sql_list_all    = "select server_name, category, timeperiod_name, alias, enabled from $TABLE_TIMEPERIODS";
my $sql_categories  = "select category_name, category_alias from $TABLE_CATEGORIES where category_filetype=$filetype and enabled=1 order by category_alias";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')             { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')                 { AddTimeperiod();       }
	elsif($action eq 'search')              { SearchTimeperiod       }
	elsif($action eq 'doImportTimeperiods') { DisplayImportForm("import.pl", $action, $filetype, $submit, $dbh, $CGISESSID, $debug); }
	elsif($action eq 'doExportTimeperiods') { DisplayExportForm("export.pl", $server_name, $category, $filetype, undef, $action, $submit, $dbh, $CGISESSID, $debug) }
	elsif($action eq 'edit')                { EditTimeperiod();      }
	elsif($action eq 'duplicate')           { DuplicateTimeperiod(); }
	elsif($action eq 'delete')              { DeleteTimeperiod();    }
	elsif($action eq 'detail')              { DetailTimeperiod();    }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddTimeperiod() {

if ($submit) {
	my $sql = "insert into $TABLE_TIMEPERIODS values(\"$server_name\", \"$category\", \"$timeperiod_name\"," . $dbh->quote($alias) . ",\"$sunday\",\"$monday\",\"$tuesday\",\"$wednesday\",\"$thursday\",\"$friday\",\"$saturday\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Timeperiod added to the database!', 'Timeperiod not added!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="timeperiodsForm">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();
 
print <<END
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub SearchTimeperiod() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub EditTimeperiod() {
	
if ($submit) 
{
	my $sql = "update $TABLE_TIMEPERIODS set server_name=\"$server_name\", category=\"$category\", alias=" . $dbh->quote($alias) . ",sunday=\"$sunday\",monday=\"$monday\",tuesday=\"$tuesday\",wednesday=\"$wednesday\",thursday=\"$thursday\",friday=\"$friday\",saturday=\"$saturday\",enabled=$enabled where timeperiod_name=\"$timeperiod_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Timeperiod updated!', 'Timeperiod not updated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_TIMEPERIODS where timeperiod_name=\"$timeperiod_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $alias       = $ref->{"alias"};
	  $sunday      = $ref->{"sunday"};
	  $monday      = $ref->{"monday"};
	  $tuesday     = $ref->{"tuesday"};
	  $wednesday   = $ref->{"wednesday"};
	  $thursday    = $ref->{"thursday"};
	  $friday      = $ref->{"friday"};
	  $saturday    = $ref->{"saturday"};
		$enabled	   = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="timeperiodsForm">
<input type="hidden" name="action"          size="50" value="$action">
<input type="hidden" name="timeperiod_name" size="50" value="$timeperiod_name">
<input type="hidden" name="debug"           size="50" value="$debug">
<input type="hidden" name="CGISESSID"       size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DeleteTimeperiod() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_TIMEPERIODS where timeperiod_name=\"$timeperiod_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Timeperiod deleted!', 'Timeperiod not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}	

}

# ---------------------------------------------------------------------------------

sub DetailTimeperiod() {

	my $sql = "select * from $TABLE_TIMEPERIODS where timeperiod_name=\"$timeperiod_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
    $category    = $ref->{"category"};
	  $alias       = $ref->{"alias"};
	  $sunday      = $ref->{"sunday"};
	  $monday      = $ref->{"monday"};
	  $tuesday     = $ref->{"tuesday"};
	  $wednesday   = $ref->{"wednesday"};
	  $thursday    = $ref->{"thursday"};
	  $friday      = $ref->{"friday"};
	  $saturday    = $ref->{"saturday"};
		$enabled	   = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();
  
print <<END
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub DuplicateTimeperiod() {
	
if ($submit) 
{
	my $sql = "insert into $TABLE_TIMEPERIODS values(\"$server_name\", \"$category\", \"$timeperiod_name\"," . $dbh->quote($alias) . ",\"$sunday\",\"$monday\",\"$tuesday\",\"$wednesday\",\"$thursday\",\"$friday\",\"$saturday\",\"$enabled\")";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'Timeperiod duplicated!', 'Timeperiod not duplicated!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_TIMEPERIODS where timeperiod_name=\"$timeperiod_name\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$server_name = $ref->{"server_name"};
		$category    = $ref->{"category"};
	  $alias       = $ref->{"alias"};
	  $sunday      = $ref->{"sunday"};
	  $monday      = $ref->{"monday"};
	  $tuesday     = $ref->{"tuesday"};
	  $wednesday   = $ref->{"wednesday"};
	  $thursday    = $ref->{"thursday"};
	  $friday      = $ref->{"friday"};
	  $saturday    = $ref->{"saturday"};
		$enabled	   = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<form method="POST" action="$prgscriptname" name="timeperiodsForm">
<input type="hidden" name="action"          size="50" value="$action">
<input type="hidden" name="debug"           size="50" value="$debug">
<input type="hidden" name="CGISESSID"       size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END  
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_dbcombobox("server_name", "server_name:", "required", $server_name, $dbh, $SQL_SERVERS, '', '', '', 0, $action, $debug);
	html_dbcombobox("category", "category:", "general", $category, $dbh, $sql_categories, '', '', '', 0, $action, $debug);
	html_emptyline(2, $action, $debug);
	html_textbox("timeperiod_name", "timeperiod_name:", "required", 50, $timeperiod_name, '', '', '', 1, $action, $debug);	
	html_textbox("alias", "alias:", "required", 50, $alias, '', '', '', 0, $action, $debug);	
	html_textbox("sunday", "sunday:", "general", 50, $sunday, '', '', '', 0, $action, $debug);	
	html_textbox("monday", "monday:", "general", 50, $monday, '', '', '', 0, $action, $debug);	
	html_textbox("tuesday", "tuesday:", "general", 50, $tuesday, '', '', '', 0, $action, $debug);	
	html_textbox("wednesday", "wednesday:", "general", 50, $wednesday, '', '', '', 0, $action, $debug);	
	html_textbox("thursday", "thursday:", "general", 50, $thursday, '', '', '', 0, $action, $debug);	
	html_textbox("friday", "friday:", "general", 50, $friday, '', '', '', 0, $action, $debug);	
	html_textbox("saturday", "saturday:", "general", 50, $saturday, '', '', '', 0, $action, $debug);	
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------
