#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :FUNCTIONS :NAGIOSBYDB);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: Tools Script";
my $prgtext       = "NCPL Tools Script";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.000";
my $description   = "This module provides several useful sync scripts.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js');
my @html_css      = ('../stylesheets/ncpl.css');

# ---------------------------------------------------------------------------------

my $cgi       = new CGI;
my $action    = (defined $cgi->param('action'))     ?  $cgi->param('action')     : 'showMenu';
my $submit    = (defined $cgi->param('submitForm')) ?  $cgi->param('submitForm') : 0;
my $debug     = (defined $cgi->param('debug'))      ?  $cgi->param('debug')      : 0;

# ---------------------------------------------------------------------------------

sub syncServiceCategoriesWithHostCategories;
sub syncHostextinfoCategoriesWithHostCategories;
sub syncNagiosByDb;
sub createNonExistingCategories;
sub nslookupHosts;

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);

	if($action eq 'syncHostextinfoCategoriesWithHostCategories') { syncHostextinfoCategoriesWithHostCategories($dbh, $debug); }
	elsif($action eq 'syncServiceCategoriesWithHostCategories')  { syncServiceCategoriesWithHostCategories($dbh, $debug); }
	elsif($action eq 'syncNagiosByDb')                           { syncNagiosByDb($dbh, $debug);   }
	elsif($action eq 'nslookupHosts')                            { nslookupHosts($dbh, 5, $debug); }
	else { showMenu();  }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();
	
exit(0);

# ---------------------------------------------------------------------------------

sub showMenu {

print <<END
<br>
<ul>
	<li><a href="tools.pl?action=nslookupHosts&amp;debug=$debug&amp;CGISESSID=$CGISESSID">NSLOOKUP for hosts</a></li>
	<li><a href="tools.pl?action=syncNagiosByDb&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize NCPL database with nagios-by-db database</a></li>
	<li><a href="tools.pl?action=syncHostextinfoCategoriesWithHostCategories&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize hostextinfo categories with host categories</a></li>
	<li><a href="tools.pl?action=syncServiceCategoriesWithHostCategories&amp;debug=$debug&amp;CGISESSID=$CGISESSID">Synchronize service categories with host categories</a></li>
</ul>
END
;

}

# ---------------------------------------------------------------------------------

sub syncNagiosByDb {
	
	my ($dbh, $debug) = @_;
	
	my $nagiosbydb = DBI->connect("dbi:mysql:$NAGIOSBYDB_CATALOG:$NAGIOSBYDB_DBSERVER:$NAGIOSBYDB_PORT", "$NAGIOSBYDB_USERNAME", "$NAGIOSBYDB_PASSWORD" ) or die("Cannot connect to the database");	
	
	if($dbh and $nagiosbydb) 
	{
		# deleting old records
		print "<p class=\"export\">Deleting old services</p>\n";
		my $sql_delete = "delete from $TABLE_SERVICES where category=\"$NAGIOSBYDB_CATEGORY\"";
		my ($sth_delete,$rows_delete) = doQueryDBI($dbh, $sql_delete, '', '', $debug);		
		my $rc_delete = $sth_delete->finish() or doErrorTrapDBI('$sth_delete->finish()', $sql_delete, $dbh->errstr, $debug);
		print "<p><font color=\"#00FF00\">OK</font> -- Old config cleanup finished!</p>\n";
		
		# inserting new records
		print "<p class=\"export\">Adding new services</p>\n";
		my $sql_select = "SELECT host_name, service_description FROM servicestatus order by host_name";
		my ($sth_select,$rows_select) = doQueryDBI($nagiosbydb, $sql_select, '', '', $debug);
		while (my $ref = $sth_select->fetchrow_hashref()) 
		{
				my $server_name									 = $NAGIOSBYDB_SERVERNAME;
				my $category                     = $NAGIOSBYDB_CATEGORY;
  		  my $host_name                    = $ref->{"host_name"};
			  my $service_description          = $ref->{"service_description"};
			  my $use                          = 'check_nagios-by-db';
			  my $contact_groups               = 'supervision';
			  my $check_command                = 'check_nagios-by-db-service';
			  my $check_command_parameters     = '"' . $ref->{"service_description"} . '"';
			  my $servicegroups                = '';
			  my $is_volatile                  = '0';
			  my $max_check_attempts           = '0';
			  my $normal_check_interval        = '0';
			  my $retry_check_interval         = '0';
			  my $active_checks_enabled        = '0';
			  my $passive_checks_enabled       = '0';
			  my $check_period                 = '';
			  my $parallelize_check            = '0';
			  my $obsess_over_service          = '0';
			  my $check_freshness              = '0';
			  my $freshness_threshold          = '0';
			  my $event_handler                = '';
			  my $event_handler_enabled        = '0';
			  my $low_flap_threshold           = '0';
			  my $high_flap_threshold          = '0';
			  my $flap_detection_enabled       = '0';
			  my $process_perf_data            = '0';
			  my $retain_status_information    = '0';
			  my $retain_nonstatus_information = '0';
			  my $notification_interval        = '0';
			  my $notification_period          = '';
			  my $notification_options         = '';
			  my $notifications_enabled        = '0';
			  my $stalking_options             = '';
			  my $register                     = '1';
			  print "* $host_name -- $service_description<br>\n";
				my $sqlSet = 'server_name="' .$server_name. '", category="' .$category. '", host_name="' .$host_name. '", service_description=' .$dbh->quote($service_description). ', `use`="' .$use. '", servicegroups="' .$servicegroups. '", is_volatile="' .$is_volatile. '", check_command=' .$dbh->quote($check_command). ', check_command_parameters=' .$dbh->quote($check_command_parameters). ', max_check_attempts="' .$max_check_attempts. '", normal_check_interval="' .$normal_check_interval. '", retry_check_interval="' .$retry_check_interval. '", active_checks_enabled="' .$active_checks_enabled. '", passive_checks_enabled="' .$passive_checks_enabled. '", check_period="' .$check_period. '", parallelize_check="' .$parallelize_check. '", obsess_over_service="' .$obsess_over_service. '", check_freshness="' .$check_freshness. '", freshness_threshold="' .$freshness_threshold. '", event_handler="' .$event_handler. '", event_handler_enabled="' .$event_handler_enabled. '", low_flap_threshold="' .$low_flap_threshold. '", high_flap_threshold="' .$high_flap_threshold. '", flap_detection_enabled="' .$flap_detection_enabled. '", process_perf_data="' .$process_perf_data. '", retain_status_information="' .$retain_status_information. '", retain_nonstatus_information="' .$retain_nonstatus_information. '", notification_interval="' .$notification_interval. '", notification_period="' .$notification_period. '", notification_options="' .$notification_options. '", notifications_enabled="' .$notifications_enabled. '", contact_groups="' .$contact_groups. '", stalking_options="' .$stalking_options. '", register="' .$register. '", enabled="1"';
				my $sql_insert = "INSERT INTO $TABLE_SERVICES SET $sqlSet";
				my ($sth_insert,$rows_insert) = doQueryDBI($dbh, $sql_insert, '', '', $debug);
				my $rc = $sth_insert->finish() or doErrorTrapDBI('$sth_insert->finish()', $sql_insert, $dbh->errstr, $debug);
		}
		my $rc_select = $sth_select->finish() or doErrorTrapDBI('$sth_select->finish()', $sql_select, $nagiosbydb->errstr, $debug);
		
	  # Consistency check 1
		print "<p class=\"export\">Consistency check</p>\n";
		my $sql_consistency_1 = "select host_name from $TABLE_HOSTS where category=\"$NAGIOSBYDB_CATEGORY\" and host_name not in (select host_name from $TABLE_SERVICES where category=\"$NAGIOSBYDB_CATEGORY\")";
		my ($sth_consistency_1,$rows_consistency_1) = doQueryDBI($dbh, $sql_consistency_1, '', '', $debug);
		my $aantal = 0;
		while (my $ref = $sth_consistency_1->fetchrow_hashref()) 
		{
			print "* " . $ref->{"host_name"} . " doesn't exist anymore, <font color=\"#FF0000\">please delete this host manually!</font><br>\n";
			$aantal += 1;
		}
		my $rc_consistency_1 = $sth_consistency_1->finish() or doErrorTrapDBI('$sth_consistency_1->finish()', $sql_consistency_1, $dbh->errstr, $debug);
		
		print "<br>\n" if ($aantal > 0);
		
		# Consistency check 2
		my $sql_consistency_2 = "select host_name from $TABLE_SERVICES where category=\"$NAGIOSBYDB_CATEGORY\" and host_name not in (select host_name from $TABLE_HOSTS where category=\"$NAGIOSBYDB_CATEGORY\")";
		my ($sth_consistency_2,$rows_consistency_2) = doQueryDBI($dbh, $sql_consistency_2, '', '', $debug);
		while (my $ref = $sth_consistency_2->fetchrow_hashref()) 
		{
			print "* " . $ref->{"host_name"} . " does not yet exist, <font color=\"#FF0000\">please create this host manually!</font><br>\n";
		}
		my $rc_consistency_2 = $sth_consistency_2->finish() or doErrorTrapDBI('$sth_consistency_2->finish()', $sql_consistency_2, $dbh->errstr, $debug);
	}
	print "<p><font color=\"#00FF00\">OK</font> -- NagiosByDB Services synchronized!</p>\n";

}

# ---------------------------------------------------------------------------------

sub syncServiceCategoriesWithHostCategories {

	my ($dbh, $debug) = @_;

	createNonExistingCategories($dbh, 11, $debug);
	
	print "<p class=\"export\">Synchronizing services for:</p>\n";
	my $sql = "select host_name,category from $TABLE_HOSTS where category <> \"\" order by host_name";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		my $host_name = $ref->{"host_name"};
		my $category  = $ref->{"category"};
		print "* $host_name<br>\n";
		my $sql_update = "update $TABLE_SERVICES set category=\"$category\" where host_name=\"$host_name\" and category<>\"$category\"";
		my ($sth_update,$rows_update) = doQueryDBI($dbh, $sql_update, '', '', $debug);
		my $rc_update = $sth_update->finish() or doErrorTrapDBI('$sth_update->finish()', $sql_update, $dbh->errstr, $debug);
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	print "<p><font color=\"#00FF00\">OK</font> -- Service categories synchronized with host categories!</p>\n";

}

# ---------------------------------------------------------------------------------

sub syncHostextinfoCategoriesWithHostCategories {

	my ($dbh, $debug) = @_;

	createNonExistingCategories($dbh, 9, $debug);
	
	print "<p class=\"export\">Synchronizing hostextinfo for:</p>\n";
	my $sql = "select host_name,category from $TABLE_HOSTS where category <> \"\" order by host_name";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		my $host_name = $ref->{"host_name"};
		my $category  = $ref->{"category"};
		print "* $host_name<br>\n";
		my $sql_update = "update $TABLE_HOSTEXTINFO set category=\"$category\" where host_name=\"$host_name\" and category<>\"$category\"";
		my ($sth_update,$rows_update) = doQueryDBI($dbh, $sql_update, '', '', $debug);
		my $rc_update = $sth_update->finish() or doErrorTrapDBI('$sth_update->finish()', $sql_update, $dbh->errstr, $debug);
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	print "<p><font color=\"#00FF00\">OK</font> -- Hostextinfo categories synchronized with host categories!</p>\n";

}

# ---------------------------------------------------------------------------------

sub createNonExistingCategories {
	
	my ($dbh, $filetype, $debug) = @_;
	
	print "<p class=\"export\">Creating non-existing categories:</p>\n";
	my $sql = "select * from $TABLE_CATEGORIES where category_filetype=6 and category_name not in (select category_name  from $TABLE_CATEGORIES where category_filetype=$filetype)";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		my $category_name  = $ref->{"category_name"};
		my $category_alias = $ref->{"category_alias"};
		my $enabled        = $ref->{"enabled"};
		my $sql_insert     = "INSERT INTO $TABLE_CATEGORIES values($filetype, \"$category_name\", \"$category_alias\", \"$enabled\")";
		my ($sth_insert,$rows_insert) = doQueryDBI($dbh, $sql_insert, '', '', $debug);
		my $rc_insert = $sth_insert->finish() or doErrorTrapDBI('$sth_insert->finish()', $sql_insert, $dbh->errstr, $debug);
		print "* $category_name<br>\n";
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	print "<p><font color=\"#00FF00\">OK</font> -- Non-existing categories created!</p>\n";
	
}

# ---------------------------------------------------------------------------------

sub nslookupHosts {
	
	my ($dbh, $timeout, $debug) = @_;
	
	use Net::Nslookup;	
	$Net::Nslookup::TIMEOUT = $timeout;
	
	print "<p class=\"export\">Doing NSLOOKUP for hosts:</p>\n";
	my $sql = "select * from $TABLE_HOSTS where category <> \"$NAGIOSBYDB_CATEGORY\" and register=1 and enabled=1";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		my $server_name = $ref->{"server_name"};
		my $host_name   = $ref->{"host_name"};
		my $address     = $ref->{"address"};
		my $a = nslookup(host => $host_name, type => "A");
		if (defined $a && $a ne '') { 
			if ($address eq $a) { print "* $server_name: $host_name -- $a<br>\n"; }
			else { print "* $server_name: $host_name -- <font color=\"#FF0000\">$address ==> $a</font><br>\n"; }
			my $sql_update = "update $TABLE_HOSTS set address=\"$a\" where server_name=\"$server_name\" and host_name=\"$host_name\" and address<>\"$a\"";
			my ($sth_update,$rows_update) = doQueryDBI($dbh, $sql_update, '', '', $debug);
			my $rc_update = $sth_update->finish() or doErrorTrapDBI('$sth_update->finish()', $sql_update, $dbh->errstr, $debug);
		}
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	print "<p><font color=\"#00FF00\">OK</font> -- NSLOOKUP finished!</p>\n";
	
}

# ---------------------------------------------------------------------------------