#!/usr/bin/perl -w
# ---------------------------------------------------------------------------------
#  
# NCPL is the legal property of its developers, whose names are too numerous
# to list here.  Please refer to the AUTHORS file distributed with this
# source distribution.
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# ---------------------------------------------------------------------------------

use strict;
use warnings;
use NCPL v1.000.043;
use NCPL qw(:DEFAULT :HTML :DB :SECURITY :SQL);

use CGI;
use CGI::Session qw/-ip-match/;
use DBI;

# ---------------------------------------------------------------------------------

my $prgname       = "NCPL :: User Management";
my $prgtext       = "User Management";
my $prgscriptname = $ENV{"SCRIPT_NAME"};
my $version 	    = "1.000.014";
my $description   = "The User Management module provides the System Administrator with the ability to control which users have access to which features.";

# ---------------------------------------------------------------------------------

my @html_js       = ('../js/logon.js' , '../js/md5.js' , '../js/users.js');
my @html_css      = ('../stylesheets/ncpl.css');
my @menu_titles   = ('Add new user'  , 'Search user' , 'List all users');
my @menu_actions  = ('add'           , 'search'      , 'listall');

# ---------------------------------------------------------------------------------

sub AddUser();
sub DeleteUser();
sub DetailUser();
sub DuplicateUser();
sub EditUser();
sub SearchUser();

# ---------------------------------------------------------------------------------

my $cgi              = new CGI;

my $user_username    = (defined $cgi->param('user_username'))    ?  $cgi->param('user_username') 	  : '';
my $user_alias       = (defined $cgi->param('user_alias')) 		   ?  $cgi->param('user_alias') 		  : '';
my $user_email       = (defined $cgi->param('user_email')) 		   ?  $cgi->param('user_email') 		  : '';
my $user_password_1  = (defined $cgi->param('user_password_1'))  ?  $cgi->param('user_password_1')  : '';
my $user_password_2  = (defined $cgi->param('user_password_2'))  ?  $cgi->param('user_password_2')  : '';
my $user_password_db = (defined $cgi->param('user_password_db')) ?  $cgi->param('user_password_db') : '';
my $user_userlevel   = (defined $cgi->param('user_userlevel'))   ?  $cgi->param('user_userlevel')   : 0;
my $enabled          = (defined $cgi->param('enabled')) 	       ?  $cgi->param('enabled')          : 0;

my ($action, $ukeys_current, $submit, $confirmed, $sortkeys, $sortfield, $sortorder, $filterkey, $filter, $page, $filetype, $debug) = init_module($cgi, "user_username", "user_username=$user_username", undef);

# ---------------------------------------------------------------------------------

my $sql_list_all     = "select $TABLE_USERS.user_username, $TABLE_USERS.user_alias, $TABLE_USERLEVELS.userlevel_alias, $TABLE_USERS.enabled from $TABLE_USERS, $TABLE_USERLEVELS where $TABLE_USERS.user_userlevel=$TABLE_USERLEVELS.userlevel_id";

# ---------------------------------------------------------------------------------

my $dbh = DBI->connect("dbi:mysql:$DB_CATALOG:$DB_SERVER:$DB_PORT", "$DB_USERNAME", "$DB_PASSWORD" ) or die("Cannot connect to the database");	

my ($session, $CGISESSID) = UserAccessControl($prgscriptname, $cgi, $APPLICATION_NAME, \@html_js, \@html_css, $action, $submit, $dbh, $debug);

if (defined $session->param("~user_logged_in") && $session->param("~user_logged_in") == 1)
{
	html_start($prgname, \@html_js, \@html_css, undef, $debug);
	html_pagetitle($prgtext, $CGISESSID, $debug);
	html_menu($prgscriptname, \@menu_titles, \@menu_actions, $CGISESSID, $debug);
	
	if($action    eq 'listall')   { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	elsif($action eq 'add')       { AddUser();       }
	elsif($action eq 'search')    { SearchUser();    }
	elsif($action eq 'edit')      { EditUser();      }
	elsif($action eq 'duplicate') { DuplicateUser(); }
	elsif($action eq 'delete')    { DeleteUser();    }
	elsif($action eq 'detail')    { DetailUser();    }
	else { ListAllRecords($prgscriptname, $dbh, $sql_list_all, $filter, $ukeys_current, $page, $sortfield, $sortorder, $sortkeys, $filterkey, $CGISESSID, $debug); }
	
	html_description($description, $debug);
	html_debug($debug);
	html_stop(1, 1, "right", $debug);
}

$dbh->disconnect();

exit(0);

# ---------------------------------------------------------------------------------

sub AddUser() {

if ($submit) {
	if($user_password_1 eq $user_password_2) {
		my $sql = "insert into $TABLE_USERS values(\"$user_username\",\"$user_password_1\",\"$user_alias\",\"$user_email\",\"$user_userlevel\",\"$enabled\")";
		my ($sth,$rows) = doQueryDBI($dbh, $sql, 'User added to the database!', 'User not added!', $debug);
		my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	} else {
		print "<p>Insert cancelled, please retype password!</p>\n";
	}
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else {
print <<END
<br>
<form method="POST" action="$prgscriptname" name="usersForm" onsubmit="return validateUsersForm();">
<input type="hidden" name="action"    size="50" value="$action">
<input type="hidden" name="debug"     size="50" value="$debug">
<input type="hidden" name="CGISESSID" size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Add" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;

}

}

# ---------------------------------------------------------------------------------

sub SearchUser() {

	html_underconstruction($debug);

}

# ---------------------------------------------------------------------------------

sub DeleteUser() {

if($confirmed) 
{
	my $sql = "delete from $TABLE_USERS where user_username=\"$user_username\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, 'User deleted!', 'User not deleted!', $debug);
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{
	ConfirmDelete($prgscriptname,$ukeys_current,$CGISESSID,$debug);
}

}

# ---------------------------------------------------------------------------------

sub DetailUser() {

my $sql = "select * from $TABLE_USERS where user_username=\"$user_username\"";
my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
while (my $ref = $sth->fetchrow_hashref())
{
	$user_alias       = $ref->{"user_alias"};
	$user_email       = $ref->{"user_email"};
	$user_password_1  = $ref->{"user_password"};
	$user_password_2  = $ref->{"user_password"};
	$user_userlevel   = $ref->{"user_userlevel"};
	$enabled          = $ref->{"enabled"};
}
my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);

print <<END
<br>
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
END
;

}

# ---------------------------------------------------------------------------------

sub EditUser() {

if ($submit) 
{
	if($user_password_1 eq $user_password_2) {	
		my $sql = '';
		if(($user_password_db ne $user_password_1) && ($user_password_db ne $user_password_2) && ($user_password_1 ne '3fc26dc90ab296e41bc5fc8f9a2a7d40') && ($user_password_2 ne '3fc26dc90ab296e41bc5fc8f9a2a7d40')) {
			$sql = "update $TABLE_USERS set user_username=\"$user_username\", user_password=\"$user_password_1\", user_alias=\"$user_alias\", user_email=\"$user_email\", user_userlevel=$user_userlevel, enabled=$enabled where user_username=\"$user_username\"";
		} else {
			$sql = "update $TABLE_USERS set user_username=\"$user_username\", user_password=\"$user_password_db\", user_alias=\"$user_alias\", user_email=\"$user_email\", user_userlevel=$user_userlevel, enabled=$enabled where user_username=\"$user_username\"";
		}
		my ($sth,$rows) = doQueryDBI($dbh, $sql, 'User updated!', 'User not updated!', $debug);
		my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	} else {
		print "<p>Update cancelled, please retype password!</p>\n";
	}
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_USERS where user_username=\"$user_username\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
		$user_alias       = $ref->{"user_alias"};
		$user_email       = $ref->{"user_email"};
		$user_password_1  = "**********";
		$user_password_2  = "**********";		
		$user_password_db = $ref->{"user_password"};
		$user_userlevel   = $ref->{"user_userlevel"};
		$enabled          = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="usersForm" onsubmit="return validateUsersForm();">
<input type="hidden" name="action"           size="50" value="$action">
<input type="hidden" name="user_username"    size="50" value="$user_username">
<input type="hidden" name="user_password_db" size="50" value="$user_password_db">
<input type="hidden" name="debug"            size="50" value="$debug">
<input type="hidden" name="CGISESSID"        size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Save" name="submitForm">&nbsp;<input type="reset" value="Undo" name="undoForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DuplicateUser() {

if ($submit) 
{
	if($user_password_1 eq $user_password_2) {	
		my $sql = "insert into $TABLE_USERS values(\"$user_username\",\"$user_password_1\",\"$user_alias\",\"$user_email\",\"$user_userlevel\",\"$enabled\")";
		my ($sth,$rows) = doQueryDBI($dbh, $sql, 'User updated!', 'User not updated!', $debug);
		my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	} else {
		print "<p>Duplicate cancelled, please retype password!</p>\n";
	}
	html_goBackTo($prgscriptname, $action, $filter, $page, $sortfield, $sortorder, $CGISESSID, $debug);
}
else 
{	
	my $sql = "select * from $TABLE_USERS where user_username=\"$user_username\"";
	my ($sth,$rows) = doQueryDBI($dbh, $sql, '', '', $debug);
	while (my $ref = $sth->fetchrow_hashref())
	{
  	$user_alias       = $ref->{"user_alias"};
		$user_email       = $ref->{"user_email"};
		$user_password_1  = $ref->{"user_password"};
		$user_password_2  = $ref->{"user_password"};
		$user_userlevel   = $ref->{"user_userlevel"};
		$enabled          = $ref->{"enabled"};
	}
	my $rc = $sth->finish() or doErrorTrapDBI('$sth->finish()', $sql, $dbh->errstr, $debug);
	
print <<END
<br>
<form method="POST" action="$prgscriptname" name="usersForm" onsubmit="return validateUsersForm();">
<input type="hidden" name="action"           size="50" value="$action">
<input type="hidden" name="debug"            size="50" value="$debug">
<input type="hidden" name="CGISESSID"        size="50" value="$CGISESSID">
<table border="0">
END
;

DisplayForm();

print <<END   
</table>
<p><input type="submit" value="Duplicate" name="submitForm">&nbsp;<input type="reset" value="Clear" name="clearForm"></p>
</form>
END
;
}

}

# ---------------------------------------------------------------------------------

sub DisplayForm() {

	html_textbox("user_username", "user_username:", "required", 50, $user_username, '', '', '', 1, $action, $debug);	
	html_textbox("user_alias", "user_alias:", "required", 50, $user_alias, '', '', '', 0, $action, $debug);	
	html_textbox("user_email", "user_email:", "required", 50, $user_email, '', '', '', 0, $action, $debug);	
	html_passwordbox("user_password_1", "user_password:", "required", 50, $user_password_1, '', '', '', 0, $action, $debug);	
	html_passwordbox("user_password_2", "re-type:", "required", 50, $user_password_2, '', '', '', 0, $action, $debug);	
	html_dbcombobox("user_userlevel", "user_userlevel:", "required", $user_userlevel, $dbh, $SQL_USERLEVELS, '', '', '', 0, $action, $debug);
	html_enabledbox("enabled", "enabled:", "required", $enabled, '', '', '', 0, $action, $debug);				

}

# ---------------------------------------------------------------------------------